import React, { useEffect, useState, useCallback } from 'react';
import { useDispatch, useSelector } from 'react-redux';
import { createStructuredSelector } from 'reselect';
import { useInjectReducer, useInjectSaga } from 'redux-injectors';
import makeSelectAddPromotion from './selectors';
import reducer from './reducer';
import saga from './saga';
import {
  Button,
  Form,
  Radio,
  Select,
  Upload,
  notification,
  Row,
  Col,
} from 'antd';
import { LoadingOutlined } from '@ant-design/icons';
import { useHistory } from 'react-router-dom';
import { PromotionIcon } from '../../styles/icon-components';
import { DATE_FORMAT } from '../../utils/constants';
import {
  validatePercent,
  fileValidation,
} from '../../utils/validation';
import { digits, timeDigits } from 'utils/number';
import { validateTimeFormat } from 'utils/validation';
import { PromotionAddNewWrapper } from './styles';
import { addPromotion, getPromotionSubjects } from './actions';
import { injectIntl } from 'react-intl';
import messages from './messages';
import messagesGlobal from 'translations/messages';
import { makeSelectMenus, makeSelectRestaurants } from '../App/selectors';
import { getMenus} from '../App/actions';
import { FormatInput, FormatTimeInput } from 'components/FormatInput';
import cacheUtils from 'utils/cacheUtils';
import FloatLabel from 'components/FormatInput/FloatLabel';
import { DEFAULT_CURRENCY } from 'utils/constants';
import _ from 'lodash';

const selector = createStructuredSelector({
  state: makeSelectAddPromotion(),
});

const menuSelector = createStructuredSelector({
  restaurants: makeSelectRestaurants(),
  menus: makeSelectMenus(),
});

// eslint-disable-next-line @typescript-eslint/no-unused-vars
function AddPromotion(props) {
  useInjectReducer({ key: 'addPromotion', reducer: reducer });
  useInjectSaga({ key: 'addPromotion', saga: saga });

  // eslint-disable-next-line @typescript-eslint/no-unused-vars
  const dispatch = useDispatch(); // eslint-disable-line @typescript-eslint/no-unused-vars
  const { intl } = props;
  const {
    state: { isFetching, subjects },
  } = useSelector(selector);
  const listDate = [
    {id: 1, name: intl.formatMessage(messages.monday)},
    {id: 2, name: intl.formatMessage(messages.tuesday)},
    {id: 3, name: intl.formatMessage(messages.wednesday)},
    {id: 4, name: intl.formatMessage(messages.thursday)},
    {id: 5, name: intl.formatMessage(messages.friday)},
    {id: 6, name: intl.formatMessage(messages.sevenday)},
    {id: 0, name: intl.formatMessage(messages.sunday)},
  ]
  const { menus, restaurants } = useSelector(menuSelector);
  const [imageUrl, setImageUrl] = useState<any>();
  const [form] = Form.useForm();
  const history = useHistory();
  const format = 'HH:mm:ss';
  const [discountBy, setDiscountBy] = useState(1);
  const [promotionType, setPromotionType] = useState(1);


  const userInfo = cacheUtils.getUserInfo();
  const currency = useCallback(
    userInfo => {
      if (userInfo) {
        const userInfoJson = JSON.parse(userInfo);
        if (userInfoJson.profile?.merchant?.setting) {
          return userInfoJson.profile?.merchant?.setting.currency;
        }
        return DEFAULT_CURRENCY;
      } else {
        return DEFAULT_CURRENCY;
      }
    },
    [userInfo],
  );
  const currencyMemoi = currency(userInfo);
  useEffect(() => {
    const merchantId = cacheUtils.getMerchantId();
    dispatch(getPromotionSubjects());
    dispatch(getMenus(merchantId));
  }, []);

  const beforeUpload = file => {
    if (!fileValidation(file.name)) {
      notification.error({
        message: intl.formatMessage(messagesGlobal.invalidFile),
      });
      return false;
    }
    const reader = new FileReader();
    reader.onloadend = e => {
      setImageUrl(reader.result!);
    };
    reader.readAsDataURL(file);
    return false;
  };

  const handleChangeDiscountBy = value => {
    form.setFieldsValue({ discountValue: null });
    setDiscountBy(value);
  };

  const onFinish = addPromotionValues => {
    const values = _.cloneDeep(addPromotionValues);
    const { photo } = values;
    if (photo && photo.file) {
      values.photo = photo.file;
    }
    values.startDate = values.startDate
      ? values.startDate!.toISOString()
      : values.startDate;
    values.endDate = values.endDate
      ? values.endDate!.toISOString()
      : values.endDate;
    values.daysOfWeek = values.daysOfWeek
      ? `[${values.daysOfWeek}]`
      : values.daysOfWeek;
    for (var key in values) {
      if (typeof values[key] === 'string') {
        if (values[key] === '') {
          delete values[key];
          continue;
        }
        values[key] = values[key].trim();
        if (/^[0-9,]+$/.test(values[key])) {
          values[key] = values[key].replace(/,/g, '');
        }
      }
    }
    values.minimumTotalApplied =
      values.minimumTotalApplied !== '' ? values.minimumTotalApplied : null;
    const { menuItemIds = [], restaurantIds = [] } = values;
    values.menuItems = JSON.stringify(
      menuItemIds.map(menuItemId => ({ menuItemId })),
    );
    if (restaurantIds === '') {
      values.restaurantIds = [];
    }
    values.restaurantIds = JSON.stringify(
      restaurantIds.map(restaurantId => restaurantId),
    );
    delete values.menuItemIds; // fix api issue
    dispatch(addPromotion(values));
  };

  const uploadButton = (
    <div>
      {isFetching ? <LoadingOutlined /> : <PromotionIcon />}
      <div className="ant-upload-text">
        {intl.formatMessage(messages.image)}
      </div>
      <span className="img-hint">
        {intl.formatMessage(messagesGlobal.encourageCondition, {
          height: 345,
          width: 140,
        })}
      </span>
    </div>
  );

  const handleChangeType = e => {
    setPromotionType(e.target.value);
    form.setFieldsValue({ discountTypeId: 2 });
    setDiscountBy(2);
  };

  return (
    <PromotionAddNewWrapper>
      <Form
        form={form}
        className="form"
        onFinish={onFinish}
      >
        {promotionType === 1 ? (
          <>
            <Row className="first-promotion-row">
              <Col
                className="general-card"
                xl={16}
                lg={16}
                md={16}
                sm={24}
                xs={24}
              >
                <div className="form-header">
                  {intl.formatMessage(messagesGlobal.general)}
                </div>
                <Form.Item
                  name="status"
                  initialValue={1}
                  style={{ display: 'none' }}
                ></Form.Item>
                <div className="form-content-exchange__promo-type">
                  <p>{intl.formatMessage(messages.promotionType)}</p>
                  <Form.Item name="discountRedeemTypeId" initialValue={1}>
                    <Radio.Group onChange={handleChangeType}>
                      <Radio value={1}>
                        {intl.formatMessage(messages.free)}
                      </Radio>
                      <Radio value={2}>
                        {intl.formatMessage(messages.exchange)}
                      </Radio>
                    </Radio.Group>
                  </Form.Item>
                </div>
                <Row>
                  <Col
                    span={12}
                    className="general-card__firs-col--free-detail"
                  >
                    <Form.Item
                      name="name"
                      rules={[
                        {
                          required: true,
                          message: intl.formatMessage(
                            messages.requirePromotionName,
                          ),
                        },
                      ]}
                    >
                      <FloatLabel
                        label={`${intl.formatMessage(messages.promotionName)}*`}
                      />
                    </Form.Item>
                    <Row className="general-card__discount">
                      <Col span={12}>
                        <Form.Item
                          name="discountTypeId"
                          style={{ display: 'inline-block' }}
                          initialValue={1}
                        >
                          <Select
                            size="large"
                            placeholder={intl.formatMessage(
                              messages.discountBy,
                            )}
                            onChange={handleChangeDiscountBy}
                          >
                            <Select.Option value={1}>
                              {intl.formatMessage(messages.percent)}
                            </Select.Option>
                            <Select.Option value={2}>
                              {intl.formatMessage(messages.amount)}
                            </Select.Option>
                            <Select.Option value={3}>
                              {intl.formatMessage(messages.attachment)}
                            </Select.Option>
                          </Select>
                        </Form.Item>
                      </Col>
                      <Col span={12}>
                        {discountBy === 1 ? (
                          <Form.Item
                            name="discountValue"
                            className="input-addon orgin-input"
                            rules={[
                              validatePercent,
                              {
                                pattern: new RegExp('^[0-9]*$'),
                                message: intl.formatMessage(
                                  messages.requireNumberInput,
                                ),
                              },
                            ]}
                          >
                            <FloatLabel addonAfter="%" />
                          </Form.Item>
                        ) : discountBy === 2 ? (
                          <Form.Item
                            name="discountValue"
                            className="input-addon orgin-input"
                            rules={[
                              {
                                required: true,
                                message: intl.formatMessage(
                                  messages.requireDiscount,
                                ),
                              },
                              {
                                pattern: new RegExp('^[0-9,]*$'),
                                message: intl.formatMessage(
                                  messages.requireNumberInput,
                                ),
                              },
                            ]}
                          >
                            <FormatInput
                              formatter={digits}
                              addonAfter={currencyMemoi}
                            />
                          </Form.Item>
                        ) : (
                          <Form.Item
                            name="discountValue"
                            className="input-addon orgin-input"
                            rules={[validatePercent]}
                          >
                            <FloatLabel addonAfter="SL" />
                          </Form.Item>
                        )}
                      </Col>
                    </Row>
                    <Form.Item
                      name="description"
                      className="form-content__desc free-detail last-item"
                    >
                      <FloatLabel
                        label={intl.formatMessage(messages.description)}
                        rows={6}
                        typeInput="TextArea"
                      />
                    </Form.Item>
                  </Col>
                  <Col span={12}>
                    <Col>
                      <Row className="form-content-exchange__row">
                        <div className="form-content-exchange__img">
                          <Form.Item name="photo">
                            <Upload
                              name="photo"
                              listType="picture-card"
                              className="form-content-exchange__upload"
                              showUploadList={false}
                              action="https://www.mocky.io/v2/5cc8019d300000980a055e76"
                              beforeUpload={beforeUpload}
                              accept="image/*"
                            >
                              {imageUrl ? (
                                <img
                                  src={imageUrl}
                                  alt="avatar"
                                  style={{ width: '100%' }}
                                />
                              ) : (
                                uploadButton
                              )}
                            </Upload>
                          </Form.Item>
                        </div>
                      </Row>
                      <Row>
                        <Form.Item name="rules" className="form-content__cond last-item">
                          <FloatLabel
                          label={intl.formatMessage(messages.terms)}
                          rows={6}
                          typeInput="TextArea"
                        />
                        </Form.Item>
                      </Row>
                    </Col>
                  </Col>
                </Row>
              </Col>
              <Col
                xl={8}
                lg={8}
                md={8}
                sm={24}
                xs={24}
                className="general-right-card"
              >
                <Row className="conditions-card">
                  <Row>
                    <div className="form-header">
                      {intl.formatMessage(messages.condition)}
                    </div>
                  </Row>
                  <Row>
                    <Row className="time-card__content--firstRow">
                      <Col span={12}>
                        <Form.Item
                          className="input-addon"
                          name="minimumTotalApplied"
                          rules={[
                            {
                              pattern: new RegExp('^[0-9,]*$'),
                              message: intl.formatMessage(
                                messages.requireNumberInput,
                              ),
                            },
                          ]}
                        >
                          <FormatInput
                            formatter={digits}
                            addonAfter={currencyMemoi}
                            placeholder={intl.formatMessage(
                              messagesGlobal.minimumAmount,
                            )}
                          />
                        </Form.Item>
                      </Col>
                      <Col span={12}>
                        <Form.Item
                          className="input-addon"
                          name="quantity"
                          rules={[
                            {
                              pattern: new RegExp('^[0-9,]*$'),
                              message: intl.formatMessage(
                                messages.requireNumberInput,
                              ),
                            },
                          ]}
                        >
                          <FloatLabel
                            label={intl.formatMessage(
                              messagesGlobal.minimumQuantity,
                            )}
                          />
                        </Form.Item>
                      </Col>
                    </Row>
                    <Form.Item
                      className="input-addon last-item"
                      name="maxTotalApplied"
                      rules={[
                        {
                          pattern: new RegExp('^[0-9,]*$'),
                          message: intl.formatMessage(
                            messages.requireNumberInput,
                          ),
                        },
                      ]}
                    >
                      <FloatLabel
                        label={intl.formatMessage(
                          messagesGlobal.totalApplied,
                        )}
                      />
                    </Form.Item>
                  </Row>
                </Row>
                <Row className="time-card">
                  <Row>
                    <div className="form-header">
                      {intl.formatMessage(messagesGlobal.time)}
                    </div>
                  </Row>
                  <Row className="time-card__content">
                    <Row className="time-card__content--firstRow">
                      <Col span={12}>
                        <Form.Item
                          name="startDate"
                          rules={[
                            ({ getFieldValue }) => ({
                              validator(rule, value) {
                                const endDateValue = getFieldValue('endDate');
                                if (
                                  !value ||
                                  !endDateValue ||
                                  endDateValue.isAfter(value)
                                ) {
                                  return Promise.resolve();
                                }
                                return Promise.reject(
                                  intl.formatMessage(
                                    messages.startDateBeforeEndDate,
                                  ),
                                );
                              },
                            }),
                          ]}
                        >
                          <FloatLabel
                            label={intl.formatMessage(messages.fromDate)}
                            className="fw"
                            format={DATE_FORMAT}
                            typeInput="datepicker"
                          />
                        </Form.Item>
                      </Col>
                      <Col span={12}>
                        <Form.Item
                          name="endDate"
                          rules={[
                            ({ getFieldValue }) => ({
                              validator(rule, value) {
                                const startDateValue = getFieldValue(
                                  'startDate',
                                );
                                if (
                                  !value ||
                                  !startDateValue ||
                                  startDateValue.isBefore(value)
                                ) {
                                  return Promise.resolve();
                                }
                                return Promise.reject(
                                  intl.formatMessage(
                                    messages.endDateAfterStartDate,
                                  ),
                                );
                              },
                            }),
                          ]}
                        >
                           <FloatLabel
                            label={intl.formatMessage(messages.toDate)}
                            className="fw"
                            format={DATE_FORMAT}
                            typeInput="datepicker"
                          />
                        </Form.Item>
                      </Col>
                    </Row>
                    <Row className="time-card__content--firstRow">
                      <Col span={12}>
                        <Form.Item
                          name="startHour"
                          rules={[
                            validateTimeFormat(
                              form,
                              intl.formatMessage(messages.requireFormatTime, {
                                format: 'HH:MM:SS',
                              }),
                            ),
                          ]}
                        >
                          <FormatTimeInput
                            formatter={timeDigits}
                            placeholder={intl.formatMessage(messages.fromTime)}
                            className="fw"
                            maxLength={8}
                            showSecond={true}
                          />
                        </Form.Item>
                      </Col>
                      <Col span={12}>
                        <Form.Item
                          name="endHour"
                          rules={[
                            validateTimeFormat(
                              form,
                              intl.formatMessage(messages.requireFormatTime, {
                                format: 'HH:MM:SS',
                              }),
                            ),
                          ]}
                        >
                          <FormatTimeInput
                            formatter={timeDigits}
                            placeholder={intl.formatMessage(messages.toTime)}
                            className="fw"
                            maxLength={8}
                            showSecond={true}
                          />
                        </Form.Item>
                      </Col>
                    </Row>
                    <Form.Item name="daysOfWeek" className="last-item mutil-select">
                      <FloatLabel
                        className="padding-mutil-select"
                        size="middle"
                        label={intl.formatMessage(messages.promotionDays)}
                        mode="multiple"
                        showArrow
                        items={listDate}
                        typeInput="selectMutil"
                     />
                    </Form.Item>
                  </Row>
                </Row>
              </Col>
            </Row>
            <Row className="second-promotion-row">
              <Col xl={12} lg={12} md={12} sm={24} xs={24}>
                <Row className="conditions-card">
                  <Row>
                    <div className="form-header">
                      {intl.formatMessage(messagesGlobal.applyFor)}
                    </div>
                  </Row>
                  <Row className="time-card__content">
                    <Row className="time-card__content--firstRow">
                      <Col span={12}>
                        <Form.Item name="restaurantIds" className="mutil-select">
                          <FloatLabel
                            className="padding-mutil-select"
                            size={14}
                            label={intl.formatMessage(
                              messagesGlobal.restaurant,
                            )}
                            mode="multiple"
                            showArrow
                            typeInput="selectMutil"
                            items={restaurants}
                          />
                        </Form.Item>
                      </Col>
                      <Col span={12}>
                        <Form.Item
                          name="subjectsId"
                          rules={[
                            {
                              required: true,
                              message: intl.formatMessage(
                                messages.requirePromotionSubjects,
                              ),
                            },
                          ]}
                          className="customerGroup"
                        >
                          <FloatLabel
                            size="middle"
                            label={`${intl.formatMessage(messages.promotionSubjects)}*`}
                            items={subjects}
                            typeInput="selectMutil"
                          />
                        </Form.Item>
                      </Col>
                    </Row>
                    <Form.Item name="menuItemIds" className="mutil-select">
                      <FloatLabel
                        className="padding-mutil-select"
                        size="middle"
                        label={intl.formatMessage(messages.product)}
                        mode="multiple"
                        showArrow
                        items={menus}
                        typeInput="selectMutil"
                      />
                    </Form.Item>
                  </Row>
                </Row>
              </Col>
            </Row>
            <Button
              className="btn-submit"
              type="primary"
              shape="round"
              size="large"
              htmlType="submit"
            >
              {intl.formatMessage(messages.save)}
            </Button>
          </>
        ) : (
          <>
            <Row className="first-promotion-row point-detail">
              <Col
                xl={16}
                lg={16}
                md={16}
                sm={24}
                xs={24}
                className="general-card"
              >
                <div className="form-header">
                  {intl.formatMessage(messagesGlobal.general)}
                </div>
                <Form.Item
                  name="status"
                  initialValue={1}
                  style={{ display: 'none' }}
                ></Form.Item>
                <div className="form-content-exchange__promo-type">
                  <p>{intl.formatMessage(messages.promotionType)}</p>
                  <Form.Item name="discountRedeemTypeId" initialValue={1}>
                    <Radio.Group onChange={handleChangeType}>
                      <Radio value={1}>
                        {intl.formatMessage(messages.free)}
                      </Radio>
                      <Radio value={2}>
                        {intl.formatMessage(messages.exchange)}
                      </Radio>
                    </Radio.Group>
                  </Form.Item>
                </div>
                <Row>
                  <Col
                    span={12}
                    className="general-card__firs-col--point-detail"
                  >
                    <Form.Item
                      name="name"
                      rules={[
                        {
                          required: true,
                          message: intl.formatMessage(
                            messages.requirePromotionName,
                          ),
                        },
                      ]}
                    >
                      <FloatLabel
                        label={`${intl.formatMessage(messages.promotionName)}*`}
                      />
                    </Form.Item>
                    <Form.Item
                      name="redeemPoints"
                      className="input-addon"
                      rules={[
                        {
                          pattern: new RegExp('^[0-9]*$'),
                          message: intl.formatMessage(
                            messages.requireNumberInput,
                          ),
                        },
                        {
                          required: true,
                          message: intl.formatMessage(
                            messages.requirePoints,
                          ),
                        },
                      ]}
                    >
                      <FloatLabel
                        addonAfter={intl.formatMessage(messages.point)}
                        label={`${intl.formatMessage(messages.points)}*`}
                      />
                    </Form.Item>
                    <Row className="general-card__discount">
                      <Col span={12}>
                        <Form.Item
                          name="discountTypeId"
                          style={{ display: 'inline-block' }}
                          initialValue={1}
                        >
                          <Select
                            size="large"
                            placeholder={intl.formatMessage(
                              messages.discountBy,
                            )}
                            onChange={handleChangeDiscountBy}
                          >
                            <Select.Option value={1}>
                              {intl.formatMessage(messages.percent)}
                            </Select.Option>
                            <Select.Option value={2}>
                              {intl.formatMessage(messages.amount)}
                            </Select.Option>
                            <Select.Option value={3}>
                              {intl.formatMessage(messages.attachment)}
                            </Select.Option>
                          </Select>
                        </Form.Item>
                      </Col>
                      <Col span={12}>
                        {discountBy === 1 ? (
                          <Form.Item
                            name="discountValue"
                            className="input-addon"
                            rules={[
                              validatePercent,
                              {
                                pattern: new RegExp('^[0-9]*$'),
                                message: intl.formatMessage(
                                  messages.requireNumberInput,
                                ),
                              },
                            ]}
                          >
                            <FloatLabel addonAfter="%" label="0" />
                          </Form.Item>
                        ) : discountBy === 2 ? (
                          <Form.Item
                            name="discountValue"
                            className="input-addon"
                            rules={[
                              {
                                required: true,
                                message: intl.formatMessage(
                                  messages.requireDiscount,
                                ),
                              },
                              {
                                pattern: new RegExp('^[0-9,]*$'),
                                message: intl.formatMessage(
                                  messages.requireNumberInput,
                                ),
                              },
                              ({ getFieldValue }) => ({
                                validator(rule, value) {
                                  if (value === undefined || value =='') {
                                    return Promise.resolve();
                                  }
                                  value = value.replaceAll(',', '');
                                  if (parseInt(value) > 0) {
                                    return Promise.resolve();
                                  }
                                  return Promise.reject(
                                    intl.formatMessage(
                                      messages.requireInputGreaterThanZero,
                                    ),
                                  );
                                },
                              }),
                            ]}
                          >
                            <FormatInput
                              formatter={digits}
                              addonAfter={currencyMemoi}
                            />
                          </Form.Item>
                        ) : (
                          <Form.Item
                            name="discountValue"
                            className="input-addon"
                            rules={[validatePercent]}
                          >
                            <FloatLabel addonAfter="SL"/>
                          </Form.Item>
                        )}
                      </Col>
                    </Row>
                  </Col>
                  <Col span={12}>
                    <Col>
                      <Row>
                        <div className="form-content-exchange__img">
                          <Form.Item name="photo">
                            <Upload
                              name="photo"
                              listType="picture-card"
                              className="form-content-exchange__upload"
                              showUploadList={false}
                              action="https://www.mocky.io/v2/5cc8019d300000980a055e76"
                              beforeUpload={beforeUpload}
                              accept="image/*"
                            >
                              {imageUrl ? (
                                <img
                                  src={imageUrl}
                                  alt="avatar"
                                  style={{ width: '100%' }}
                                />
                              ) : (
                                uploadButton
                              )}
                            </Upload>
                          </Form.Item>
                        </div>
                      </Row>
                    </Col>
                  </Col>
                  <Row className="general-last-row">
                    <Col span={12}>
                      <Form.Item
                        name="description"
                        className="form-content__cond"
                      >
                        <FloatLabel
                        label={intl.formatMessage(messages.description)}
                        rows={6}
                        typeInput="TextArea"
                      />
                      </Form.Item>
                    </Col>
                    <Col span={12}>
                      <Form.Item name="rules" className="form-content__cond">
                      <FloatLabel
                        label={intl.formatMessage(messages.terms)}
                        rows={6}
                        typeInput="TextArea"
                      />
                      </Form.Item>
                    </Col>
                  </Row>
                </Row>
              </Col>
              <Col
                xl={8}
                lg={8}
                md={8}
                sm={24}
                xs={24}
                className="general-right-card"
              >
                <Row className="conditions-card">
                  <Row>
                    <div className="form-header">
                      {intl.formatMessage(messagesGlobal.applyFor)}
                    </div>
                  </Row>
                  <Row>
                    <Row className="time-card__content--firstRow">
                      <Form.Item name="restaurantIds" className="mutil-select">
                        <FloatLabel
                          className="padding-mutil-select"
                          size="middle"
                          label={intl.formatMessage(
                            messagesGlobal.restaurant,
                          )}
                          mode="multiple"
                          showArrow
                          typeInput="selectMutil"
                          items={restaurants}
                        />
                      </Form.Item>
                      <Form.Item
                        name="subjectsId"
                        rules={[
                          {
                            required: true,
                            message: intl.formatMessage(
                              messages.requirePromotionSubjects,
                            ),
                          },
                        ]}
                        className="customerGroup"
                      >
                        <FloatLabel
                          size="middle"
                          label={`${intl.formatMessage(messages.promotionSubjects)}*`}
                          typeInput="selectMutil"
                          items={subjects}
                        />
                      </Form.Item>
                    </Row>
                    <Form.Item name="daysOfWeek" className="mutil-select">
                      <FloatLabel
                        className="padding-mutil-select"
                        size="middle"
                        label={intl.formatMessage(messages.promotionDays)}
                        mode="multiple"
                        showArrow
                        typeInput="selectMutil"
                        items={listDate}
                     />
                    </Form.Item>
                  </Row>
                </Row>
                <Row className="time-card">
                  <Row>
                    <div className="form-header">
                      {intl.formatMessage(messagesGlobal.time)}
                    </div>
                  </Row>
                  <Row className="time-card__content">
                    <Row className="time-card__content--firstRow">
                      <Col span={12}>
                        <Form.Item
                          name="startDate"
                          rules={[
                            ({ getFieldValue }) => ({
                              validator(rule, value) {
                                const endDateValue = getFieldValue('endDate');
                                if (
                                  !value ||
                                  !endDateValue ||
                                  endDateValue.isAfter(value)
                                ) {
                                  return Promise.resolve();
                                }
                                return Promise.reject(
                                  intl.formatMessage(
                                    messages.startDateBeforeEndDate,
                                  ),
                                );
                              },
                            }),
                          ]}
                        >
                          <FloatLabel
                            label={intl.formatMessage(messages.fromDate)}
                            className="fw"
                            format={DATE_FORMAT}
                            typeInput="datepicker"
                          />
                        </Form.Item>
                      </Col>
                      <Col span={12}>
                        <Form.Item
                          name="endDate"
                          rules={[
                            ({ getFieldValue }) => ({
                              validator(rule, value) {
                                const startDateValue = getFieldValue(
                                  'startDate',
                                );
                                if (
                                  !value ||
                                  !startDateValue ||
                                  startDateValue.isBefore(value)
                                ) {
                                  return Promise.resolve();
                                }
                                return Promise.reject(
                                  intl.formatMessage(
                                    messages.endDateAfterStartDate,
                                  ),
                                );
                              },
                            }),
                          ]}
                        >
                          <FloatLabel
                            label={intl.formatMessage(messages.toDate)}
                            className="fw"
                            format={DATE_FORMAT}
                            typeInput="datepicker"
                          />
                        </Form.Item>
                      </Col>
                    </Row>
                    <Row className="time-card__content--firstRow">
                      <Col span={12}>
                        <Form.Item
                          name="startHour"
                          rules={[
                            validateTimeFormat(
                              form,
                              intl.formatMessage(messages.requireFormatTime, {
                                format: 'HH:MM:SS',
                              }),
                            ),
                          ]}
                        >
                          <FormatTimeInput
                            formatter={timeDigits}
                            placeholder={intl.formatMessage(messages.fromTime)}
                            className="fw"
                            maxLength={8}
                            showSecond={true}
                          />
                        </Form.Item>
                      </Col>
                      <Col span={12}>
                        <Form.Item
                          name="endHour"
                          rules={[
                            validateTimeFormat(
                              form,
                              intl.formatMessage(messages.requireFormatTime, {
                                format: 'HH:MM:SS',
                              }),
                            ),
                          ]}
                        >
                          <FormatTimeInput
                            formatter={timeDigits}
                            placeholder={intl.formatMessage(messages.toTime)}
                            className="fw"
                            maxLength={8}
                            showSecond={true}
                          />
                        </Form.Item>
                      </Col>
                    </Row>
                    <Form.Item name="menuItemIds" className="mutil-select">
                      <FloatLabel
                        className="padding-mutil-select"
                        size="middle"
                        label={`${intl.formatMessage(messages.product)}`}
                        mode="multiple"
                        showArrow
                        typeInput="selectMutil"
                        items={menus}
                      />
                    </Form.Item>
                  </Row>
                </Row>
              </Col>
            </Row>
            <Button
              className="btn-submit"
              type="primary"
              shape="round"
              size="large"
              htmlType="submit"
            >
              {intl.formatMessage(messages.save)}
            </Button>
          </>
        )}
      </Form>
    </PromotionAddNewWrapper>
  );
}

export default injectIntl(AddPromotion);
