import React, { useState, useEffect } from 'react';
import { Card, Input, Button, Form, Alert, notification, Spin, Modal } from 'antd';
import styled from 'styled-components';

import { MailOutlined, LockOutlined } from '@ant-design/icons';
import { useInjectReducer } from 'redux-injectors';
import authReducer from './authReducer';
import { LoginFormModel } from './types';
import axios, { AxiosResponse } from 'axios';
import { LOGIN, FORGOT_PASSWORD_URL } from 'containers/App/urls';
import { TOKEN_KEY, API_COMMON_ERROR_MESSAGE, USER_INFO_KEY } from 'containers/App/constants';
import { fetchAPI } from 'utils/fetchAPI';
import { useHistory } from 'react-router-dom';
import { RoutePaths } from 'containers/App/routes';
import { useDispatch } from 'react-redux';
import { AuthActionTypes } from './authConstants';
import { loginSuccessAction } from './authActions';
import messagesGlobal from 'translations/messages';
import { FormattedMessage, injectIntl } from 'react-intl';
import BUBBLES_LOGO from '../../images/bg-bubbles-logo.svg';
import { MOBILE_BREAK_POINT, TABLET_BREAK_POINT } from 'styles/constants';
import FloatLabel from 'components/FormatInput/FloatLabel';

const LoginWrapper = styled.div`
  position: relative;
  left: 55vw;
  top: 180px;
  width: 400px;
  border-radius: 10px;
  .ant-card {
    height: 400px;
    width: 400px;
    border: none;
    border-radius: 10px;
    box-shadow: 0 0 15px 0 rgba(255, 255, 255, 0.35);
    .ant-card-body {
      padding-top: 0;
    }
  }

  @media only screen and (max-width: ${TABLET_BREAK_POINT}px) {
    position: initial;
    text-align: center;
    margin: 10% 25%;
    width: 50%;
    .ant-card {
      height: 100%;
      width: 90%;
      .ant-card-body {
        padding: 10px;
      }
    }
  }

  @media only screen and (max-width: ${MOBILE_BREAK_POINT}px) {
    position: initial;
    text-align: center;
    margin: 25% 5%;
    width: 100%;
    .ant-card {
      height: 100%;
      width: 90%;
      .ant-card-body {
        padding: 10px;
      }
    }
  }
`;

const ModalWrapper = styled.div`
  .container {
    display: flex;
    flex-direction: column;
    flex-wrap: nowrap;
    justify-content: space-between;
    align-items: center;
    align-content: stretch;
    padding: 6px;
    .title {
      color: #333333;
      font-size: 18px;
      font-weight: 700;
      line-height: 19px;
      text-align: center;
      padding-bottom: 25px;
    }
    .email {
      width: 270px;
      height: 44px;
    }
    .save-button {
      width: 270px;
      height: 44px;
      border-radius: 22px;
      background-color: #9cd339;
    }
  }
`

const LoginHeader = styled.div`
  font-size: 30px;
  text-align: center;
  color: #fff;
  padding: 41px 0px 48px;
`;

const InputWrapper = styled.div`
  padding: 5px 0px;
  .ant-input-affix-wrapper {
    padding-left: 20px;
    background: transparent;
    border-radius: 29px;
    color: #fff;
    .ant-input-prefix {
      font-size: 16px;
    }
    input {
      padding: 0 0 0 10px!important;
      color: #000;
      background: transparent;
    }
  }
`;

const ButtonWrapper = styled.div`
  padding-top: 5px;
  button {
    width: 100%;
    border-radius: 29px;
    font-size: 16px;
  }
`;

const ForgotPasswordWrapper = styled.div`
  text-align: center;
  text-decoration: underline;
  font-size: 16px;
  color: #333333;
  padding-top: 20px;
  cursor: pointer;
`;

function Login(props) {
  const { intl } = props;
  const [form] = Form.useForm();
  const [popupForm] = Form.useForm();
  const [, forceUpdate] = useState();
  const [loading, setLoading] = useState(false);
  const [ visible, setVisible ] = useState(false);
  const history = useHistory();
  const dispatch = useDispatch();
  useInjectReducer({ key: 'auth', reducer: authReducer });

  // To disable submit button at the beginning.
  useEffect(() => {
    forceUpdate({});
  }, []);
  const onFinish = async (values: LoginFormModel) => {
    setLoading(true);
    const { username, password } = values;
    try {
      const loginResponse = await axios.post<LoginFormModel, AxiosResponse<any>>(
        LOGIN,
        { username, password: password },
      );
      setLoading(false);
      if (loginResponse?.data?.data?.token) {
        localStorage.setItem(TOKEN_KEY, loginResponse.data.data.token);
        localStorage.setItem(USER_INFO_KEY, JSON.stringify(loginResponse.data.data));
        dispatch(loginSuccessAction(loginResponse.data.data));
        history.replace(RoutePaths["/"]);
      } else {
        if (loginResponse.data.error) {
          notification.open({
            type: "error",
            message: loginResponse.data.message || API_COMMON_ERROR_MESSAGE,
            description: intl.formatMessage(messagesGlobal.loginErr),
          });
        }
      };
    } catch (e) {
      setLoading(false);
      notification.open({
        type: "error",
        message: intl.formatMessage(messagesGlobal.loginFailure),
        description: intl.formatMessage(messagesGlobal.accountWarning),
      });
    }
  };

  const showModal = () => {
    setVisible(true);
  };

  const handleOk = values => {
    fetchAPI({
      method: 'POST',
      url: FORGOT_PASSWORD_URL,
      data: values
    }).then(response => {
      if (response) {
        notification.open({
          type: 'success',
          message: intl.formatMessage(messagesGlobal.messageRestoreAcc)
        })
        setVisible(false);
      }
    }).catch(error => {
      setVisible(false);
    })
  };

  const handleCancel = e => {
    console.log(e);
    setVisible(false);
  };


  return (
    <LoginWrapper>
      <Spin spinning={loading}>
        <Card>
          <LoginHeader><img src={BUBBLES_LOGO}/></LoginHeader>
          <Form form={form} name="horizontal_login" onFinish={onFinish}>
            <InputWrapper >
              <Form.Item
                name="username"
                rules={[{ required: true, message: <FormattedMessage
                  {...messagesGlobal.requiredUserName}
                /> }]}
              >
                <FloatLabel allowClear label={intl.formatMessage(messagesGlobal.account)} prefix={<MailOutlined />} style={{color: "#D8D8D8"}} size="large"/>
              </Form.Item>
            </InputWrapper>
            <InputWrapper >
              <Form.Item
                name="password"
                rules={[{ required: true, message: <FormattedMessage
                  {...messagesGlobal.requiredPassword}
                /> }]}
              >
                <FloatLabel allowClear label={intl.formatMessage(messagesGlobal.password)} prefix={<LockOutlined />} type="password" style={{color: "#D8D8D8"}} size="large"/>
              </Form.Item>
            </InputWrapper>
            <ButtonWrapper>
              <Form.Item shouldUpdate={true}>
                {() => (
                  <Button
                    type="primary"
                    htmlType="submit"
                  >
                    <FormattedMessage
                  {...messagesGlobal.login}
                />
                  </Button>
                )}
              </Form.Item>
            </ButtonWrapper>
          </Form>
          <ForgotPasswordWrapper onClick={showModal}><FormattedMessage
                  {...messagesGlobal.forgotPassword}
                /></ForgotPasswordWrapper>
        </Card>
      </Spin>
      <Modal
        visible={visible}
        onOk={handleOk}
        onCancel={handleCancel}
        centered
        footer={null}
        width={330}
      >
        <ModalWrapper>
          <Form form={popupForm}>
            <div className="container">
              <div className="title"><FormattedMessage
                  {...messagesGlobal.forgotPassword}
                /></div>
              <Form.Item name="username" className="email" rules={[{required: true, message: <FormattedMessage
                  {...messagesGlobal.requiredUserName}
                />}]}>
                <FloatLabel label={`${intl.formatMessage(messagesGlobal.accountPlaceholder)}`} />
              </Form.Item>
              <Button className="save-button" type="primary"
                onClick={() => {
                  popupForm
                    .validateFields()
                    .then(values => {
                      handleOk(values);
                      popupForm.resetFields();
                    })
                    .catch(info => {
                      console.log('Validate Failed:', info);
                    });
                }}
              ><FormattedMessage
              {...messagesGlobal.sendBtn}
            /></Button>
            </div>
          </Form>
        </ModalWrapper>
      </Modal>
    </LoginWrapper>
  )
}
export default injectIntl(Login);