import React, { useEffect, useState } from 'react';
import { Button, DatePicker, Form, Input, Select, Row, Col } from 'antd';
import { useParams } from 'react-router-dom';
import moment from 'moment';
import { FacebookIcon, AppleIcon } from 'styles/icon-components';
import { updateCustomer } from '../actions';
import { useDispatch, useSelector } from 'react-redux';
import { createStructuredSelector } from 'reselect';
import { CustomerInfosWrapper } from '../Styles';
import { DATE_FORMAT } from '../../../utils/constants';
import { extractPhoneNumber } from '../../../utils/common';
import messages from '../messages';
import messagesGlobal from "../../../translations/messages";
import { injectIntl } from 'react-intl';
import { makeSelectPhoneCountries } from '../../App/selectors';
import { usePhoneAddon } from '../../../components/CustomHook/usePhoneAddon';
import FloatLabel from 'components/FormatInput/FloatLabel';
const { Option } = Select;

const globalSelector = createStructuredSelector({
  phoneCountries: makeSelectPhoneCountries(),
});

const facebookPrefix = (
  <span style={{ padding: '10px' }}>
    <FacebookIcon />
  </span>
);

const applePrefix = (
  <span style={{ padding: '5px' }}>
    <AppleIcon />
  </span>
);

function Information(props) {
  const dispatch = useDispatch();
  const { phoneCountries } = useSelector(globalSelector);
  const prefixSelector = usePhoneAddon(phoneCountries);
  const { customer, intl } = props;
  const selectGender = [
    {id: "male", name: intl.formatMessage(messages.male)},
    {id: "female", name: intl.formatMessage(messages.female)},
  ]
  const [code, number] = extractPhoneNumber(
    customer.phone,
    customer.phoneCountryCode,
  );
  customer.phone = number;
  customer.phoneCountryCode = code;

  customer.dateOfBirth = customer.dateOfBirth
    ? moment(customer.dateOfBirth)
    : undefined;
  const [form] = Form.useForm();
  // @ts-ignore
  const { id } = useParams();

  const onFinish = values => {
    let { dateOfBirth, phone, phoneCountryCode } = values;
    values.phoneNumber = phoneCountryCode + phone;
    delete values.phone;
    values.dateOfBirth = dateOfBirth
      ? moment(dateOfBirth).format(DATE_FORMAT)
      : dateOfBirth;
    dispatch(updateCustomer(id, values));
  };

  useEffect(() => {
    form.setFieldsValue(customer);
  }, [customer]);

  return (
    <>
      <Row className="customer-info-tab">
        <Col xl={8} lg={10} md={12} sm={14} xs={24} className="information-body">
          <Form form={form} onFinish={onFinish}>
            <CustomerInfosWrapper>
              <div className="customer-info_header">
                <h3 className="title-global">{intl.formatMessage(messages.customerInfo)}</h3>
              </div>
              <div className="customer-info_main">
                <div style={{ display: 'flex' }}>
                  <Form.Item
                    name="lastName"
                    className="customer-info_main_item"
                  >
                    <FloatLabel
                      label={intl.formatMessage(messages.firstName)}
                    />
                  </Form.Item>
                  <Form.Item
                    name="firstName"
                    className="customer-info_main_item"
                  >
                    <FloatLabel
                      label={intl.formatMessage(messages.lastName)}
                    />
                  </Form.Item>
                  <Form.Item name="gender" className="customer-info_main_item">
                    <FloatLabel label={intl.formatMessage(messages.gender)} items={selectGender} typeInput="selectMutil" className="padding-mutil-select"/>
                  </Form.Item>
                </div>
                <div style={{ display: 'flex' }}>
                  <Form.Item
                    name="phone"
                    className="customer-info_main_item"
                    rules={[
                      {
                        message: intl.formatMessage(
                          messagesGlobal.invalidPhoneNumber,
                        ),
                        pattern: /^[\d]+$/,
                      },
                      () => ({
                        validator(rule, value) {
                          if (value.length > 10) {
                            return Promise.reject(
                              intl.formatMessage(messagesGlobal.invalidPhoneNumber),
                            );
                          } else {
                            return Promise.resolve();
                          }
                        },
                      }),
                    ]}
                  >
                    <FloatLabel
                      addonBefore={prefixSelector}
                      label={intl.formatMessage(messages.phoneNumber)}
                    />
                  </Form.Item>
                  <Form.Item
                    name="dateOfBirth"
                    className="customer-info_main_item"
                  >
                    <FloatLabel
                      format={DATE_FORMAT}
                      style={{ width: '100%' }}
                      label={intl.formatMessage(messages.dob)}
                      typeInput="datepicker"
                    />
                  </Form.Item>
                </div>
                <Form.Item
                  name="email"
                  className="customer-info_main_item"
                  rules={[
                    {
                      message: intl.formatMessage(messages.invalidEmail),
                      type: 'email',
                    },
                  ]}
                >
                  <FloatLabel label="Email" />
                </Form.Item>
                <Form.Item name="address" className="customer-info_main_item">
                  <FloatLabel label={intl.formatMessage(messages.location)} />
                </Form.Item>
                <div style={{ display: 'flex' }}>
                  {customer.facebookId && (
                    <Form.Item noStyle className="customer-info_main_item">
                      {facebookPrefix}
                    </Form.Item>
                  )}
                  {customer.appleId && (
                    <Form.Item noStyle className="customer-info_main_item">
                      {applePrefix}
                    </Form.Item>
                  )}
                </div>
              </div>
            </CustomerInfosWrapper>
            <div className="btn-group">
              <Button type="primary" htmlType="submit" className="btn-save">
                {intl.formatMessage(messages.save)}
              </Button>
            </div>
          </Form>
        </Col>
      </Row>

    </>
  );
}

export default injectIntl(Information);
