import React, { useEffect, useState,useCallback } from 'react';
import { Upload, notification, Switch, Form } from 'antd';
import { LoadingOutlined } from '@ant-design/icons';
import {
  AvatarCustomerIcon,
  EditUploadImageIcon,
} from 'styles/icon-components';
import { PUBLIC_API, API_URL } from 'containers/App/urls';
import moment from 'moment';
import { fetchAPI } from 'utils/fetchAPI';
import BG_SCORE from '../../../images/newScore.svg';
import BG_SPENT from '../../../images/newSpent.svg';
import BG_QR_CODE from '../../../images/qr_code.svg';
import { TOKEN_KEY } from 'containers/App/constants';
import { roundNumber } from 'utils/number';
import { CustomerReputaionWrapper } from '../Styles';
import { getBase64 } from '../../../utils/common';
import { DATE_FORMAT } from 'utils/constants';
import messages from '../messages';
import messageGlobal from 'translations/messages';
import { FormattedMessage, injectIntl } from 'react-intl';
import NumberFormat from 'react-number-format';
import { fileValidation } from 'utils/validation';
import cacheUtils from 'utils/cacheUtils';
import { DEFAULT_CURRENCY } from 'utils/constants';
import { useCurrencyHook } from 'components/CustomHook/useCurrencyHook';



function Reputation(props) {
  const { customer, intl } = props;
  const {
    id,
    avatar,
    firstName,
    lastName,
    score,
    total = 0,
    qrcode,
    createdTime,
    lastServeTime
  } = customer;
  const [imageUrl, setImageUrl] = useState<any>();
  const [loading, setLoading] = useState(false);
  const [status, setStatus] = useState<number>();
  const token = localStorage.getItem(TOKEN_KEY);
  const userInfo = cacheUtils.getUserInfo();



  const currency = useCallback(
    userInfo => {
      if (userInfo) {
        const userInfoJson = JSON.parse(userInfo);
        if (userInfoJson.profile?.merchant?.setting) {
          return userInfoJson.profile?.merchant?.setting.currency;
        }
        return DEFAULT_CURRENCY;
      } else {
        return DEFAULT_CURRENCY;
      }
    },
    [userInfo],
  );
  const currencyMemoi = currency(userInfo);
  const uploadButton = (
    <div>
      {loading ? <LoadingOutlined /> : <AvatarCustomerIcon />}
      <div className="ant-upload-text avatar__upload__thumb__text">
        {intl.formatMessage(messageGlobal.avatar)}
      </div>
    </div>
  );
  const beforeUpload = file => {
    if (!fileValidation(file.name)) {
      notification.error({
        message: intl.formatMessage(messageGlobal.invalidFile),
      });
      return false;
    }
    const reader = new FileReader();
    reader.onloadend = e => {
      setImageUrl(reader.result!);
    };
    reader.readAsDataURL(file);
    return true;
  };

  const changeStatus = async value => {
    const response = await fetchAPI({
      method: !value ? 'DELETE' : 'PUT',
      url: !value
        ? `${API_URL}/users/block-user/${id}`
        : `${API_URL}/users/unblock-user/${id}`,
      formData: false,
    });
    if (response) {
      setStatus(value ? 1 : 3);
    }
  };

  const handleChange = info => {
    if (info.file.status === 'uploading') {
      setLoading(true);
      return;
    }
    if (info.file.status === 'done') {
      getBase64(info.file.originFileObj, imageUrl => {
        setLoading(false);
        setImageUrl(imageUrl);
      });
    }
  };
  useEffect(() => {
    if (customer) {
      setImageUrl(
        customer.avatar ? `${PUBLIC_API}/${customer.avatar}` : customer.avatar,
      );
      setStatus(customer.status ? customer.status.statusId : 3);
    }
  }, [customer]);
  return (
    <CustomerReputaionWrapper>
      <div className="info">
        <div>
          <Upload
            name="avatar"
            listType="picture-card"
            className="info__avatar"
            showUploadList={false}
            action={`${API_URL}/users/customer/${id}`}
            method="PUT"
            headers={{
              Authorization: `Bearer ${token}`,
            }}
            beforeUpload={beforeUpload}
            onChange={handleChange}
            accept="image/*"
          >
            {imageUrl ? (
              <img
                src={imageUrl}
                alt="avatar"
                style={{ width: '70px', height: '70px', objectFit: 'cover' }}
              />
            ) : (
              uploadButton
            )}
            {imageUrl && (
              <div className="edit-upload-image-icon">
                <EditUploadImageIcon />
              </div>
            )}
          </Upload>
        </div>
        <div className="info__general">
          <p className="info__name">{[firstName, lastName].join(' ')}</p>
          <div className="info__desc">
            <span className="info__date">
              {intl.formatMessage(messages.createdTime, {
                createdTime: moment(createdTime).format(DATE_FORMAT),
              })}
            </span>
            <div className="restaurant-infos__text__minor">
              <Switch
                checked={status === 1 ? true : false}
                onChange={changeStatus}
                style={{ marginRight: '15px' }}
              />
              <span style={{ color: status === 1 ? '#9CD339' : '#9C9696' }}>
                {status === 1 ? (
                  <FormattedMessage {...messageGlobal.activating} />
                ) : (
                  <FormattedMessage {...messageGlobal.stopAccount} />
                )}
              </span>
            </div>
          </div>
        </div>
      </div>
      <div className="reputation">
        <div className="reputation__using">
          <div>
            <p className="reputation__title">
              <FormattedMessage {...messageGlobal.lastTimeUsing} />
            </p>
          </div>
          <div className="time">
            {lastServeTime? moment(lastServeTime).format(DATE_FORMAT): null}
          </div>
        </div>
        <div className="reputation__spent">
          <div className="reputation__score">
            <img className="reputation__thumb" src={BG_SCORE} />
            <p className="reputation__title">
              <FormattedMessage {...messageGlobal.point} />
            </p>
            <span className="reputation__number">{score}</span>
          </div>
          <div>
            <img className="reputation__thumb" src={BG_SPENT} />
            <p className="reputation__title">
              <FormattedMessage {...messageGlobal.spending} />
            </p>
            <span className="reputation__number" style={{ color: '#dc3b7f' }}>
              <NumberFormat
                value={roundNumber(total)}
                displayType={'text'}
                thousandSeparator={true}
                renderText={value => value+` ${currencyMemoi}`}
              >
                {total}
              </NumberFormat>
            </span>
          </div>
        </div>
        <div className="reputation__qr-code">
          {qrcode ? (
            <img
              className="reputation__qr-image"
              src={`data:image/svg+xml,${escape(qrcode)}`}
            />
          ) : (
            <img src={BG_QR_CODE} />
          )}
        </div>
      </div>
    </CustomerReputaionWrapper>
  );
}

export default injectIntl(Reputation);
