import React, { useEffect, useCallback } from "react";
import { useDispatch, useSelector, } from "react-redux";
import { createStructuredSelector } from "reselect";

import { useInjectReducer, useInjectSaga } from "redux-injectors";

import makeSelectCustomerList from "./selectors";
import reducer, { initialState } from "./reducer";
import saga from "./saga";
import { Spin } from "antd";
import { LoadingOutlined } from '@ant-design/icons';
import Card from "./Card";
import Header from "./Header";
import { deleteCustomer, getCustomers, setFilter, setPaging } from "./actions";
import { CustomerListWrapper } from "./styles";
import { injectIntl } from "react-intl";
import messages from "translations/messages";

const stateSelector = createStructuredSelector({
  customerList: makeSelectCustomerList(),
});
interface Props {}

// eslint-disable-next-line @typescript-eslint/no-unused-vars
function CustomerList({ intl }) {
  useInjectReducer({ key: 'customerList', reducer: reducer });
  useInjectSaga({ key: 'customerList', saga: saga });

  // eslint-disable-next-line @typescript-eslint/no-unused-vars
  const { customerList } = useSelector(stateSelector);
  const { isFetching, loadMore,  customers, mode, paging: { page, limit, total }, filter: { mode: filterMode, search } } = customerList;

  const dispatch = useDispatch(); // eslint-disable-line @typescript-eslint/no-unused-vars

  const onscroll = useCallback(() => {
    if (document.documentElement.scrollHeight === (document.documentElement.scrollTop + window.innerHeight)
      && (total > page * limit)) {
        dispatch(getCustomers(filterMode, search ? search.trim() : '', page + 1, limit, true));
      }
  }, [page, limit, total]);

  useEffect(() => {
    dispatch(getCustomers('all', '', initialState.paging.page, initialState.paging.limit, false));
  }, []);

  useEffect(() => {
    window.addEventListener('scroll', onscroll, false);
    return () => {
      window.removeEventListener('scroll', onscroll, false);
    }
  }, [page, limit, total])


  const onClickSearch = (filterValues) => {
    const { mode, search } = filterValues;
    dispatch(setFilter(mode, search));
    dispatch(getCustomers(mode, search ? search.trim() : '', 1, limit, false));
  }

  const onClickRemove = (id) => {
    dispatch(deleteCustomer(id));
  };


  useEffect(() => {
    if (page === 1) {
      setTimeout(()=>{
        try{
          const elm = document.getElementById("customer-list");
          if(elm){
            if ( elm.offsetHeight < window.innerHeight && (total > page * limit)) {
              dispatch(getCustomers(filterMode, search ? search.trim() : "", page + 1, limit, true));
            }
          }
        }catch (e) {

        }
      },1000)
    }
  }, [total]);

  return (
    <CustomerListWrapper>
      <Header mode={mode} onClickSearch={onClickSearch}/>
      <Spin spinning={isFetching}>
        <div className="list">
          {
            customers.map(customer => (
              <div className="card">
                <Card customer={customer} onClickRemove={onClickRemove} />
              </div>
            ))
          }
        </div>
        {
            loadMore && (
              <Spin indicator={<LoadingOutlined style={{ fontSize: 24 }} spin />} tip={intl.formatMessage(messages.loading)}>
                <div></div>
              </Spin>
            )
          }
      </Spin>
    </CustomerListWrapper>
  )
}

export default injectIntl(CustomerList);
