import React, { useEffect, useState, useCallback } from 'react';
import { useDispatch, useSelector } from 'react-redux';
import { createStructuredSelector } from 'reselect';
import message from 'translations/messages';
import { useInjectReducer, useInjectSaga } from 'redux-injectors';
import { fetchAPI } from 'utils/fetchAPI';
import makeSelectPromotionDetail from './selectors';
import reducer from './reducer';
import saga from './saga';
import { useParams } from 'react-router-dom';
import { Button, Form, Upload, notification } from 'antd';
import IMG_RES_DEFAULT from '../../images/restaurant-placeholder.svg';
import { NewUploadIcon } from '../../styles/icon-components';
import { LoadingOutlined } from '@ant-design/icons';
import { TOKEN_KEY } from '../App/constants';
import Avatar from 'react-avatar';
import { digits } from 'utils/number';
import {
  ContainerImg,
  ContainerEditImg,
} from 'styles/custom-styled-components';
import { PUBLIC_API, API_URL } from '../App/urls';
import { PromotionIcon } from '../../styles/icon-components';
import { fileValidation } from '../../utils/validation';
import { PromotionAddNewWrapper } from './styles';
import { getBase64 } from '../../utils/common';
import { initials } from 'utils/string';
import { getGift, updateGift, addGift } from './actions';
import { DATE_FORMAT } from '../../utils/constants';
import { injectIntl } from 'react-intl';
import messages from './messages';
import messagesGlobal from 'translations/messages';
import { FormatInput } from 'components/FormatInput';
import cacheUtils from 'utils/cacheUtils';
import FloatLabel from 'components/FormatInput/FloatLabel';
import moment from 'moment';

const stateSelector = createStructuredSelector({
  giftDetail: makeSelectPromotionDetail(),
});

interface Props {
  readonly intl: any;
  isCreate?: boolean;
}
// eslint-disable-next-line @typescript-eslint/no-unused-vars
function GiftCardDetail(props: Props) {
  useInjectReducer({ key: 'giftDetail', reducer: reducer });
  useInjectSaga({ key: 'giftDetail', saga: saga });

  // eslint-disable-next-line @typescript-eslint/no-unused-vars
  const { giftDetail } = useSelector(stateSelector);
  const dispatch = useDispatch(); // eslint-disable-line @typescript-eslint/no-unused-vars
  const { gift, isFetching } = giftDetail;

  const userInfo = cacheUtils.getUserInfo();
  const userInfoJson = JSON.parse(userInfo ? userInfo : '');
  const { merchant } = userInfoJson?.profile;
  const currency = useCallback(
    userInfo => {
      if (userInfo) {
        if (userInfoJson.profile?.merchant?.setting) {
          return userInfoJson.profile?.merchant?.setting.currency;
        }
        return 'VND';
      } else {
        return 'VND';
      }
    },
    [userInfo],
  );
  const currencyMemoi = currency(userInfo);
  // @ts-ignore
  const { id } = useParams();
  const [imageUrl, setImageUrl] = useState<any>();
  const [price, setPrice] = useState<any>();
  const [startDate, setStartDate] = useState<any>();
  const [endDate, setEndDate] = useState<any>();
  const [form] = Form.useForm();
  const token = localStorage.getItem(TOKEN_KEY);

  const { intl, isCreate } = props;

  useEffect(() => {
    if (id) {
      dispatch(getGift(id));
    }
    return () => {
      form.setFieldsValue({});
    };
  }, []);
  useEffect(() => {
    if (!isCreate && gift) {
      setImageUrl(gift && gift.photo ? `${PUBLIC_API}/${gift?.photo}` : null);
      gift.amount = gift.amount
        ? Math.round(parseInt(gift.amount)).toString()
        : undefined;
      gift.price = gift.price
        ? Math.round(parseInt(gift.price)).toString()
        : undefined;
      // @ts-ignore
      form.setFieldsValue(gift);
      setPrice(gift.price);
      setStartDate(moment(gift.startDate).format(DATE_FORMAT));
      setEndDate(moment(gift.endDate).format(DATE_FORMAT));
    }
  }, [gift]);

  const beforeUpload = file => {
    if (!fileValidation(file.name)) {
      notification.error({
        message: intl.formatMessage(messagesGlobal.invalidFile),
      });
      return false;
    }
    const reader = new FileReader();
    reader.onloadend = e => {
      setImageUrl(reader.result!);
    };
    reader.readAsDataURL(file);
    return false;
  };

  const handleChangePrice = info => {
    setPrice(info.target.value);
  };

  const handleChangeStartDate = (date, dateString) => {
    setStartDate(dateString);
  };

  const handleChangeEndDate = (date, dateString) => {
    setEndDate(dateString);
  };

  const onFinish = async values => {
    console.log(values);
    for (var key in values) {
      if (typeof values[key] === 'string') {
        values[key] = values[key].trim();
        if (/^[0-9,]+$/.test(values[key])) {
          values[key] = values[key].replace(/,/g, '');
        }
      }
    }
    if (id) {
      delete values.photo;
      let photo = gift && gift.photo ? gift.photo : '';
      const { files } = values;
      const imageFile = files?.file;
      if (imageFile) {
        const formData = new FormData();
        formData.append('photo', imageFile!);
        photo = await fetchAPI({
          method: 'PUT',
          url: `${API_URL}/gift-cards/${id}`,
          formData: true,
          data: formData,
        });
        delete values.files;
      }
      dispatch(updateGift({ id, gift: values }));
    } else {
      const { photo } = values;
      if (photo && photo.file) {
        values.photo = photo.file;
      }
      dispatch(addGift({ gift: values }));
    }
  };

  const uploadButton = (
    <div>
      {isFetching ? (
        <LoadingOutlined />
      ) : id ? (
        <img src={IMG_RES_DEFAULT} />
      ) : (
        <PromotionIcon />
      )}
      <div className="ant-upload-text" style={{ marginTop: '15px' }}>
        {intl.formatMessage(messages.image)}
      </div>
      <span className="img-hint">
        {intl.formatMessage(messagesGlobal.encourageCondition, {
          height: 345,
          width: 140,
        })}
      </span>
    </div>
  );

  return (
    <PromotionAddNewWrapper>
      <Form
        form={form}
        className="form"
        style={{ height: '100%', marginTop: id ? 20 : 0 }}
        onFinish={onFinish}
      >
        <div className="promo-header" hidden={!id}>
          <ContainerEditImg
            imgBase64={imageUrl}
            height={120}
            className="promo-header__img  update-gift"
          >
            <Form.Item name="files">
              <Upload
                name="photo"
                listType="picture-card"
                className="promo-header__upload"
                showUploadList={false}
                beforeUpload={beforeUpload}
                accept="image/*"
              >
                <div></div>
              </Upload>
            </Form.Item>
          </ContainerEditImg>
          <div className="promo-header__info">
            <h3 className="promo-header__info-name">
              {gift ? gift!.name : ''}
            </h3>
            <span className="promo-header__info-time">
              {intl.formatMessage(messages.duration, {
                from: gift?.startDate?.format(DATE_FORMAT),
                to: gift?.endDate?.format(DATE_FORMAT),
              })}
            </span>
          </div>
        </div>

        <>
          <div className="form-content-exchange">
            <div>
              <Form.Item
                name="name"
                rules={[
                  {
                    required: true,
                    message: `${intl.formatMessage(
                      messagesGlobal.requiredInput,
                    )} ${intl
                      .formatMessage(messagesGlobal.giftName)
                      .toLowerCase()}!`,
                  },
                ]}
              >
                <FloatLabel
                  label={`${intl.formatMessage(messagesGlobal.giftName)}*`}
                />
              </Form.Item>
              <div className="form-content-exchange__discount-percent">
                <Form.Item
                  name="amount"
                  className="input-addon"
                  rules={[
                    {
                      required: true,
                      message: intl.formatMessage(messages.requireNumberInput),
                    },
                  ]}
                  style={{
                    display: 'inline-block',
                    width: 'calc(50% - 12px)',
                  }}
                >
                  <FormatInput
                    formatter={digits}
                    addonAfter={currencyMemoi}
                    placeholder={`${intl.formatMessage(
                      messagesGlobal.giftPrice,
                    )}*`}
                    onChange={handleChangePrice}
                  />
                </Form.Item>
                <Form.Item
                  name="price"
                  className="input-addon"
                  rules={[
                    {
                      required: true,

                      message: intl.formatMessage(messages.requireDiscount),
                    },
                    ({ getFieldValue }) => ({
                      validator(rule, value) {
                        const discount = getFieldValue(['amount']);
                        if (
                          value &&
                          discount &&
                          parseInt(value.toString().replace(/,/g, '')) >
                            parseInt(discount.toString().replace(/,/g, ''))
                        ) {
                          return Promise.reject(
                            intl.formatMessage(message.requiredValidPrice),
                          );
                        }
                        return Promise.resolve();
                      },
                    }),
                    {
                      pattern: new RegExp('^[0-9,]*$'),
                      message: intl.formatMessage(messages.requireNumberInput),
                    },
                  ]}
                  style={{
                    display: 'inline-block',
                    width: 'calc(50% - 12px)',
                  }}
                >
                  <FormatInput
                    formatter={digits}
                    addonAfter={currencyMemoi}
                    placeholder={`${intl.formatMessage(
                      messagesGlobal.discountPrice,
                    )}*`}
                  />
                </Form.Item>
              </div>
              <div className="form-content-exchange__discount-percent">
                <Form.Item
                  name="startDate"
                  rules={[
                    ({ getFieldValue }) => ({
                      validator(rule, value) {
                        const endDateValue = getFieldValue('endDate');
                        if (
                          !value ||
                          !endDateValue ||
                          endDateValue.isAfter(value)
                        ) {
                          return Promise.resolve();
                        }
                        return Promise.reject(
                          intl.formatMessage(messages.startDateBeforeEndDate),
                        );
                      },
                    }),
                  ]}
                >
                  <FloatLabel
                    label={intl.formatMessage(messages.fromDate)}
                    className="fw"
                    format={DATE_FORMAT}
                    onChange={handleChangeStartDate}
                    typeInput="datepicker"
                  />
                </Form.Item>
                <Form.Item
                  name="endDate"
                  rules={[
                    ({ getFieldValue }) => ({
                      validator(rule, value) {
                        const startDateValue = getFieldValue('startDate');
                        if (
                          !value ||
                          !startDateValue ||
                          startDateValue.isBefore(value)
                        ) {
                          return Promise.resolve();
                        }
                        return Promise.reject(
                          intl.formatMessage(messages.endDateAfterStartDate),
                        );
                      },
                    }),
                  ]}
                >
                  <FloatLabel
                    label={intl.formatMessage(messages.toDate)}
                    className="fw"
                    format={DATE_FORMAT}
                    onChange={handleChangeEndDate}
                    typeInput="datepicker"
                  />
                </Form.Item>
              </div>
              <Form.Item
                name="note"
                className="form-content-exchange__desc"
                style={{ display: id ? 'none' : 'flex' }}
              >
                <FloatLabel
                  label={intl.formatMessage(messages.description)}
                  rows={6}
                  typeInput="TextArea"
                />
              </Form.Item>
            </div>
            <Form.Item
              name="note"
              className="form-content-exchange__desc"
              style={{ display: !!!id ? 'none' : 'flex' }}
            >
              <FloatLabel
                label={intl.formatMessage(messages.description)}
                rows={6}
                typeInput="TextArea"
              />
            </Form.Item>
            <ContainerImg
              imgBase64={imageUrl}
              height={150}
              className="form-content-exchange__img add-gift"
              hidden={!!id}
            >
              <Form.Item name="photo">
                <Upload
                  name="photo"
                  listType="picture-card"
                  className="form-content-exchange__upload"
                  showUploadList={false}
                  action="https://www.mocky.io/v2/5cc8019d300000980a055e76"
                  beforeUpload={beforeUpload}
                  accept="image/*"
                >
                  <div className="edit-upload-image-shop">
                    {merchant.photo ? (
                      <img
                        className="user__avatar"
                        src={`${PUBLIC_API}/${merchant.photo}`}
                      />
                    ) : (
                      <Avatar
                        className="user__avatar"
                        initials={initials}
                        name={merchant.name}
                        size="30"
                      />
                    )}
                    <span className="user__name--name">{merchant.name}</span>
                  </div>
                  {price && (
                    <div className="edit-upload-image-amount">
                      <span>{`${price} ${currencyMemoi}`}</span>
                    </div>
                  )}
                  {!!startDate && !!endDate && (
                    <div className="edit-upload-image-time">
                      <span>
                        {intl.formatMessage(messages.duration, {
                          from: startDate ? startDate : '',
                          to: endDate ? endDate : '',
                        })}
                      </span>
                    </div>
                  )}
                  <div className="edit-upload-image-icon">
                    <span>
                      <NewUploadIcon />
                    </span>
                  </div>
                </Upload>
              </Form.Item>
            </ContainerImg>
          </div>
          <Button
            className="btn-submit"
            type="primary"
            shape="round"
            size="large"
            htmlType="submit"
          >
            {intl.formatMessage(messages.save)}
          </Button>
        </>
      </Form>
    </PromotionAddNewWrapper>
  );
}

export default injectIntl(GiftCardDetail);
