import React, { ReactElement, useState, useEffect, CSSProperties, useReducer } from "react";
import styled from 'styles/styled-components';
import BG_LOGIN from '../../images/bg-login.svg';
import THINK_LOGO from '../../images/bg-bubbles-logo.svg';
import { Layout as AntdLayout, Menu, Dropdown } from 'antd';
import { HEADER_HEIGHT, PRIMARY_COLOR, TABLET_BREAK_POINT, BLACK_COLOR } from 'styles/constants';
import { FormattedMessage } from 'react-intl';
import messages from 'translations/messages';
import _ from 'lodash';
import { useDispatch, useSelector } from 'react-redux';
import { useHistory } from 'react-router-dom';
import { loginSuccessAction } from 'containers/Auth/authActions';
import { useInjectReducer } from 'redux-injectors';
import authReducer from 'containers/Auth/authReducer';
import { RoutePaths } from 'containers/App/routes';
import {
  DashBoardIcon, StoreIcon, StaffIcon, MenuIcon, CustomerIcon, CustomerIconActive, GiftIcon,
  ReportIcon,
  ReportIconActive,
  SettingIcon,
  CardIcon,
  PayrollIcon,
  SettingIconActive,
  GiftIconActive,
  CardIconActive,
  PayrollIconActive,
  MenuIconActive,
  ServiceIconActive,
  ServiceIcon,
  StaffIconActive,
  StoreIconActive,
  DashBoardIconActive,
  StorageIcon,
  StorageIconActive
} from 'styles/icon-components';
import { USER_INFO_KEY, TOKEN_KEY } from 'containers/App/constants';
import { PUBLIC_API } from 'containers/App/urls';
import ChangePasswordModal from 'components/ChangePasswordModal';
import Avatar from 'react-avatar';
import { initials } from 'utils/string';

export type LayoutType = 'FULL' | 'MAIN';

const FullLayout = styled.div`
  background: url(${BG_LOGIN}) no-repeat center center fixed;
  background-size: cover;
  height: 100vh;
  overflow: hidden;
`;

const MainLayoutWrapper = styled.div`
  overflow: hidden;
  .ant-layout {
    background: #fafafa;
  }
  .ant-layout-header {
    padding: 0;
    background: #fff;
    height: ${HEADER_HEIGHT};
  }
  .ant-menu-item > span:last-child {
    vertical-align: middle;
  }
  .main-layout-header {
    display: flex;
    border-bottom: solid 1px rgba(0, 0, 0, 0.15);
    box-shadow: 0 2px 4px 0 rgba(0, 0, 0, 0.15);
    .user {
      &__name {
        justify-self: end;
        line-height: ${HEADER_HEIGHT};
        padding-right: 10px;
        margin-left: auto;
        margin-right: 10px;
        &--name {
          color: #DC5454;
        }
      }
      &__avatar {
        border-radius: 100%;
        height: 30px;
        width: 30px;
        margin-left: 10px;
        object-fit: cover;
      }
    }
  }
  .ant-menu-item::after {
    right: auto;
    left: 0;
    border-radius: 1.5px;
  }
  .ant-menu:not(.ant-menu-horizontal) .ant-menu-item-selected {
    background: none;
  }
  .ant-layout-sider {
    box-shadow: 1px 0 3px 0 rgba(0, 0, 0, 0.1);
  }
  @media screen { 
    .touch-card:hover {
      background-color: rgba(0, 0, 0, 0.20000000298023224);
    }
  } 
  @media handheld { 
    .touch-card:click {
      background-color: rgba(0, 0, 0, 0.20000000298023224);
    }
  }
  @media screen and (max-width:800px) { 
    .touch-card:click {
      background-color: rgba(0, 0, 0, 0.20000000298023224);
    }
  }
`;

const Logo = styled.div`
  height: 60px;
  margin: 20px 0px;
  text-align: center;
`;

const menuKeys = [RoutePaths.dashboard, RoutePaths.restaurant, RoutePaths.staff, RoutePaths.service, RoutePaths.menu, RoutePaths.payroll, RoutePaths.storage,
RoutePaths.customer, RoutePaths.promotion, RoutePaths.payment, RoutePaths.report, RoutePaths.setting];
interface LayoutProps {
  type: LayoutType;
  children: ReactElement;
  header?: ReactElement;
}

export function Layout(props: LayoutProps): ReactElement {
  useInjectReducer({ key: 'auth', reducer: authReducer });

  const { type, children, header } = props;

  if (type === 'FULL') {
    return <FullLayout>{children}</FullLayout>;
  }
  return <MainLayout header={header}>{children}</MainLayout>;
}

const getDefaultActiveMenuKey = (pathname) => {
  const defaultActiveMenuKey = menuKeys.find(key => pathname.includes(key));
  if (pathname === RoutePaths["/"]) {
    return RoutePaths.dashboard;
  }
  return defaultActiveMenuKey!;
}

function MainLayout(props): ReactElement {
  const userInfoString = localStorage.getItem(USER_INFO_KEY);
  if (!userInfoString) {
    return <span>...</span>;
  }
  const userInfo = JSON.parse(userInfoString);
  const { id, avatar, firstName, lastName, permissions: { Web } } = userInfo.profile!;
  // const isHamburgerDisplay = window.innerWidth <= TABLET_BREAK_POINT;
  const dispatch = useDispatch();
  const history = useHistory();
  const defaultActiveMenuKey = getDefaultActiveMenuKey(history.location.pathname);
  const { children, header } = props;
  const [activeMenuKey, setActiveMenuKey] = useState(defaultActiveMenuKey);
  const [showPopup, setShowpopup] = useState(false);
  const [, forceUpdate] = useReducer(x => x + 1, 0);

  // console.log('re-render', window.innerWidth, isHamburgerDisplay);

  useEffect(() => {
    window.addEventListener('orientationchange', () => {
      // can not find better solution
      window.setTimeout(() => {
        forceUpdate();
      }, 200);
    }, false)
  }, [])

  const { Header, Sider, Content } = AntdLayout;
  const handleLogout = () => {
    localStorage.removeItem(TOKEN_KEY);
    localStorage.removeItem(USER_INFO_KEY);
    dispatch(loginSuccessAction(null));
  }

  const handleClick = (e) => {
    setActiveMenuKey(e.key);
    history.push(e.key);
  }

  const handleClickLogo = (e) => {
    history.push(RoutePaths["/"]);
  }

  const onClickChangePassword = () => {
    setShowpopup(true);
  };

  const onClickCancelChangePassword = () => {
    setShowpopup(false);
  };

  const onClickEditProfile = () => {
    history.push(RoutePaths.profile);
  }

  const [isHamburgerDisplay, setHamburgerDisplay] = useState(window.innerWidth <= TABLET_BREAK_POINT);

  const siderStyle: CSSProperties = isHamburgerDisplay ? {
    position: "fixed",
    top: '0px',
    height: '100%',
    zIndex: 1000
  } : {};

  const headerTextStyle: CSSProperties = isHamburgerDisplay ? {
    marginLeft: 40,
  } : {};

  const menuItemStyle = {
    fontSize: '15px',
    fontWeight: 500,
  };



  useEffect(() => {
    function handleResize() {
      setHamburgerDisplay(checkHamburgerDisplay())
    }

    window.addEventListener('resize', handleResize);
    return () => window.removeEventListener('resize', handleResize);
  }, []);


  const menu = (
    <Menu>
      <Menu.Item onClick={onClickEditProfile}>
        <FormattedMessage {...messages.editInfo}/>
      </Menu.Item>
      <Menu.Item onClick={onClickChangePassword}>
        <FormattedMessage {...messages.changePass}/>
      </Menu.Item>
      <Menu.Item onClick={handleLogout}>
        <span>
          <FormattedMessage {...messages.logout}/>
        </span>
      </Menu.Item>
    </Menu>
  );

  function havePermission(permissions: any[], tabId) {
    return _.findIndex(permissions, permission => permission.id === tabId) !== -1;
  }

  return (
    <MainLayoutWrapper>
      <AntdLayout>
        <Sider
          breakpoint="lg"
          width={220}
          className="site-layout-background sider-menu"
          collapsedWidth={0}
          collapsible
          trigger={isHamburgerDisplay ? <div></div> : <div></div>}
          zeroWidthTriggerStyle={
            {
              color: `white`,
              background: `${PRIMARY_COLOR}`,
              top: 0,
              height: `${HEADER_HEIGHT}`
            }
          }
          style={siderStyle}
        >
          <Menu
            mode="inline"
            defaultSelectedKeys={[activeMenuKey]}
            defaultOpenKeys={['sub1']}
            style={{ height: `100vh`, borderRight: 0, maxHeight: `100vh`, overflowY: `auto`, overflowX: `hidden` }}
            onClick={handleClick}
          >
            <Logo onClick={handleClickLogo}><img src={THINK_LOGO} height={60} /></Logo>
            {havePermission(Web, 11) && (<Menu.Item style={menuItemStyle} key={RoutePaths.dashboard} icon={activeMenuKey === RoutePaths.dashboard ? <DashBoardIconActive /> : <DashBoardIcon />}>
              <FormattedMessage {...messages.dashboard} />
            </Menu.Item>)}
            {havePermission(Web, 12) && <Menu.Item style={menuItemStyle} key={RoutePaths.restaurant} icon={activeMenuKey === RoutePaths.restaurant ? <StoreIconActive /> : <StoreIcon />} >
              <FormattedMessage {...messages.restaurant} />
            </Menu.Item>}
            {havePermission(Web, 13) && <Menu.Item style={menuItemStyle} key={RoutePaths.staff} icon={activeMenuKey === RoutePaths.staff ? <StaffIconActive /> : <StaffIcon />}>
              <FormattedMessage {...messages.staff} />
            </Menu.Item>}
            {havePermission(Web, 14) && <Menu.Item style={menuItemStyle} key={RoutePaths.service} icon={activeMenuKey === RoutePaths.service ? <ServiceIconActive /> : <ServiceIcon />}>
              <FormattedMessage {...messages.service} />
            </Menu.Item>}
            {havePermission(Web, 22) && <Menu.Item style={menuItemStyle} key={RoutePaths.menu} icon={activeMenuKey === RoutePaths.menu ? <MenuIconActive /> : <MenuIcon />}>
              <FormattedMessage {...messages.product} />
            </Menu.Item>}
            {havePermission(Web, 14) && <Menu.Item style={menuItemStyle} key={RoutePaths.storage} icon={activeMenuKey === RoutePaths.storage ? <StorageIconActive /> : <StorageIcon />}>
              <FormattedMessage {...messages.storage} />
            </Menu.Item>}
            {havePermission(Web, 15) && <Menu.Item style={menuItemStyle} key={RoutePaths.customer} icon={activeMenuKey === RoutePaths.customer ? <CustomerIconActive /> : <CustomerIcon />}>
              <FormattedMessage {...messages.customer} />
            </Menu.Item>}
            {havePermission(Web, 16) && <Menu.Item style={menuItemStyle} key={RoutePaths.promotion} icon={activeMenuKey === RoutePaths.promotion ? <GiftIconActive /> : <GiftIcon />}>
              <FormattedMessage {...messages.promotion} />
            </Menu.Item>}
            {havePermission(Web, 17) && <Menu.Item style={menuItemStyle} key={RoutePaths.payment} icon={activeMenuKey === RoutePaths.payment ? <CardIconActive /> : <CardIcon />}>
              <FormattedMessage {...messages.payment} />
            </Menu.Item>}
            {havePermission(Web, 17) && <Menu.Item style={menuItemStyle} key={RoutePaths.payroll} icon={activeMenuKey === RoutePaths.payroll ? <PayrollIconActive /> : <PayrollIcon />}>
              <FormattedMessage {...messages.payroll} />
            </Menu.Item>}
            {havePermission(Web, 18) && <Menu.Item style={menuItemStyle} key={RoutePaths.report} icon={activeMenuKey === RoutePaths.report ? <ReportIconActive /> : <ReportIcon />}>
              <FormattedMessage {...messages.report} />
            </Menu.Item>}
            {havePermission(Web, 19) && <Menu.Item style={menuItemStyle} key={RoutePaths.setting} icon={activeMenuKey === RoutePaths.setting ? <SettingIconActive /> : <SettingIcon />}>
              <FormattedMessage {...messages.setting} />
            </Menu.Item>}
          </Menu>
        </Sider>
        <AntdLayout>
          <Header className="main-layout-header">
            <div style={headerTextStyle}>{header}</div>
            <Dropdown overlay={menu} trigger={['click']}>
              <div className="user__name">
                <span> <FormattedMessage {...messages.hi}/>{`, `}</span><span className="user__name--name">{[firstName, lastName].join(' ')}</span>
                {avatar ? <img className="user__avatar" src={`${PUBLIC_API}/${avatar}`} /> : <Avatar className="user__avatar" initials={initials} name={[firstName, lastName].join(' ')} size="30" /> }
              </div>
            </Dropdown>
          </Header>
          <Content
            className="site-layout-background"
            style={{
              margin: 0,
              minHeight: 280,
            }}
          >
            {children}
          </Content>
          <ChangePasswordModal
            id={id}
            visible={showPopup}
            onOk={onClickCancelChangePassword}
            onCancel={onClickCancelChangePassword}
          />
        </AntdLayout>
      </AntdLayout>
    </MainLayoutWrapper >
  );
}


function checkHamburgerDisplay() {
  const { innerWidth } = window;
  if(innerWidth <= TABLET_BREAK_POINT){
    return true;
  }
  return  false
}
