import React, { useState, useEffect } from 'react';
import styled from 'styled-components';
import _ from 'lodash';
import { useSelector, useDispatch } from 'react-redux';
import { createStructuredSelector } from 'reselect';
import { makeSelectCategoryId } from '../Store/selectors';
import reducer from '../Store/reducer';
import saga from '../Store/saga';
import { setCategory } from '../Store/actions';
import { validNumber } from "utils/common";

import { useInjectReducer, useInjectSaga } from 'redux-injectors';

import MenuListHeader from './menuListHeader';
import MenuListMenuTab from './menuListMenuTab';
import { MenuListComboTab } from './menuListComboTab';

import { fetchAPI } from 'utils/fetchAPI';
import {
  MENUS_MENU_ITEMS_MERCHANT_URL,
} from 'containers/App/urls';
import { MenuModel } from './menuListMenuModels';
import { MenuTypes } from '../menuContants';
import MenuItem from 'antd/lib/menu/MenuItem';

const stateSelector = createStructuredSelector({
  params: makeSelectCategoryId(),
});

const MenuListWrapper = styled.div`
  font-family: Muli, sans-serif;
  .menu-list {
    padding: 20px;
  }
`;

function getCombo(data) {
  return data.reduce(
    (accumulate, category) =>
      accumulate.concat(
        _.remove(
          category.menuItems,
          (menuItem: any) =>
            menuItem.products.length > 1,
        ),
      ),
    [],
  );
}

export function MenuList(props) {
  const { type } = props;

  useInjectReducer({ key: 'menu', reducer: reducer });
  useInjectSaga({ key: 'menu', saga: saga });

  const { params } = useSelector(stateSelector);
  const dispatch = useDispatch();
  const [loading, setLoading] = useState(true);
  const [categoryId, setCategoryId] = useState<number | 0>();
  const [menuListOrigin, setMenuListOrigin] = useState<MenuModel[]>([]);
  const [menuList, setMenuList] = useState<MenuModel[]>([]);
  const [comboList, setComboList] = useState<MenuItem[]>([]);
  const handleCategoryChange = categoryId => {
    dispatch(setCategory(categoryId));
    if (!categoryId) {
      setMenuList(menuListOrigin);
    } else {
      setMenuList(
        menuListOrigin.filter(menu => menu.categoryId === categoryId),
      );
    }
  };

  const handleSearch = async obj => {
    if (obj.keyword) {
      setLoading(true);
      const urlreplace = `${MENUS_MENU_ITEMS_MERCHANT_URL}?type=1&searchText=${obj.keyword.trim()}`;
      const response = await fetchAPI({
        method: 'GET',
        url: urlreplace,
      });

      if (response) {
        setMenuList(
          params.categoryId
            ? response.filter(item => item.categoryId === params.categoryId)
            : response,
        );
        setComboList(getCombo(response));
        setLoading(false);
      }
    } else {
      getMenus();
    }
  };

  const getMenus = async () => {
    try {
      const response = await fetchAPI({
        method: 'GET',
        url: `${MENUS_MENU_ITEMS_MERCHANT_URL}?type=1`,
      });

      if (response) {
        const urlParams = new URLSearchParams(window.location.search);
        const categoryIdParams = urlParams.get('categoryId');
        const categoryId = categoryIdParams && validNumber(categoryIdParams) ? Number(categoryIdParams) : 0;
        setCategoryId(categoryId);
        setMenuList(
          categoryId
            ? response.filter(item => item.categoryId === categoryId)
            : response,
        );
        setMenuListOrigin(response);
        setComboList(getCombo(response));
      }
    } finally {
      setLoading(false);
    }
  };

  useEffect(() => {
    getMenus();
  }, []);

  return (
    <MenuListWrapper>
      <div className="menu-list">
        <MenuListHeader
          type={type}
          handleSearch={handleSearch}
          handleCategoryChange={handleCategoryChange}
          categoryId={categoryId}
        />
        {type === MenuTypes.COMBO ? (
          <MenuListComboTab
            type={type}
            loading={loading}
            comboList={comboList}
            getMenus={getMenus}
          />
        ) : (
          <MenuListMenuTab
            type={type}
            loading={loading}
            menuList={menuList}
            getMenus={getMenus}
          />
        )}
      </div>
    </MenuListWrapper>
  );
}
