import React from 'react';
import { Table, Typography, Button, Pagination } from 'antd';
import styled from 'styled-components';

import moment from 'moment';
import NumberFormat from 'react-number-format';
import { injectIntl } from 'react-intl';
import { useDispatch, useSelector } from 'react-redux';
import { createStructuredSelector } from 'reselect';
import { makePayrolls } from '../selectors';
import {
  getPayrolls,
  showPayrollModal,
  getPayroll,
  setSearch
} from '../actions';
import messages from 'translations/messages';
import IMG_INVENTORY_EDIT from 'images/inventory-edit.svg';
import { useCurrencyHook } from 'components/CustomHook/useCurrencyHook';

const ListWrapper = styled.div`
  width: 100%;
  box-shadow: 0 1px 3px 0 rgba(0, 0, 0, 0.1);
  background-color: #ffffff;
  font-family: Muli;
  margin: 1px 0;
  padding: 0;
  .ant-typography-ellipsis-single-line {
    font-size: 15px;
    font-weight: 600;
    overflow: unset;
    white-space: unset;
    @media screen and (max-width: 768px) {
      font-size: 12px;
    }
    @media screen and (max-width: 420px) {
      font-size: 13px;
    }
  }
  .ant-table-cell {
    @media screen and (max-width: 768px) {
      font-size: 10px;
    }
    @media screen and (max-width: 420px) {
      font-size: 9px;
    }
  }
  @media screen and (min-width: 1200px) {
    padding: 0 0;
  }
  .working-hours {
    &_header {
      h3 {
        font-size: 16px;
        font-weight: bold;
        color: #333333;
      }
      @media screen and (min-width: 1200px) {
        margin: 20px 20px 30px;
      }
    }
    &_main {
      .txt_green {
        color: #35a211;
        text-align: center;
      }
      .txt_red {
        color: #dc5454;
        text-align: center;
      }
      .wrapper-group-btn {
        display: flex;
        justify-content: space-around;
        flex-wrap: nowrap;
      }
      .wrapper-btn {
        width: 44px;
        height: 44px;
        .ant-btn-circle.ant-btn-lg {
          min-width: 100%;
        }
        .detail-btn{
          width: 100%;
          height: 100%;
          border-radius: 50%;
        }
      }
      .action-row,
      .remain-row {
        text-align: center;
      }
      .ant-table-thead > tr > th,
      .ant-table-tbody > tr > td,
      .ant-table tfoot > tr > th,
      .ant-table tfoot > tr > td {
        padding: 15px 10px;
      }
      @media screen and (max-width: 1200px) {
        .wrapper-btn {
          width: 35px;
          height: 35px;
          min-width: 0px;
          svg {
            font-size: 8px;
            height: 25px;
          }
        }
      }

      @media screen and (max-width: 765px) {
        .wrapper-btn {
          width: 30px;
          height: 30px;
          min-width: 0px;
          svg {
            font-size: 5px;
            height: 20px;
          }
        }
      }
      @media screen and (max-width: 576px) {
        .ant-table-thead > tr > th,
        .ant-table-tbody > tr > td,
        .ant-table tfoot > tr > th,
        .ant-table tfoot > tr > td {
          padding: 15px 5px;
        }
      }
    }
  }
`;

const stateSelector = createStructuredSelector({
  payrolls: makePayrolls(),
});

function showTotal(total, range) {
  return `${range.join('-')} of ${total}`;
}

export const List = injectIntl(props => {
  const { intl, search } = props;
  const { payrolls } = useSelector(stateSelector);
  const currencyMemoi = useCurrencyHook();

  const dispatch = useDispatch();
  const handleClick = key => {
    dispatch(getPayroll(key.id));
    dispatch(showPayrollModal());
  };

  const handleClickPageNumber = (page, pageSize) => {
    const newSearch = { ...search, page: page, limit:  pageSize};
    dispatch(getPayrolls(newSearch));
    dispatch(setSearch(newSearch));
  };

  const handleClickChangeSize = (current, size) => {
    const newSearch = { ...search, limit: size, page: 1 };
    dispatch(getPayrolls(newSearch));
    dispatch(setSearch(newSearch));
  };

  const config = {
    pagination: {
      pageSizeOptions: ['10', '30', '40'],
      showSizeChanger: true,
      total: payrolls.count,
      size: 'small' as any,
      showTotal: (total, range) => showTotal(total, range),
      onChange: (page, pageSize) => handleClickPageNumber(page, pageSize),
      onShowSizeChange: (current, size) => handleClickChangeSize(current, size),
    },
  };
  const columns = [
    {
      title: (
        <Typography.Text ellipsis>
           {intl.formatMessage(messages.staffPayrollName)}
        </Typography.Text>
      ),
      dataIndex: 'fullName',
      key: 'fullName',
      width: '18%',
      render: (text, row, index) => {
        return {
          children: <span>{text}</span>,
          props: {
            colSpan: 1,
          },
        };
      },
    },
    {
      title: (
        <Typography.Text ellipsis>
          {intl.formatMessage(messages.mst)}
        </Typography.Text>
      ),
      dataIndex: ['user', 'mst'],
      responsive: ['xxl', 'xl', 'lg', 'md'] as any,
      key: 'user.mst',
      width: '8%',
      render: (text, row, index) => {
        return {
          children: <span>{text}</span>,
          props: {
            colSpan: 1,
          },
        };
      },
    },
    {
      title: (
        <Typography.Text ellipsis>
          {intl.formatMessage(messages.titleStore)}
        </Typography.Text>
      ),
      dataIndex: ['user', 'restaurant', 'name'],
      key: 'storeNm',
      width: '19%',
      render: (text, row, index) => {
        return {
          children: <span>{text}</span>,
          props: {
            colSpan: 1,
          },
        };
      },
    },
    {
      title: (
        <Typography.Text ellipsis>
          {intl.formatMessage(messages.periodPayroll)}
        </Typography.Text>
      ),
      dataIndex: 'payPeriod',
      responsive: ['xxl', 'xl', 'lg', 'md', 'sm'] as any,
      key: 'payPeriod',
      width: '17%',
      render: (text, row, index) => {
        return {
          children: <span>{text}</span>,
          props: {
            colSpan: 1,
          },
        };
      },
    },
    {
      title: (
        <Typography.Text ellipsis>
         {intl.formatMessage(messages.workedHour)}
        </Typography.Text>
      ),
      dataIndex: 'totalHours',
      key: 'totalHours',
      width: '12%',
      render: (text, row, index) => {
        return {
          children: (
            <span>
              <NumberFormat
                value={text ? text : 'N/A'}
                displayType={'text'}
                thousandSeparator={false}
                renderText={value => value}
              />
            </span>
          ),
        };
      },
    },
    {
      title: (
        <Typography.Text ellipsis>
          {intl.formatMessage(messages.earnings)}
        </Typography.Text>
      ),
      dataIndex: 'subAmount',
      responsive: ['xxl', 'xl', 'lg', 'md', 'sm'] as any,
      key: 'subAmount',
      width: '10%',
      className: 'remain-row',
      render: (text, row, index) => {
        return {
          children: (
            <span>
              <NumberFormat
                value={text}
                displayType={'text'}
                thousandSeparator={true}
                renderText={value => `${value} ${currencyMemoi}`}
              />
            </span>
          ),
        };
      },
    },
    {
      title: (
        <Typography.Text ellipsis>
          {intl.formatMessage(messages.netPay)}
        </Typography.Text>
      ),
      dataIndex: 'totalAmount',
      key: 'totalAmount',
      width: '10%',
      className: 'action-row',
      render: (text, row, index) => {
        return {
          children: (
            <span>
              <NumberFormat
                value={text}
                displayType={'text'}
                thousandSeparator={true}
                renderText={value => `${value} ${currencyMemoi}`}
              />
            </span>
          ),
        };
      },
    },
    {
      title: <></>,
      dataIndex: 'action',
      key: 'action',
      width: '6%',
      className: 'action-row',
      render: (text, row, index) => (
        <span className="wrapper-group-btn">
          <div className="wrapper-btn">
            <img
              src={IMG_INVENTORY_EDIT}
              alt=""
              onClick={e => handleClick(row)}
              className="detail-btn list-pro"
            />
          </div>
        </span>
      ),
    },
  ];
  return (
    <ListWrapper>
      <div className="working-hours_main">
        <Table
          dataSource={payrolls.list}
          columns={columns}
          rowKey={record => record.key}
          pagination={config.pagination}
        />
      </div>
    </ListWrapper>
  );
});
