import React, { useState, useEffect } from 'react';
import { Modal, Button, Form, Spin, Checkbox } from 'antd';
import { useParams } from 'react-router-dom';
import { fetchAPI, JsonToFormData } from 'utils/fetchAPI';
import { AREA, STAFF, ASSIGN_STAFF, SECTIONS } from 'containers/App/urls';
import { StaffModel } from 'containers/StaffList/types';
import { FormattedMessage, injectIntl } from 'react-intl';
import messages from 'translations/messages';
import styled from 'styled-components';
import { CheckboxChangeEvent } from 'antd/lib/checkbox';
import _ from 'lodash';
const CheckBoxWrapper = styled.div`
  width: 50%;
  .header {
    font-size: 15px;
    font-weight: 700;
    padding-bottom: 10px;
  }
  .checkbox-row {
    padding-bottom: 10px;
  }
`;
interface Props {
  visible: boolean;
  onCancel: () => void;
  callBackRestaurantStaff: () => void;
  initialRestaurantStaff: StaffModel;
  staffList: StaffModel[];
  intl: any;
}

export const RestaurantStaffModal = (props: Props) => {
  const {
    visible,
    onCancel,
    callBackRestaurantStaff,
    initialRestaurantStaff,
    intl,
  } = props;
  const [form] = Form.useForm();
  // @ts-ignore
  const { id: restaurantId } = useParams();
  const [loading, setLoading] = useState(true);
  const [groupList, setGroupList] = useState<any>([]);
  const [checkedGroups, setCheckedGroups] = useState<any>([]);

  const getGroupList = async () => {
    try {
      const response = await fetchAPI({
        method: 'GET',
        url: `${SECTIONS}?restaurantId=${restaurantId}`,
      });
      if (response) {
        setGroupList(response);
      }
    } finally {
      setLoading(false);
    }
  };
  const handleOnCancel = () => {
    onCancel();
    form.resetFields();
    setCheckedGroups([]);
  };

  const handleCheckedGroup = (e: CheckboxChangeEvent, id: number) => {
    const isChecked = e.target.checked;
    let newCheckedGroups;
    if (isChecked) {
      newCheckedGroups = [...checkedGroups, id];
    } else {
      newCheckedGroups = checkedGroups.filter(checkedId => checkedId !== id);
    }
    setCheckedGroups(newCheckedGroups);
  };

  const onFinish = async value => {
    const modifiedValues = {
      restaurantId: restaurantId,
      sectionIds: checkedGroups,
      userIds: [initialRestaurantStaff.id],
    };
    const response = await fetchAPI({
      method: 'PUT',
      url: ASSIGN_STAFF,
      data: modifiedValues,
    });
    if (response) {
      callBackRestaurantStaff();
      onCancel();
      form.resetFields();
      setCheckedGroups([]);
    }
  };

  useEffect(() => {
    getGroupList();
    setCheckedGroups(
      initialRestaurantStaff.sectionIds
        ? initialRestaurantStaff.sectionIds.map(section => parseInt(section))
        : [],
    );
  }, [visible]);
  return (
    <Form form={form} name="restaurant-staff" onFinish={onFinish}>
      <Modal
        visible={visible}
        width={380}
        onCancel={handleOnCancel}
        centered
        footer={[
          <Button
            type="primary"
            disabled={groupList.length > 0 ? false : true}
            htmlType="submit"
            className="w-100"
            onClick={() => {
              form
                .validateFields()
                .then(values => {
                  onFinish(values);
                })
                .catch(info => {});
            }}
          >
            <FormattedMessage {...messages.save} />
          </Button>,
        ]}
      >
        {loading ? (
          <Spin />
        ) : (
          <>
            <h2 className="text-center">
              <FormattedMessage {...messages.assignment} />
            </h2>
            <div className="d-flex space-between">
              {groupList.length > 0 ? (
                <CheckBoxWrapper>
                  <div className="header">
                    <FormattedMessage {...messages.group} />
                  </div>
                  {groupList.length &&
                    groupList.map(group => (
                      <div className="checkbox-row">
                        <Checkbox
                          onChange={e => handleCheckedGroup(e, group.id)}
                          checked={checkedGroups.includes(group.id)}
                        >
                          {group.name}
                        </Checkbox>
                      </div>
                    ))}
                </CheckBoxWrapper>
              ) : (
                <FormattedMessage {...messages.noGroup} />
              )}
            </div>
          </>
        )}
      </Modal>
    </Form>
  );
};
export default injectIntl(RestaurantStaffModal);
