import React, { useState, useEffect, Children } from 'react';
import { Modal, Button, Form, Spin, Select, Checkbox } from 'antd';
import { InputWrapper } from 'styles/custom-styled-components';
import { useParams } from 'react-router-dom';
import { fetchAPI } from 'utils/fetchAPI';
import { AREA, STAFF, ASSIGN_STAFF, SECTIONS } from 'containers/App/urls';
import { StaffModel } from 'containers/StaffList/types';
import { FormattedMessage, injectIntl } from 'react-intl';
import messages from 'translations/messages';
import styled from 'styled-components';
import { CheckboxChangeEvent } from 'antd/lib/checkbox';
import _ from 'lodash';
import FloatLabel from 'components/FormatInput/FloatLabel';
const CheckBoxWrapper = styled.div`
  width: 50%;
  .header {
    font-size: 14px;
    font-weight: 700;
    padding-bottom: 10px;
  }
  .checkbox-row {
    padding-bottom: 10px;
  }
`;
interface Props {
  visible: boolean;
  onCancel: () => void;
  getRestaurantStaffs: (id) => void;
  initialRestaurantStaff?: StaffModel;
  staffList: StaffModel[];
  intl: any;
}

const RestaurantStaffModal = (props: Props) => {
  const {
    visible,
    onCancel,
    getRestaurantStaffs,
    initialRestaurantStaff,
    staffList: currentStaffList,
    intl,
  } = props;
  const [form] = Form.useForm();
  // @ts-ignore
  const { id: restaurantId } = useParams();
  const [loading, setLoading] = useState(true);
  const [groupList, setGroupList] = useState<any>([]);
  const [areaList, setAreaList] = useState<any>([]);
  const [staffList, setStaffList] = useState<StaffModel[]>([]);
  const [selectedStaffs, setSelectedStaffs] = useState<Array<StaffModel>>([]);
  const [checkedAreas, setCheckedAreas] = useState<Array<number>>([]);
  const [checkedGroups, setCheckedGroups] = useState<Array<number>>([]);

  const getAllStaffUser = async () => {
    try {
      const response = await fetchAPI({
        method: 'GET',
        url: `${STAFF}?isFree=true&page=1&limit=1000`,
      });
      if (response) {
        const filteredStaffList = response.list.filter(o => {
          const isInCurrentStaffList = currentStaffList.find(
            c => c.id === o.id,
          );
          if (isInCurrentStaffList) {
            return false;
          }
          return true;
        });
        setStaffList(filteredStaffList);
      }
    } finally {
      setLoading(false);
    }
  };

  const getAreaList = async () => {
    try {
      const response = await fetchAPI({
        method: 'GET',
        url: `${AREA}?restaurantId=${restaurantId}`,
      });
      if (response) {
        setAreaList(response);
      }
    } finally {
      setLoading(false);
    }
  };

  const getGroupList = async () => {
    try {
      const response = await fetchAPI({
        method: 'GET',
        url: `${SECTIONS}?restaurantId=${restaurantId}`,
      });
      if (response) {
        setGroupList(response);
      }
    } finally {
      setLoading(false);
    }
  };

  const handleSelectStaffs = (staffIds: Array<number>) => {
    const newSelectedStaffs = staffList.filter(staff =>
      staffIds.includes(staff.id),
    );
    setSelectedStaffs(newSelectedStaffs);
  };

  const handleOnCancel = () => {
    onCancel();
    form.resetFields();
    setCheckedAreas([]);
    setCheckedGroups([]);
    setSelectedStaffs([]);
  };

  const handleCheckedArea = (e: CheckboxChangeEvent, id: number) => {
    const isChecked = e.target.checked;
    let newCheckedAreas;
    if (isChecked) {
      newCheckedAreas = [...checkedAreas, id];
    } else {
      newCheckedAreas = checkedAreas.filter(checkedId => checkedId !== id);
    }
    setCheckedAreas(newCheckedAreas);
  };
  const handleCheckedGroup = (e: CheckboxChangeEvent, id: number) => {
    const isChecked = e.target.checked;
    let newCheckedGroups;
    if (isChecked) {
      newCheckedGroups = [...checkedGroups, id];
    } else {
      newCheckedGroups = checkedGroups.filter(checkedId => checkedId !== id);
    }
    setCheckedGroups(newCheckedGroups);
  };

  const onFinish = async value => {
    const modifiedValues = {
      restaurantId: restaurantId,
      sectionIds: checkedGroups,
      userIds: value.staffId,
      areaIds: checkedAreas,
    };
    const response = await fetchAPI({
      method: 'PUT',
      url: ASSIGN_STAFF,
      data: modifiedValues,
    });
    if (response) {
      getRestaurantStaffs(restaurantId);
      onCancel();
      form.resetFields();
      setCheckedAreas([]);
      setCheckedGroups([]);
      setSelectedStaffs([]);
    }
  };

  useEffect(() => {
    getAllStaffUser();
    getAreaList();
    getGroupList();
  }, [currentStaffList, visible]);
  const newGroupList = groupList.map(group => (
    <div className="checkbox-row">
      <Checkbox
        onChange={e => handleCheckedGroup(e, group.id)}
        checked={checkedGroups.includes(group.id)}
      >
        {group.name}
      </Checkbox>
    </div>
  ));
  return (
    <Form form={form} name="restaurant-staff" onFinish={onFinish}>
      <Modal
        visible={visible}
        width={380}
        onCancel={handleOnCancel}
        centered
        footer={[
          <Button
            type="primary"
            htmlType="submit"
            className="w-100"
            onClick={() => {
              form
                .validateFields()
                .then(values => {
                  onFinish(values);
                })
                .catch(info => {});
            }}
          >
            <FormattedMessage {...messages.save} />
          </Button>,
        ]}
      >
        {loading ? (
          <Spin />
        ) : (
          <>
            <h2 className="text-center">
              <FormattedMessage {...messages.addStaff} />
            </h2>
            <InputWrapper>
              <Form.Item
                name="staffId"
                rules={[
                  {
                    required: true,
                    message: <FormattedMessage {...messages.requiredStaff} />,
                  },
                ]}
                className="mutil-select"
              >
                <FloatLabel
                  className="padding-mutil-select"
                  label={intl.formatMessage(messages.staffPlaceHolder)}
                  showSearch
                  mode="multiple"
                  filterOption={(inputValue, option) =>
                    option.props.children
                      .toUpperCase()
                      .indexOf(inputValue.toUpperCase()) !== -1
                  }
                  maxTagCount={2}
                  typeInput="selectMutil"
                  items={
                    staffList.length
                      ? staffList.map(staff => {
                          const name = `${staff.firstName || ''} ${
                            staff.lastName || ''
                          } (${_.join(staff.roleNames, ', ')})`;
                          return { id: staff.id, name: name };
                        })
                      : []
                  }
                />
              </Form.Item>
            </InputWrapper>

            <div className="d-flex space-between">
              <CheckBoxWrapper>
                {newGroupList.length > 0 && <>{newGroupList}</>}
              </CheckBoxWrapper>
            </div>
          </>
        )}
      </Modal>
    </Form>
  );
};
export default injectIntl(RestaurantStaffModal);
