import React, {useState} from 'react';
import { Modal, Button, Form, Input, Select } from 'antd';
import { InputWrapper } from 'styles/custom-styled-components';
import { fetchAPI } from 'utils/fetchAPI';
import { TABLE, TABLES } from 'containers/App/urls';
import { DragItemModel } from '../../../../../Store/types';
import {
  makeSelectCurrentFloorId,
  getDragItemsSuccess,
  updateDragItem,
  createDragItemSuccess,
  makeSelectDragItems
} from 'containers/Restaurant/Store';
import { createStructuredSelector } from 'reselect';
import { useSelector, useDispatch } from 'react-redux';
import { FormattedMessage } from 'react-intl';
import messages from 'translations/messages';
import FloatLabel from 'components/FormatInput/FloatLabel'

interface Props {
  visible: boolean;
  onCancel: () => void;
  initialDragItem?: DragItemModel;
  currentFloorId?: number;
}

const stateSelector = createStructuredSelector({
  currentFloorId: makeSelectCurrentFloorId(),
  dragItems: makeSelectDragItems(),
});

export const AreaModal = (props: Props) => {
  const { currentFloorId, dragItems } = useSelector(stateSelector);
  const dispatch = useDispatch();
  const [loading, setLoading] = useState(false);
  const { visible, onCancel, initialDragItem } = props;
  const [form] = Form.useForm();

  const handleOnCancel = () => {
    onCancel();
    form.resetFields();
  }

  const onFinish = async (values) => {
    try {
      setLoading(true);
      const modifiedValues = { ...values, floorId: currentFloorId!, shapeId: 1 }; // type = 1: table, table = 2: area
      const response = await fetchAPI({
        method: 'POST',
        data: modifiedValues,
        url: TABLE
      });
      if (response) {
        const dragItemIndex = dragItems[dragItems.findIndex(item => item.id === initialDragItem?.id)];
        const dragItemResponse = { ...response[0], height: dragItemIndex.height, width: dragItemIndex.width,positionX: dragItemIndex.width, positionY: dragItemIndex.positionY, rotationDegree: dragItemIndex.rotationDegree }
        dispatch(updateDragItem(dragItemResponse));
        onCancel();
        form.resetFields();
      }
    } finally {
      setLoading(false);
    }
  }
  return (
    <Form
      form={form}
      name="horizontal_login"
    >
      <Modal
        visible={visible}
        centered
        width={400}
        onCancel={handleOnCancel}
        footer={
          [<Button
            type="primary"
            htmlType="submit"
            className="w-100"
            loading={loading}
            onClick={() => {
              form
                .validateFields()
                .then(values => {
                  onFinish(values);
                })
                .catch(info => {
                  console.log('Validate Failed:', info);
                });
            }}><FormattedMessage {...messages.save} /></Button>]
        }
      >
        <h2 className="text-center"><FormattedMessage {...messages.createArea} /></h2>

        <FormattedMessage {...messages.areaName}>
          {
            areaName =>
              <InputWrapper >
                <Form.Item
                  name="name"
                  rules={[{ required: true, message: <FormattedMessage {...messages.requiredInput} /> }]}
                >

                  <FloatLabel allowClear label={`${areaName}*`}/>
                </Form.Item>
              </InputWrapper>
          }
        </FormattedMessage>
        <InputWrapper >
          <Form.Item
            name="type"
            rules={[{ required: true, message: <FormattedMessage {...messages.requiredInput} /> }]}
          >
            <FloatLabel label={<FormattedMessage {...messages.areaType} />}
              items={[{id: 2, name: <FormattedMessage {...messages.kitchen} />},
                {id: 3, name: <FormattedMessage {...messages.bartender} />},
                {id: 4, name: <FormattedMessage {...messages.takeAwayArea} />},
                {id: 5, name: <FormattedMessage {...messages.other} />},
              ]}
              typeInput="selectMutil"
            />
          </Form.Item>
        </InputWrapper>
      </Modal>
    </Form >
  )
}
