import React, { useState, useEffect } from 'react';
import { Modal, Button, Form, Input, Select } from 'antd';
import { InputWrapper } from 'styles/custom-styled-components';
import { fetchAPI } from 'utils/fetchAPI';
import { TABLES, TABLE, SECTIONS } from 'containers/App/urls';
import { DragItemModel, GroupModel } from '../../../../../Store/types';
import { useDispatch, useSelector } from 'react-redux';
import {
  createDragItemSuccess,
  getDragItemsSuccess,
  makeSelectCurrentFloorId,
} from 'containers/Restaurant/Store';
import { createStructuredSelector } from 'reselect';
import messages from 'translations/messages';
import { FormattedMessage } from 'react-intl';
import { validateRequiredSpace } from '../../../../../../../utils/validation';
import FloatLabel from 'components/FormatInput/FloatLabel';

interface Props {
  visible: boolean;
  onCancel: () => void;
  initialDragItem?: DragItemModel;
  shapeId: number;
}

const stateSelector = createStructuredSelector({
  currentFloorId: makeSelectCurrentFloorId(),
});

export const TableModal = (props: Props) => {
  const { visible, onCancel, initialDragItem, shapeId } = props;
  if (!visible) {
    return <span />;
  }
  const { currentFloorId } = useSelector(stateSelector);
  const dispatch = useDispatch();

  const [groupList, setGroupList] = useState<GroupModel[]>([]);
  const [loading, setLoading] = useState(false);
  const [form] = Form.useForm();
  const onFinish = async values => {
    try {
      for (var key in values) {
        if (typeof values[key] === 'string') {
          values[key] = values[key].trim();
        }
      }
      setLoading(true);
      const modifiedValues = {
        ...initialDragItem,
        ...values,
        floorId: currentFloorId!,
        numberOfSeat: 1,
      };
      const response = await fetchAPI({
        method: 'POST',
        data: modifiedValues,
        url: TABLE,
      });
      if (response) {
        dispatch(createDragItemSuccess(response));
        onCancel();
        form.resetFields();
      }
    } finally {
      setLoading(false);
    }
  };
  const getGroupList = async currentFloorId => {
    const response = await fetchAPI({
      method: 'GET',
      url: `${SECTIONS}?floorId=${currentFloorId}`,
    });
    if (response) {
      setGroupList(response);
    }
  };
  const handleOnCancel = () => {
    onCancel();
    form.resetFields();
  };
  useEffect(() => {
    getGroupList(currentFloorId);
  }, [currentFloorId]);

  if (loading) {
    return <span />;
  }
  return (
    <Form form={form} name="horizontal_login" initialValues={initialDragItem}>
      <Modal
        visible={visible}
        width={400}
        centered
        onCancel={handleOnCancel}
        maskClosable={true}
        footer={[
          <Button
            type="primary"
            htmlType="submit"
            className="w-100"
            loading={loading}
            onClick={() => {
              form
                .validateFields()
                .then(values => {
                  onFinish(values);
                })
                .catch(info => {
                  console.log('Validate Failed:', info);
                });
            }}
          >
            <FormattedMessage {...messages.save} />
          </Button>,
        ]}
      >
        <h2 className="text-center">
          {initialDragItem?.shapeId === 2 ? (
            <FormattedMessage {...messages.createTable} />
          ) : (
            <FormattedMessage {...messages.createChair} />
          )}
        </h2>
        <FormattedMessage {...messages.tableName}>
          {tableName => (
            <InputWrapper>
              <Form.Item name="name" rules={[validateRequiredSpace]}>
                <FloatLabel allowClear 
                  label=
                    {initialDragItem?.shapeId === 2 ?`${tableName}*` : <FormattedMessage {...messages.chairName} />}
                />
              </Form.Item>
            </InputWrapper>
          )}
        </FormattedMessage>
        <InputWrapper>
          <Form.Item name="sectionId">
            <FloatLabel
              className="staff-filters__restaurant"
              label={<FormattedMessage {...messages.selectGroup} />}
              items={groupList}
              typeInput="selectMutil"
            />
          </Form.Item>
        </InputWrapper>
      </Modal>
    </Form>
  );
};
