import React, { useState, useEffect } from 'react';
import styled from 'styled-components';
import { Button, Popconfirm, notification } from 'antd';
import { fetchAPI, replaceUrl } from 'utils/fetchAPI';
import { SECTIONS, SECTION_DETAIL } from 'containers/App/urls';
import { GroupModel } from 'containers/Restaurant/Store/types';
import { GroupModal } from './Groups-children';
import { RemoveIcon, PlusCharacter } from '../../../../../../styles/icon-components';
import { useHistory } from 'react-router-dom';
import { PRIMARY_COLOR } from 'styles/constants';
import { createStructuredSelector } from 'reselect';
import { makeSelectCurrentFloorId } from 'containers/Restaurant/Store';
import { useSelector } from 'react-redux';
import { FormattedMessage } from 'react-intl';
import messages from 'translations/messages';

const GroupWrapper = styled.div`
  cursor: pointer;
  padding-top: 10px;
  .header {
    padding: 5px 10px;
    display: flex;
    justify-content: space-between;
    align-items: center;
  }
  .group {
    padding: 10px;
    background: #F2F2F2;
    border-bottom: solid 1px white;
    display: flex;
    align-items: center;
    justify-content: space-between;
  }
  .active {
    color: ${PRIMARY_COLOR};
  }
`;

const stateSelector = createStructuredSelector({
  currentFloorId: makeSelectCurrentFloorId(),
});

export const Groups = (props) => {
  const { currentFloorId } = useSelector(stateSelector);
  const history = useHistory();
  const [openGroupModal, setOpenGroupModal] = useState(false);
  const [groupList, setGroupList] = useState<GroupModel[]>([]);
  const [loading, setLoading] = useState(false);
  const [selectedGroupId, setSelectedGroupId] = useState();
  const getGroupList = async (currentFloorId) => {
    try {
      const response = await fetchAPI({
        method: 'GET',
        url: `${SECTIONS}?floorId=${currentFloorId}`,
      });
      if (response) {
        setGroupList(response);
      }
    } finally {
      setLoading(false);
    }
  }
  const deleteGroup = async (floorId) => {
    const response = await fetchAPI({
      method: 'DELETE',
      url: replaceUrl(SECTION_DETAIL, { id: floorId })
    })
    if (response) {
      getGroupList(currentFloorId);
    }
  }
  const handleOnSelectGroup = (groupId) => {
    const urlSearchParams = new URLSearchParams(history.location.search);
    urlSearchParams.set('selectedGroupId', groupId);
    history.push({ search: urlSearchParams.toString() });
    setSelectedGroupId(groupId);
  }

  useEffect(() => {
    getGroupList(currentFloorId);
  }, [currentFloorId]);

  return (
    <GroupWrapper>
      <div className="header">
        <span><FormattedMessage {...messages.group} /></span>
        <Button type="primary" shape="circle" onClick={() => setOpenGroupModal(true)}><PlusCharacter/></Button>
      </div>
      {
        loading ? <div><FormattedMessage {...messages.loading} /></div>
          : groupList.map(group =>
            <div
              key={group.id}
              className={`group ${group.id === selectedGroupId ? 'active' : ''}`}
              onClick={() => handleOnSelectGroup(group.id)}
            >
              {group.name}
              <Popconfirm
                title={<FormattedMessage {...messages.deleteConfirmContent} values={{name: group.name}}/>}
                onConfirm={() => deleteGroup(group.id!)}
                okText={<FormattedMessage {...messages.deleteBtn} />}
                cancelText={<FormattedMessage {...messages.cancelBtn} />}
              >
                <a href="#">
                  <RemoveIcon />
                </a>
              </Popconfirm>
            </div>)
      }
      <GroupModal
        getGroups={() => getGroupList(currentFloorId)}
        onCancel={() => setOpenGroupModal(false)}
        visible={openGroupModal}
        initialGroup={{}}
        currentFloorId={currentFloorId}
      />
    </GroupWrapper>
  );
}
