import React, { useState, useEffect } from 'react';
import { DragItemModel } from 'containers/Restaurant/Store/types';
import {
  Table
} from 'styles/table-area-components';
import TableEmpty from 'images/table_empty.svg';
import TableDone from 'images/table_done.svg';
import ChairEmpty from 'images/chair_empty.png';
import ChairDone from 'images/chair_done.png';
import { PRIMARY_COLOR } from 'styles/constants';
import { Popover, List, notification } from 'antd';
import styled from 'styled-components';
import { fetchAPI, replaceUrl } from 'utils/fetchAPI';
import { TABLES, TABLE_ADD_TO_SECTION, TABLE_DELETE_FROM_SECTION, TABLE_DETAIL } from 'containers/App/urls';
import { getSearchParamValueByParams } from 'utils/searchParams';
import { useHistory } from 'react-router-dom';
import { useDispatch } from 'react-redux';
import { getDragItemsSuccess, updateDragItem, deleteDragItem } from 'containers/Restaurant/Store';
import { RemoveIcon, PlusIcon, EditIcon } from 'styles/icon-components';
import { FormattedMessage, injectIntl } from 'react-intl';
import messages from 'translations/messages';
import { TableEditModal } from './tableEditModal';


export const DragItemName = styled.span`
  position: absolute;
  margin-left: auto;
  margin-right: auto;
  top: 45%;
  left: 0;
  right: 0;
  text-align: center;
  color: white;
  font-size: 14px !important;
  overflow: hidden;
  display: -webkit-box;
  color: ${(props : { selected?: boolean}) => props.selected ? '#ffffff' : '#1A1A1A' };
  -webkit-line-clamp: 1;
  /* autoprefixer: off */
  -webkit-box-orient: vertical;
  /* autoprefixer: on */
`;

const RenderTable = injectIntl((props: Props) => {
  const { item, intl } = props;
  const { name } = item;
  const [ editModal, setEditModal ] = useState(false);
  const [ isVisible, setVisible ] = useState(false);
  const history = useHistory();
  const dispatch = useDispatch();
  const handleCancel = () => {
    setEditModal(false);
  };
  const handleAddToCurrentGroup = async () => {
    const selectedGroupId = getSearchParamValueByParams(history.location.search, 'selectedGroupId');
    if (!selectedGroupId) {
      return notification.info({ message: intl.formatMessage(messages.askChooseGroup)});
    }
    const response = await fetchAPI({
      method: 'POST',
      data: { sectionId: selectedGroupId },
      url: replaceUrl(TABLE_ADD_TO_SECTION, { id: item.id})
    });
    if (response) {
      const dragItemResponse = { ...item, sectionId: parseInt(selectedGroupId) };
      dispatch(updateDragItem(dragItemResponse));
    }
    setVisible(false);
  }
  const handleRemoveFromCurrentGroup = async () => {
    const selectedGroupId = getSearchParamValueByParams(history.location.search, 'selectedGroupId');
    if (!selectedGroupId) {
      return notification.info({ message: intl.formatMessage(messages.addToCurrentGroup) });
    }
    const response = await fetchAPI({
      method: 'DELETE',
      data: { sectionId: selectedGroupId },
      url: replaceUrl(TABLE_DELETE_FROM_SECTION, { id: item.id})
    });
    if (response) {
      const dragItemResponse = { ...item };
      delete dragItemResponse.sectionId;
      dispatch(updateDragItem(dragItemResponse));
    }
    setVisible(false);
  }
  const handleDeleteTable = async () => {
    const res = await fetchAPI({
      method: 'DELETE',
      url: replaceUrl(TABLE_DETAIL, { id: item.id })
    });
    if (res) {
      dispatch(deleteDragItem(item.id));
    }
    setVisible(false);
  }

  const handleEditTable = () => {
    setEditModal(true);
    setVisible(false);
  }
  const handleClickChange = () => {
    setVisible(!isVisible);
  }
  const content = (
    <List>
      <List.Item onClick={handleAddToCurrentGroup} ><PlusIcon /><span className="list__item-title"><FormattedMessage {...messages.addToCurrentGroup} /></span></List.Item>
      <List.Item onClick={handleRemoveFromCurrentGroup}><RemoveIcon color='currentColor' /><span className="list__item-title"><FormattedMessage {...messages.removeFromCurrentGroup} /></span></List.Item>
      <List.Item onClick={handleDeleteTable}><RemoveIcon color='currentColor' /><span className="list__item-title"><FormattedMessage {...messages.deleteTable} /></span></List.Item>
      <List.Item onClick={handleEditTable}><EditIcon /><span className="list__item-title"><FormattedMessage {...messages.editTableName} /></span></List.Item>
    </List>
  );
  return (
    <>
      <Popover content={content} trigger={['click']} visible={isVisible} onVisibleChange={handleClickChange}>
        <DragItemName style={{ transform: `rotate(-${props.rotationDegree}deg`, cursor: 'pointer' }}>{name}</DragItemName>
      </Popover>
      {editModal && <TableEditModal visible={editModal} onCancel={handleCancel} initialDragItem={item}/> }
      {renderImage(props)}
    </>
  )
})

const renderImage = (props: Props) => {
  const { isSelected, item, width, height } = props;
  const { shapeId, numberOfSeat } = item;
  const imagePair = shapeId === 2 ? [TableDone, TableEmpty ] : [ ChairDone, ChairEmpty];
  return isSelected ? <img src={imagePair[0]} width={width} height={height} /> : <img src={imagePair[1]} width={width} height={height} />;
}

const renderArea = (props: Props) => {
  const { isSelected, item, width, height, rotationDegree } = props;
  const color = isSelected ? '#4A4A4A' : '#9FA3A7';
  const { name } = item;
  return (
    <>
      <DragItemName style={{ transform: `rotate(-${props.rotationDegree}deg`}}>{name}</DragItemName>
      <img src={Table} width={width} height={height}/>
    </>
  )
}

interface Props {
  isSelected: boolean;
  item: DragItemModel;
  width: number;
  height: number;
  rotationDegree: number;
  intl?: any;
}

export const ImageItem = (props: Props) => {

  const { item } = props;
  const { type } = item;
  if (+type === 1) {
    <RenderTable {...props} />;
  } else {
    return renderArea(props);
  }
  return <RenderTable {...props} />;
}
