import React, { useState, useEffect } from 'react';
import { Modal, Button, Form, Input, notification, Spin, Select } from 'antd';
import { InputWrapper } from 'styles/custom-styled-components';
import { fetchAPI, replaceUrl } from 'utils/fetchAPI';
import { TABLES, TABLE_DETAIL, SECTIONS } from 'containers/App/urls';
import { DragItemModel, GroupModel } from '../../../../../Store/types';
import { useDispatch, useSelector } from 'react-redux';
import {
  getDragItemsSuccess,
  makeSelectCurrentFloorId,
  makeSelectDragItems,
  updateDragItem,
} from 'containers/Restaurant/Store';
import { createStructuredSelector } from 'reselect';
import { validateRequiredSpace } from '../../../../../../../utils/validation';
import messages from 'translations/messages';
import { FormattedMessage } from 'react-intl';
import reducer from '../../../../../Store/reducer';
import { useInjectReducer } from 'redux-injectors';
import FloatLabel from 'components/FormatInput/FloatLabel';

interface Props {
  visible: boolean;
  onCancel: () => void;
  initialDragItem?: DragItemModel;
  shapeId?: number;
}

const stateSelector = createStructuredSelector({
  currentFloorId: makeSelectCurrentFloorId(),
  dragItems: makeSelectDragItems(),
});

export const TableEditModal = (props: Props) => {
  const { visible, onCancel, initialDragItem, shapeId } = props;
  if (!visible) {
    return <span />;
  }
  const { currentFloorId, dragItems } = useSelector(stateSelector);
  useInjectReducer({ key: 'layout', reducer: reducer });
  const dispatch = useDispatch();
  const [loading, setLoading] = useState(false);
  const [groupList, setGroupList] = useState<GroupModel[]>([]);
  const [form] = Form.useForm();
  const onFinish = async values => {
    try {
      for (var key in values) {
        if (typeof values[key] === 'string') {
          values[key] = values[key].trim();
        }
      }
      if(values.sectionName){
        values.sectionId = values.sectionName;
      }
      delete values.sectionName;

      setLoading(true);
      const response = await fetchAPI({
        method: 'PUT',
        url: replaceUrl(TABLE_DETAIL, {
          id: initialDragItem ? initialDragItem.id : null,
        }),
        data: values,
      });
      if (response) {
        const dragItemIndex =
          dragItems[
            dragItems.findIndex(item => item.id === initialDragItem?.id)
          ];
        const dragItemResponse = {
          ...response,
          height: dragItemIndex.height,
          width: dragItemIndex.width,
          positionX: dragItemIndex.positionX,
          positionY: dragItemIndex.positionY,
          rotationDegree: dragItemIndex.rotationDegree,
        };
        dispatch(updateDragItem(dragItemResponse));
        onCancel();
      }
    } finally {
      setLoading(false);
    }
  };

  const getGroupList = async currentFloorId => {
    const response = await fetchAPI({
      method: 'GET',
      url: `${SECTIONS}?floorId=${currentFloorId}`,
    });
    if (response) {
      setGroupList(response);
      if (initialDragItem?.shapeId)
        if(initialDragItem.sectionId && response.length > 0){
          const findSection = response.find(e => e.id == initialDragItem.sectionId);
          if(findSection){
            form.setFieldsValue({ sectionName: findSection.name });
          }
        }
        form.setFieldsValue({ sectionId: Number(initialDragItem?.sectionId) });
    }
  };

  useEffect(() => {
    getGroupList(currentFloorId);
  }, [currentFloorId]);

  const handleOnCancel = () => {
    onCancel();
    form.resetFields();
  };

  return (
    <Form form={form} name="horizontal_login" initialValues={initialDragItem}>
      <Modal
        visible={visible}
        width={400}
        centered
        onCancel={handleOnCancel}
        footer={[
          <Button
            type="primary"
            htmlType="submit"
            className="w-100"
            loading={loading}
            onClick={() => {
              form
                .validateFields()
                .then(values => {
                  onFinish(values);
                })
                .catch(info => {
                  console.log('Validate Failed:', info);
                });
            }}
          >
            Lưu
          </Button>,
        ]}
      >
        <h2 className="text-center">
          {initialDragItem?.shapeId === 2 ? (
            <FormattedMessage {...messages.editTable} />
          ) : (
            <FormattedMessage {...messages.editChair} />
          )}
        </h2>
        <FormattedMessage {...messages.tableName}>
          {tableName => (
            <InputWrapper>
              <Form.Item name="name" rules={[validateRequiredSpace]}>
                <FloatLabel
                  allowClear
                  label={
                    initialDragItem?.shapeId === 2 ? (
                      `${tableName}*`
                    ) : (
                      <FormattedMessage {...messages.chairName} />
                    )
                  }
                />
              </Form.Item>
            </InputWrapper>
          )}
        </FormattedMessage>
        <InputWrapper>
          <Form.Item
            name={"sectionName"}
          >
            <FloatLabel
              className="staff-filters__restaurant"
              label={<FormattedMessage {...messages.selectGroup} />}
              items={groupList}
              typeInput="selectMutil"
            />
          </Form.Item>
        </InputWrapper>
      </Modal>
    </Form>
  );
};
