import React, { useState, useEffect } from 'react';
import { Modal, Button, Form, Input, notification, message } from 'antd';
import { InputWrapper } from 'styles/custom-styled-components';
import { FloorModel } from 'containers/Restaurant/Store/types';
import { useParams } from 'react-router-dom';
import { fetchAPI, replaceUrl } from 'utils/fetchAPI';
import { FLOOR, FLOOR_DETAIL } from 'containers/App/urls';
import _ from 'lodash';
import messages from 'translations/messages';
import { FormattedMessage, injectIntl } from 'react-intl';
import { digits } from 'utils/number';
import { validateRequiredSpace } from '../../../../../../utils/validation';
import { FormatInput } from 'components/FormatInput';
import FloatLabel from 'components/FormatInput/FloatLabel';
interface Props {
  visible: boolean;
  onCancel: () => void;
  getFloors: () => void;
  initialFloor?: FloorModel;
}

const FloorModal = (props: Props) => {
  const { visible, onCancel, getFloors, initialFloor } = props;
  const [form] = Form.useForm();
  const [loading, setLoading] = useState(false);
  // @ts-ignore
  const { id } = useParams();
  const handleCancel = () => {
    onCancel();
    form.setFieldsValue({});
  };
  const onFinish = async values => {
    try {
      for (var key in values) {
        if (typeof values[key] === 'string') {
          values[key] = values[key].trim();
        }
      }
      if (_.isEmpty(initialFloor)) {
        setLoading(true);
        const modifiedValues = {
          ...values,
          restaurantId: id,
          photo: 'Some photo',
        };
        const response = await fetchAPI({
          method: 'POST',
          data: modifiedValues,
          url: FLOOR,
        });
        if (response) {
          getFloors();
          form.setFieldsValue({
            name: '',
            width: undefined,
            height: undefined,
          });
          onCancel();
        }
        setLoading(false);
      } else {
        setLoading(true);
        const modifiedValues = { ...values, photo: 'Some photo' };
        const response = await fetchAPI({
          method: 'PUT',
          data: modifiedValues,
          url: replaceUrl(FLOOR_DETAIL, { id: initialFloor!.id }),
        });
        if (response) {
          getFloors();
          onCancel();
          form.setFieldsValue({
            name: '',
            width: undefined,
            height: undefined,
          });
        }
        setLoading(false);
      }
    } finally {
      setLoading(false);
    }
  };
  useEffect(() => {
    if (initialFloor) {
      form.setFieldsValue({ ...initialFloor });
    } else {
      form.setFieldsValue({ name: '', width: undefined, height: undefined });
    }
  }, [initialFloor]);

  return (
    <Form form={form} name="horizontal_login" onFinish={onFinish}>
      <Modal
        visible={visible}
        centered
        width={400}
        onCancel={handleCancel}
        footer={[
          <Button
            type="primary"
            htmlType="submit"
            className="w-100"
            loading={loading}
            onClick={() => {
              form
                .validateFields()
                .then(values => {
                  onFinish(values);
                })
                .catch(info => {
                  console.log('Validate Failed:', info);
                });
            }}
          >
            <FormattedMessage {...messages.save} />
          </Button>,
        ]}
      >
        <h2 className="text-center">
          {initialFloor ? (
            <FormattedMessage {...messages.editFloor} />
          ) : (
            <FormattedMessage {...messages.createFloor} />
          )}
        </h2>
        <FormattedMessage {...messages.floorName}>
          {floorName => (
            <InputWrapper>
              <Form.Item name="name" rules={[validateRequiredSpace]}>
                <FloatLabel allowClear label={`${floorName}*`} />
              </Form.Item>
            </InputWrapper>
          )}
        </FormattedMessage>
        <FormattedMessage {...messages.width}>
          {width => (
            <InputWrapper>
              <Form.Item
                name="width"
                rules={[
                  {
                    required: true,
                    message: <FormattedMessage {...messages.requiredInput} />,
                  },
                  ({ getFieldValue }) => ({
                    validator(rule, value) {
                      let text;
                      if (value > 10000) {
                        text = (
                          <FormattedMessage {...messages.requiredMaxAcreage} />
                        );
                        return Promise.reject(`${text}`);
                      } else if (value <= 0) {
                        text = (
                          <FormattedMessage {...messages.requiredMinAcreage} />
                        );
                        return Promise.reject(`${text}`);
                      }
                      return Promise.resolve();
                    },
                  }),
                ]}
              >
                <FormatInput
                  placeholder={width}
                  min={1}
                  addonAfter="m"
                />
              </Form.Item>
            </InputWrapper>
          )}
        </FormattedMessage>
        <FormattedMessage {...messages.height} name="height">
          {height => (
            <InputWrapper>
              <Form.Item
                name="height"
                rules={[
                  {
                    required: true,
                    message: <FormattedMessage {...messages.requiredInput} />,
                  },
                  ({ getFieldValue }) => ({
                    validator(rule, value) {
                      let text;
                      if (value > 10000) {
                        text = (
                          <FormattedMessage {...messages.requiredMaxAcreage} />
                        );
                        return Promise.reject(`${text}`);
                      } else if (value <= 0) {
                        text = (
                          <FormattedMessage {...messages.requiredMinAcreage} />
                        );
                        return Promise.reject(`${text}`);
                      }
                      return Promise.resolve();
                    },
                  }),
                ]}
              >
                <FormatInput
                  placeholder={height}
                  min={1}
                  addonAfter="m"
                />
              </Form.Item>
            </InputWrapper>
          )}
        </FormattedMessage>
      </Modal>
    </Form>
  );
};
export default injectIntl(FloorModal);
