import React, { useState, useEffect } from 'react';
import styled from 'styled-components';
import { Space, Button, notification, Popconfirm } from 'antd';
import { FloorModel } from 'containers/Restaurant/Store/types';
import FloorModal from './Floors-children/floorModal';
import { EditIcon, RemoveIcon, PlusCharacter } from 'styles/icon-components';
import { useHistory, useParams } from 'react-router-dom';
import { RoutePaths } from 'containers/App/routes';
import { getSearchParamValueByParams } from 'utils/searchParams';
import { fetchAPI, replaceUrl } from 'utils/fetchAPI';
import { FLOOR_DETAIL } from 'containers/App/urls';
import { useDispatch } from 'react-redux';
import { setCurrentFloorIdAction, setAreaFloorAction } from '../../../Store/actions';
import { FormattedMessage } from 'react-intl';
import messages from 'translations/messages';
const FloorListWrapper = styled.div`
  display: flex;
  overflow: auto;
  margin-bottom: 20px;
  justify-content: flex-start;
  margin: 10px;
  div:nth-child(n+1) {
    margin: 0 5px;
  }
`;

const FloorWrapper = styled.div<{ visiting: boolean }>`
  background: ${props => props.visiting ? '#4A4A4A' : '#fff'};
  color: ${props => props.visiting ? 'white' : '#black'};
  border: ${props => props.visiting ? 'none' : 'solid 1px #DDD'};
  display: flex;
  align-items: center;
  border-radius: 22px;
  height: 44px;
  overflow: visible;
  cursor: pointer;
  .floor-name {
    border-right: solid 1px #DDD;
    padding: 10px;
    line-height: 20px;
  }
`;

interface Props {
  readonly floorList: Array<FloorModel>;
  readonly getFloors: any;
}
export const Floors = (props: Props) => {
  const { floorList, getFloors } = props;
  const dispatch = useDispatch();
  const history = useHistory();
  const [openModal, setOpenModal] = useState(false);
  const [initialFloor, setInitialFloor] = useState();
  const [currentFloor, setCurrentFloor] = useState(floorList[0]);
  const onCancel = () => {
    setOpenModal(false);
  }
  const setCurrentFloorId = (floorId) => {
    const urlSearchParams = new URLSearchParams(history.location.search);
    urlSearchParams.set('currentFloorId', floorId);
    history.push({ search: urlSearchParams.toString() });
    const newCurrentFloor = floorList.find(floor => floor.id === +floorId);
    if (newCurrentFloor) {
      setCurrentFloor(newCurrentFloor);
      dispatch(setCurrentFloorIdAction(floorId));
    }
  }

  const setAreaFloor = (floor) => {
    dispatch(setAreaFloorAction({width: floor.width, height: floor.height}));
  }

  const handleAddButton = () => {
    setInitialFloor(undefined);
    setOpenModal(true);
  }

  const handleEdit = (floor) => {
    setInitialFloor(floor);
    setOpenModal(true);
  }

  const handleDelete = async (floor) => {
    const response = await fetchAPI({
      method: 'DELETE',
      url: replaceUrl(FLOOR_DETAIL, { id: floor!.id })
    });
    if (response) {
      getFloors();
    }
  }

  useEffect(() => {
    const currentFloorId = getSearchParamValueByParams(history.location.search, 'currentFloorId');
    if (currentFloorId && +currentFloorId !== floorList[0].id) {
      const newCurrentFloor = floorList.find(floor => floor.id === +currentFloorId);
      if (newCurrentFloor) {
        setCurrentFloor(newCurrentFloor);
      }
    } else {
      if (floorList.length) {
        setCurrentFloorId(floorList[0].id)
      }
    }
  }, [])
  return (
    <div style={{ overflow: 'auto' }}>
      <FloorListWrapper>
        {
          floorList?.length ?
            floorList.map(floor => {
              const visiting = currentFloor?.id === floor.id;
              return (<>
                  <FloorWrapper
                    visiting={visiting}
                    key={floor.id}
                  >
                  <span
                    className="floor-name"
                    onClick={() => {setCurrentFloorId(floor.id), setAreaFloor(floor)}}
                    style={{ width: '80px', whiteSpace: 'nowrap', overflow: 'hidden' }}
                  >
                    {floor.name}
                  </span>
                  <div style={{ minWidth: '60px' }}>
                    <span
                      onClick={() => handleEdit(floor)}
                    ><EditIcon /></span>
                    <Popconfirm
                      title={<FormattedMessage {...messages.deleteConfirmMess} value={{name: floor.name}} />}
                      onConfirm={() => handleDelete(floor)}
                      okText={<FormattedMessage {...messages.deleteBtn} />}
                      cancelText={<FormattedMessage {...messages.cancelBtn} />}
                    >
                      <a href="#">
                        <RemoveIcon />
                      </a>
                    </Popconfirm>
                  </div>
                  </FloorWrapper>
                </>
              )

            })
            : <div><FormattedMessage {...messages.noFloors} /></div>
        }
        <Button
          shape="circle"
          type="primary"
          size="large"
          onClick={handleAddButton}
        ><PlusCharacter /></Button>
        <FloorModal
          getFloors={getFloors}
          onCancel={onCancel}
          visible={openModal}
          initialFloor={initialFloor}
        />
      </FloorListWrapper>
    </div>
  )
}
