import React, { useState, useRef, useEffect } from 'react';
import { Modal, Button, Select, Input, Form, Card, Menu, Spin, notification } from 'antd';
import { FormattedMessage, injectIntl } from 'react-intl';
import messages from 'translations/messages';
import styled from 'styled-components';
import GoogleMapReact from 'google-map-react';
import PlacesAutocomplete, {
  geocodeByAddress,
  getLatLng,
} from 'react-places-autocomplete';
import { fetchAPI, JsonToFormData } from 'utils/fetchAPI';
import { RESTAURANT } from 'containers/App/urls';
import { PRIMARY_COLOR, MOBILE_BREAK_POINT } from 'styles/constants';
import IC_MARK from '../../../../images/ic-mark.svg';
import { USER_INFO_KEY } from 'containers/App/constants';
import { StoreIcon } from 'styles/icon-components';
import { useDispatch, useSelector } from 'react-redux';
import messagesGlobal from 'translations/messages';
import { validateRequiredSpace, fileValidation } from 'utils/validation';
import { getRestaurants as actionGetRes } from "../../../App/actions";
import { makeSelectPhoneCountries } from "../../../App/selectors";
import { usePhoneAddon } from '../../../../components/CustomHook/usePhoneAddon'
import { createStructuredSelector } from "reselect";
import FloatLabel from 'components/FormatInput/FloatLabel';

const stateSelector = createStructuredSelector({
  phoneCountries: makeSelectPhoneCountries(),
});


const CreateRestaurantInfoWrapper = styled.div`
  display: flex;
  span.ant-input-group-addon {
    border-bottom-left-radius: 20px;
    border-top-left-radius: 20px;
  }
  span.ant-input-group-addon + span.ant-input-affix-wrapper {
    border-bottom-left-radius: 0px !important;
    border-top-left-radius: 0px !important;
  }
  .create-restaurant {
    &__info {
      width: 40%;
      .ant-upload {
        width: 100%!important;
        margin-bottom: 20px;
      }
      &__image {
        width: 100%;
        cursor: pointer;
        border: 1px solid #E6E6E6;
        border-radius: 8px;
        width: 100%;
        height: 200px;
        display: flex;
        align-items: center;
        justify-items: center;
        &__blank {
          flex-grow: 1;
          text-align: center;
          color: #E6E6E6;
          svg {
            width: 40px;
            height: 30px;
            margin-bottom: 10px;
          }
        }
        img {
          border: dashed 1px ${PRIMARY_COLOR};
          border-radius: 8px;
          width: 100%;
          height: 200px;
        }
      }
    }
    &__map {
      width: 60%;
      margin-left: 20px;
      border-radius: 8px;
      overflow: hidden;
    }
  }
  .ant-form-item.first-form-item {
    margin-top: 20px;
  }
  @media only screen and (max-width: ${MOBILE_BREAK_POINT}px){
    display: flex;
    flex-wrap: wrap;
    .create-restaurant {
      &__info {
        flex-grow: 1;
      }
      &__map {
        flex-grow: 1;
      }
    }
  }
`;

const InputWrapper = styled.div`

`;

const AutoCompleteAddressWrapper = styled.div`
  height: 250;
  width: 100%;
  z-index: 1000;
  position: absolute;
  overflow: visible;
`;

interface Props {
  visible: boolean;
  onCancel(): any;
  getRestaurants: any;
  intl?: any;
}

interface Coordinate {
  lat: number,
  lng: number,
}

export const CreateRestaurantModal = injectIntl((props: Props) => {
  const { visible, onCancel, getRestaurants, intl } = props;
  const [defaultCoordinate, setDefaultCoordinate] = useState<Coordinate>({ lat: 16.0755022, lng: 108.22002117 });
  const { phoneCountries } = useSelector(stateSelector);
  const prefixSelector = usePhoneAddon(phoneCountries);
  const [address, setAddress] = useState('');
  const [coordinate, setCoordinate] = useState<Coordinate>({ lat: 16.0755022, lng: 108.22002117 });
  const [loading, setLoading] = useState(false);
  const [imageUrl, setImageUrl] = useState<any>();
  const [imageFile, setImageFile] = useState();
  const inputRef = useRef<{ files: any, click: any }>();
  const [form] = Form.useForm();
  const dispatch = useDispatch();
  let bodyFormData = new FormData();
  const handleChange = addr => {
    setAddress(addr);
  };

  const handleSelect = addr => {
    form.setFieldsValue({ ...form.getFieldsValue(), fullAddress: addr });
    geocodeByAddress(addr)
      .then(results => getLatLng(results[0]))
      .then(latLng => setCoordinate(latLng))
      .catch(error => console.error('Error', error));
  };

  const handleOnCancel = () => {
    onCancel();
    form.resetFields();
    setImageUrl(undefined);
    setImageFile(undefined);
    setCoordinate(defaultCoordinate);
    form.setFieldsValue({ ...form.getFieldsValue(), fullAddress: '' });
  };

  const onFinish = async (values) => {
    setLoading(true);
    const userInfoString = localStorage.getItem(USER_INFO_KEY);
    const userInfo = JSON.parse(userInfoString!);
    const { merchantId } = userInfo.profile!;
    const { phoneCountryCode, phoneNumber} = values;

    for (var key in values) {
      if (typeof values[key] === 'string') {
        values[key] = values[key].trim();
      }
      if ( Array.isArray(values[key])) {
        values[key].forEach(subKey => typeof subKey === 'string' ? subKey.trim() : subKey);
      }
    }
    const modifiedValues = {
      ...values,
      phoneNumber: `${phoneCountryCode}${phoneNumber}`,
      merchantId: merchantId,
      detail: "Some testing restaurant",
      restaurantTypeId: 1,
      pickUpEffectiveTime: 1,
    }
    bodyFormData = JsonToFormData(modifiedValues);
    bodyFormData.append('aaaac', imageFile!);

    const response = await fetchAPI({
      method: 'POST',
      url: RESTAURANT,
      formData: true,
      data: bodyFormData,
    })
    if (response) {
      dispatch(actionGetRes(merchantId));
      getRestaurants();
      setLoading(false);
      onCancel();
      form.resetFields();
      setImageUrl(undefined);
      setImageFile(undefined);
      setCoordinate(defaultCoordinate);
    } else {
      setLoading(false);
    }
  }

  const handleChangeUploadImage = info => {
    const file = inputRef?.current?.files[0];
    if (!fileValidation(file.name))
      return notification.error({ message: intl.formatMessage(messages.invalidFile)});
    const reader = new FileReader();
    const url = reader.readAsDataURL(file);
    reader.onloadend = function (e) {
      setImageUrl(reader.result!);
    }
    setImageFile(file);
  }

  return (
    <Form
      form={form}
      name="horizontal_login"
      onFinish={onFinish}
    >
      <Modal
        visible={visible}
        width={800}
        onCancel={handleOnCancel}
        centered
        footer={
          [<Button
            type="primary"
            htmlType="submit"
            loading={loading}
            onClick={() => {
              form
                .validateFields()
                .then(values => {
                  onFinish(values);
                })
                .catch(info => {
                  console.log('Validate Failed:', info);
                });
            }}><FormattedMessage {...messages.save} /></Button>]
        }
      >
        <h3 style={{ textAlign: "center", marginBottom: 20, fontSize: 15 }}><FormattedMessage {...messages.addStore} /></h3>
        <CreateRestaurantInfoWrapper>
          <div className="create-restaurant__info">
            <div className="create-restaurant__info__image" onClick={() => inputRef.current!.click()} >
              {
                imageUrl ?
                  <img src={imageUrl} style={{objectFit: 'cover'}} />
                  : (
                    <>
                  <div
                    className="create-restaurant__info__image__blank"
                  >
                    <StoreIcon />
                    <div><FormattedMessage {...messages.restaurantImage} /></div>
                    <span className="img-hint"><
                    FormattedMessage
                  {...messagesGlobal.encourageCondition}
                  values={{ height: 280,
                    width: 200, }}
                /></span>
                  </div>
                </>
                  )
              }
            </div>
            <input
              type="file"
              ref={inputRef as any}
              accept="image/*"
              onChange={handleChangeUploadImage}
              style={{ visibility: 'hidden', height: 0, position: "fixed" }}
            />
            <FormattedMessage {...messages.storeName} >
              {storeName =>
                <InputWrapper>
                  <Form.Item
                    name="name"
                    rules={[validateRequiredSpace]}
                    className="first-form-item"
                  >
                    <FloatLabel label={`${storeName}*`} name="name" />
                  </Form.Item>
                </InputWrapper>
              }
            </FormattedMessage>
            <FormattedMessage {...messages.address}>
              {
                address =>
                  <InputWrapper>
                    <Form.Item
                      name="fullAddress"
                      rules={[validateRequiredSpace]}
                    >
                      <PlacesAutocomplete
                        value={address}
                        onChange={handleChange}
                        onSelect={handleSelect}
                      >
                        {({ getInputProps, suggestions, getSuggestionItemProps, loading }) => (
                          <div>
                            <FloatLabel {...getInputProps({
                                className: 'location-search-input',
                              })} value={form.getFieldValue('fullAddress')} maxLength={100} 
                              label={`${address}*`}
                              />
                            <AutoCompleteAddressWrapper>
                              {loading ? <div style={{ textAlign: "center", top: '20px', position: "relative" }}> <Spin /> </div> :
                                <Menu>
                                  {suggestions.map(suggestion => {
                                    return (
                                      <Menu.Item {...getSuggestionItemProps(suggestion)}>
                                        <span>{suggestion.description}</span>
                                      </Menu.Item>
                                    );
                                  })}
                                </Menu>
                              }
                            </AutoCompleteAddressWrapper>
                          </div>
                        )}
                      </PlacesAutocomplete>
                    </Form.Item>
                  </InputWrapper>
              }
            </FormattedMessage>
            <FormattedMessage {...messages.phoneNumber} >
              {phoneNumber =>
                <InputWrapper>
                  <Form.Item
                    name="phoneNumber"
                    rules={[{ required: true, message: <FormattedMessage {...messages.requiredPhone} /> }, { message: <FormattedMessage {...messages.invalidPhone} />, pattern: /^[\d]+$/},() => ({
                      validator(rule, value) {
                        if (value.length > 10) {
                          return Promise.reject(
                            intl.formatMessage(messages.invalidPhone),
                          );
                        } else {
                          return Promise.resolve();
                        }
                      },
                    }),
                  ]}
                  >
                    <FloatLabel addonBefore={prefixSelector} allowClear  label={`${phoneNumber}*`}/>
                  </Form.Item>
                </InputWrapper>
              }
            </FormattedMessage>
            <FormattedMessage {...messages.email} >
              {email =>
                <InputWrapper>
                  <Form.Item
                    name="email"
                    rules={[{ message: <FormattedMessage {...messages.invalidEmail} />, type:'email'}]}
                  >
                    <FloatLabel allowClear  label={`${email}*`}/>
                  </Form.Item>
                </InputWrapper>
              }
            </FormattedMessage>
          </div>
          <div className="create-restaurant__map">
            <GoogleMapReact
              bootstrapURLKeys={{ key: 'AIzaSyBGX1AI9VbgVLWkrMyUFLPXx__-U0ZFR4Y' }}
              defaultCenter={coordinate}
              defaultZoom={15}
              center={coordinate}
            >
              <img
                data-lat={coordinate.lat}
                data-lng={coordinate.lng}
                data-text="My Marker"
                style={{ width: '20px', height: '20px' }}
                src={IC_MARK}
              />
            </GoogleMapReact>
          </div>
        </CreateRestaurantInfoWrapper>
      </Modal>
    </Form >
  );
})

