import React, { useState } from 'react';
import styled from 'styled-components';
import { RestaurantModel } from '../Store/types';
import { CARD_HEIGHT } from 'styles/constants';
import { EnvironmentOutlined, PhoneOutlined, MailOutlined } from '@ant-design/icons';
import { Button, Space, Modal, notification } from 'antd';
import { getRestaurants as actionGetRes} from "../../App/actions";
import { useHistory } from 'react-router-dom';
import { RoutePaths } from 'containers/App/routes';
import { Card } from 'styles/custom-styled-components';
import { PUBLIC_API, RESTAURANT_BY_ID_URL } from 'containers/App/urls';
import { ExclamationCircleOutlined } from '@ant-design/icons';
import { fetchAPI, replaceUrl } from 'utils/fetchAPI';
import { useDispatch } from 'react-redux';
import { USER_INFO_KEY } from 'containers/App/constants';
import { injectIntl } from 'react-intl';
import messages from 'translations/messages';
import { EditIcon, RemoveIcon, MenuItemIcon } from 'styles/icon-components';

const HoveredCardWrapper = styled.div`
  position: relative;
  top: calc(-${CARD_HEIGHT} / 2);
  text-align: center;
  svg {
    width: 20px;
    height: 17px;
  }
`;

interface Props {
  restaurant: RestaurantModel;
  getRestaurants?: any;
  intl?: any;
}

const HoveredCard = injectIntl((props: Props) => {
  const { restaurant, getRestaurants, intl } = props;
  const userInfoString = localStorage.getItem(USER_INFO_KEY);
  const userInfo = JSON.parse(userInfoString!);
  const dispatch = useDispatch();
  const { merchantId } = userInfo.profile!;

  const history = useHistory();
  const handleOnClickEdit = () => history.push(`${RoutePaths.restaurantDetail}/${restaurant.id}`)
  const deleteRestaurant = async() => {
    const response = await fetchAPI({
      method: 'DELETE',
      url: replaceUrl(RESTAURANT_BY_ID_URL, { id: restaurant.id })
    })
    if (response) {
      getRestaurants();
      dispatch(actionGetRes(merchantId));
    }
  }
  const confirm = () => {
    Modal.confirm({
      title: intl.formatMessage(messages.deleteConfirmMess),
      icon: <ExclamationCircleOutlined />,
      content: intl.formatMessage(messages.deleteConfirmContent, {name: restaurant.name}),
      okText: intl.formatMessage(messages.deleteBtn),
      cancelText: intl.formatMessage(messages.cancelBtn),
      maskClosable: true,
      onOk: deleteRestaurant
    });
  }
    return (
    <HoveredCardWrapper>
      <Space>
        <Button
          shape="circle"
          style={{ width: 50, height: 50 }}
          onClick={handleOnClickEdit}
        ><EditIcon /></Button>
        <Button
          shape="circle" style={{ width: 50, height: 50 }}
          onClick={confirm}
        ><RemoveIcon /></Button>
      </Space>
    </HoveredCardWrapper >
  )
})

function CardDetail(props: Props) {
  const { restaurant } = props;
  const {
    fullAddress,
    email,
    name,
    phoneNumber,
    photo,
    isPublic
  } = restaurant;
  return (
    <>
      <div className="restaurant-card__header">
      <div style={{position: 'relative'}}>{ photo ? <img className="image" src={`${PUBLIC_API}/${photo}`} /> : <div className="default-menu"><MenuItemIcon /></div>}
      <span className="restaurant-card__dot" style={{backgroundColor: isPublic ? '#9cd339' : '#9C9696'}}></span>
      </div>
        <span className="name">{name}</span>
      </div>
      <hr />
      {fullAddress && <div className="restaurant-card__row">
        <EnvironmentOutlined />
        <span className="restaurant-card__row--info">{fullAddress}</span>
      </div>
      }
      {phoneNumber && <div className="restaurant-card__row">
        <PhoneOutlined />
        <span className="restaurant-card__row--info">{phoneNumber}</span>
        </div>
      }
      {email  && <div className="restaurant-card__row">
        <MailOutlined />
        <span className="restaurant-card__row--info">{email}</span>
      </div>
      }
    </>
  )
}

export function RestaurantCard(props: Props) {
  const { restaurant, getRestaurants } = props;
  const [isHovered, setHovered] = useState(false);

  return (
    <Card
      isHovered={isHovered}
      onMouseOver={() => setHovered(true)}
      onMouseLeave={() => setHovered(false)}
      onTouchMoveCapture={() => setHovered(!isHovered)}
      className="touch-card"
    >
      <CardDetail restaurant={restaurant} />
      {isHovered && <HoveredCard restaurant={restaurant} getRestaurants={getRestaurants}/>}
    </Card >
  );
}
