import React, {useState, useEffect} from 'react';
import styled from 'styled-components';
import {Link, Route} from "react-router-dom";
import {Form, Select, Input, Button, Space} from 'antd';
import {SearchOutlined} from '@ant-design/icons';

import {CategoryModel} from 'containers/Service/ServiceEdit/serviceDetailModels';
import {MENUS_MENU_CATEGORIES_URL} from 'containers/App/urls';
import {fetchAPI} from 'utils/fetchAPI';
import { PlusCharacter } from 'styles/icon-components';
import { injectIntl } from 'react-intl';
import messages from 'translations/messages';
import FloatLabel from 'components/FormatInput/FloatLabel';

const ServiceListHeaderWrapper = styled.div`
  .header {
    &__title {
      font-size: 21px;
      font-weight: 600;
      color: rgb(51, 51, 51);
      padding-bottom: 10px;
    }
    &__toolbar {
      display: flex;
      @media only screen and (max-width: 736px) {
        flex-wrap: wrap;
        flex-direction: column-reverse;
        padding-bottom: 15px;
      }
      @media only screen and (min-device-width: 768px) and (max-device-width: 1024px) and (-webkit-min-device-pixel-ratio: 1) {
        flex-wrap: wrap;
        flex-direction: column-reverse;
        padding-bottom: 15px;
      }
    }
  }
  .ant-select-selection-item {
    line-height: 12px !important;
  }
  .tabs {
    display: flex;
    flex-direction: row;
    flex-wrap: nowrap;
    justify-content: flex-start;
    list-style: none;
    padding: 0;
    margin: 0;

    &__menu {
      font-size: 16px;
      padding: 17px 10px;
      margin-right: 15px;
      border-bottom: 1px solid #ccc;
      cursor: pointer;
      color: inherit;
      &.active {
        color: #9cd339;
        border-bottom: 2px solid #9cd339;
      }
    }
  }

  .filter {
    display: flex;
    margin-left: auto;

    form {
      display: flex;
      @media only screen and (max-width: 736px) {
        display: block;
      }
      .ant-form-item {
        margin-bottom: 0 !important;
      }
    }

    @media only screen and (max-width: 736px) {
      display: block;
      margin: 15px 0 !important;
    }

    @media only screen and (min-device-width: 768px) and (max-device-width: 1024px) and (-webkit-min-device-pixel-ratio: 1) {
      margin: 15px;
      margin-left: 0 !important;
    }

    &__all {
      width: 220px;
      margin-right: 10px;
      > div {
        width: 100%;
      }
      @media only screen and (max-width: 736px) {
        margin: 0;
        width: 100% !important;
      }
    }

    &__search {
      @media only screen and (max-width: 736px) {
        margin-top: 15px;
        width: 100%;
      }
    }
    @media only screen and (min-device-width: 768px) and (max-device-width: 1024px) and (-webkit-min-device-pixel-ratio: 1) {
      width: 100%;
    }
  }
`;

function ServiceListHeader(props) {
  const { handleSearch, handleCategoryChange, categoryId, intl} = props;
  const [serviceCategories, setServiceCategories] = useState<CategoryModel[]>() as any;
  const [form] = Form.useForm();

  const getServiceCategories = async () => {
    const response = await fetchAPI({
      method: 'GET',
      url: `${MENUS_MENU_CATEGORIES_URL}?productType=2`,
    });
    if (response) {
      setServiceCategories([...response, { id: 0, name: intl.formatMessage(messages.allSelectCategories)}]);
    }
  };
  useEffect(() => {
    if (categoryId) {
      form.setFieldsValue({
        category: categoryId,
        keyword: '',
      });
    }
  }, [categoryId]);


  useEffect(() => {
    getServiceCategories();
  }, []);


  return (
  <ServiceListHeaderWrapper>
    <div className="title-global-header">{intl.formatMessage(messages.service)}</div>
    <div className="header__toolbar">
      <div className="filter">
        <Form form={form}>
            <div className="filter__all">
              <Form.Item name="category">
                <FloatLabel defaultValue={0} size="large" onChange={handleCategoryChange}
                items={serviceCategories ? serviceCategories : []}
                typeInput="selectMutil"
                label={intl.formatMessage(messages.category)}
                className="padding-mutil-select"
                />
              </Form.Item>
            </div>
          <div className="filter__search">
            <Space>
              <Form.Item name="keyword">
                <FloatLabel size="large" width={300} label={intl.formatMessage(messages.search)} prefix={<SearchOutlined/>} onPressEnter={
                  () => {
                    form
                      .validateFields()
                      .then(values => {
                        handleSearch(values);
                      })
                      .catch(info => {
                        console.log('Validate Failed:', info);
                      });
                  }
                }/>
              </Form.Item>
              <Button size="large" type="primary" onClick={() => {
                form
                  .validateFields()
                  .then(values => {
                    handleSearch(values);
                  })
                  .catch(info => {
                    console.log('Validate Failed:', info);
                  });

              }}>{intl.formatMessage(messages.search)}</Button>
              <Link to='/service/add'>
                <Button size="large" type="primary" shape="circle" style={{padding: 0}}> <PlusCharacter /> </Button>
              </Link>
            </Space>
          </div>
        </Form>

      </div>
    </div>
  </ServiceListHeaderWrapper>);
}
export default injectIntl(ServiceListHeader);