import React, { useState, useEffect } from 'react';
import {
  Modal,
  Button,
  Input,
  Form,
  Typography,
  Space,
  Tag,
  Row,
} from 'antd';
import { FormattedMessage } from 'react-intl';
import globalMessages from 'translations/messages';
import styled from 'styled-components';
import { fetchAPI, replaceUrl } from 'utils/fetchAPI';
import { COOKING_REQUEST, COOKING_REQUEST_DETAIL} from 'containers/App/urls';
import { PreparationGroupModel } from '../../Store/types';
import { PlusOutlined } from '@ant-design/icons';
import { injectIntl } from "react-intl";
import { addCookingRequestGroup, updateCookingRequestGroup } from '../../Store/actions';
import { useSelector, useDispatch } from 'react-redux';
import { makeSelectFletch } from '../../Store/selectors';
import { createStructuredSelector } from 'reselect';
import FloatLabel from 'components/FormatInput/FloatLabel';

const stateSelector = createStructuredSelector({
  isFletching: makeSelectFletch(),
});

const CreateEditPreparationWrapper = styled.div`
  display: flex;
  justify-content: center;
  .ant-tag {
    padding: 0px 10px;
    background-color: #4a4a4a;
    border-radius: 16px;
    height: 32px;
    display: flex;
    align-items: center;
    justify-content: center;
    color: #ffffff;
    .anticon.anticon-close {
      vertical-align: baseline;
      color: #ffffff;
      margin-left: 5px;
    }
  }
  button.add-preration-item {
    span {
      display: inline-flex;
    }
  }
  .modal-edit {
    &__menu {
      &--title {
        font-size: 13px;
        line-height: 19px;
        text-align: center;
      }
    }
  }
`;
const { Title } = Typography;
interface Props {
  visible: boolean;
  onCancel(): any;
  getPreparationList: any;
  isCreate: boolean;
  preparation?: PreparationGroupModel;
  messages?: any;
  intl?: any;
  index?: any;
}

function CreateEditPreparation(props: Props) {
  const {
    visible,
    onCancel,
    isCreate,
    preparation,
    intl,
    index
  } = props;
  const [form] = Form.useForm();
  const [prerationItems, setPrerationItems] = useState([] as any);
  const { isFletching } = useSelector(stateSelector);
  const dispatch = useDispatch();
  const addPreparation = async e => {
    const item = form.getFieldsValue(['item']);
    form.resetFields(['item']);
    console.log("preparation ", preparation);
    if (item && item.item && item.item !== '') {
      const dataApi = await fetchAPI({
        method: 'POST',
        url: COOKING_REQUEST,
        data: {
          name: item.item,
          cookingRequestGroupId: preparation?.id ? preparation?.id : null,
        },
      });
      setPrerationItems(oldItems => [
        ...oldItems,
        { id: dataApi.id, name: dataApi.name },
      ]);
    }
  };
  const handleClose = async (idx, item) => {
    if (item.id) {
      await fetchAPI({
        method: 'DELETE',
        url: replaceUrl(COOKING_REQUEST_DETAIL, { id: item.id }),
      });
    }
    const tags = prerationItems.filter((tag, index) => index !== idx);
    setPrerationItems(tags);
  };

  const onFinish = async values => {
    // const cookingRequests = prerationItems.map(item => item.id);
    const preparationModdifier = {
      name: values.name,
      cookingRequests: prerationItems,
    };
   isCreate
      ? dispatch(addCookingRequestGroup(preparationModdifier))
      : dispatch(updateCookingRequestGroup({id: preparation ? preparation.id : undefined, ...preparationModdifier}, index));
      onCancel();
  };

  useEffect(() => {
    setPrerationItems(preparation ? preparation.cookingRequests : []);
  }, []);

  return (
    <Form
      form={form}
      name="horizontal_login"
      onFinish={onFinish}
      layout="horizontal"
      size="middle"
      initialValues={preparation ? preparation : {}}
    >
      <Modal
        visible={visible}
        onCancel={onCancel}
        width={330}
        centered
        footer={null}
      >
        <CreateEditPreparationWrapper>
          <Space direction="vertical" style={{ width: '100%' }}>
            <Title level={4} className="modal-edit__menu--title">
              {isCreate ? (
                <FormattedMessage {...globalMessages.addPreparation} />
              ) : (
                <FormattedMessage {...globalMessages.editPreparation} />
              )}
            </Title>
            <Form.Item
              colon={false}
              name="name"
              rules={[
                { required: true, message: intl.formatMessage(globalMessages.requiredMethodOfPreparation) },
              ]}
              style={{ width: '100%', marginBottom: '11px' }}
            >
              <FloatLabel label={`${intl.formatMessage(globalMessages.name)}*`} maxLength={30} />
            </Form.Item>
            <Form.Item
              colon={false}
              name="item"
              style={{ width: '100%', marginBottom: '0px' }}
            >
              <FloatLabel 
                label={`${intl.formatMessage(globalMessages.methodOfPreparation)}*`}
                maxLength={50}
                suffix={
                  <Button
                    type="primary"
                    shape="circle"
                    icon={<PlusOutlined />}
                    size="small"
                    className="add-preration-item"
                    onClick={e => addPreparation(e)}
                  />
                }
              />
            </Form.Item>
            <Row>
              {prerationItems
                ? prerationItems.map((item, index) => (
                    <Tag
                      closable
                      onClose={e => {
                        e.preventDefault();
                        handleClose(index, item);
                      }}
                      style={{maxWidth: '280px' }}
                    >
                      {item.name && item.name.length > 40 ? `${item.name.slice(0, 40)}...` : item.name}
                    </Tag>
                  ))
                : ''}
            </Row>
            <Form.Item>
              <Button
                type="primary"
                htmlType="submit"
                style={{ width: '100%' }}
                loading={isFletching}
                onClick={() => {
                  form
                    .validateFields()
                    .then(values => {
                      onFinish(values);
                    })
                    .catch(info => {
                      console.log('Validate Failed:', info);
                    });
                }}
              >
               {intl.formatMessage(globalMessages.save)}
              </Button>
            </Form.Item>
          </Space>
        </CreateEditPreparationWrapper>
      </Modal>
    </Form>
  );
}
export default injectIntl(CreateEditPreparation);