import React, { useEffect, useState } from 'react';
import styled from 'styled-components';
import { fetchAPI } from 'utils/fetchAPI';
import { PAYROLL_URL } from 'containers/App/urls';
import { PlusOutlined } from '@ant-design/icons';
import { Typography, Radio, Row, Col, Button, Space, Spin } from 'antd';
import { PayrollCard } from './cardPayroll';
import CreateEditPayrollCategory from './createEditPayRollCategory';
import { injectIntl } from 'react-intl';
import { PayrollModel, OptionPayrollModel } from '../Store/types';
import messagesGlobal from 'translations/messages';
import messages from '../I18t/messages';
import { chunk } from 'lodash';
import { makeSelectPayrollSetting, makeSelectFletch, makeSelectMerchantSetting } from '../Store/selectors';
import { useSelector, useDispatch } from 'react-redux';
import { setMerchantSetting } from '../Store/actions';
import { createStructuredSelector } from 'reselect';
const stateSelector = createStructuredSelector({
  payrolls: makeSelectPayrollSetting(),
  isFletching: makeSelectFletch(),
  merchantSetting: makeSelectMerchantSetting(),
});

const PayrollSettingWrapper = styled.div`
  width: 100%;
  margin-bottom: 20px;
  .ant-divider.ant-divider-horizontal {
    margin: 3px 0;
  }
  .setting-menu {
    display: flex;
    justify-content: space-between;
    margin-bottom: 15px;
  }
  button {
    span {
      display: inline-flex;
    }
  }
  h4 {
    flex: 1;
  }
  .no-payroll {
    color: white;
    background-color: #dd3880;
  }
  .payroll {
    color: white;
    background-color: #00aaf6;
  }
  .ant-row {
    .first-box {
      padding: 20px;
      .ant-radio-group {
        padding: 22.5px 0px;
      }
    }
  }
  div.ant-typography,
  .ant-typography p {
    margin-bottom: 0;
  }
  .ant-col.label-input {
    >.ant-row {
      height: 100%;
    }
  }
`;
const { Title, Paragraph } = Typography;
function PayrollSetting({ intl }) {
  const [payrollList, setPayrollList] = useState<PayrollModel>({
    payPeriod: 1,
    options: [] as OptionPayrollModel[],
  });
  const dispatch = useDispatch(); // eslint-disable-line @typescript-eslint/no-unused-vars
  const { payrolls, isFletching, merchantSetting } = useSelector(stateSelector);
  const [openCreatePayrollModal, setOpenCreatePayrollModal] = useState(false);

  const handleChangePeriod = e => {
    setPayrollList({ ...payrollList, payPeriod: e.target.value });
  };
  const handleChangePayroll = value => {
    setPayrollList(value);
  };

  const onFinish = () => {
    dispatch(setMerchantSetting({ payrollSetting: payrollList }));
  };
  useEffect(() => {
    if (payrolls) {
      setPayrollList(payrolls);
    }
  }, [payrolls]);

  return (
    <Spin spinning={isFletching}>
      <PayrollSettingWrapper>
        <Row gutter={[10, 20]}>
          <Col className="label-input" xl={6} lg={10} md={10} sm={24} xs={24}>
            <Row>
              <div className="first-box">
                <Title level={4} className="setting-menu-title">
                  {intl.formatMessage(messagesGlobal.payrollPeriod)}
                </Title>
                <Radio.Group
                  name="payPeriod"
                  value={payrollList.payPeriod}
                  onChange={handleChangePeriod}
                >
                  <Space size={20}>
                    <Radio value={2}>
                      {intl.formatMessage(messagesGlobal.weekly)}
                    </Radio>
                    <Radio value={1}>
                      {intl.formatMessage(messagesGlobal.monthly)}
                    </Radio>
                  </Space>
                </Radio.Group>
              </div>
            </Row>
          </Col>
          <Col className="label-input" xl={10} lg={12} md={12} sm={24} xs={24}>
            <div className="first-box" style={{minHeight: '139px'}}>
              <div className="setting-menu">
                <Title level={4} className="setting-menu-title">
                  {`${intl.formatMessage(
                    messages.income,
                  )} / ${intl.formatMessage(messages.deduction).toLowerCase()}`}
                </Title>
                <Button
                  type="primary"
                  shape="circle"
                  icon={<PlusOutlined />}
                  size="large"
                  onClick={() => setOpenCreatePayrollModal(true)}
                />
              </div>
              {chunk(payrollList.options, 4).map((subPayroll, index) => (
                <Row gutter={[12, 12]}>
                  {subPayroll.map((payroll, subIndex) => (
                    <PayrollCard
                      payrollList={payrollList}
                      payroll={payroll}
                      key={index}
                      payrollIdx={{index: index, subIndex: subIndex}}
                      handleChangePayroll={handleChangePayroll}
                    />
                  ))}
                </Row>
              ))}
            </div>
          </Col>
          {openCreatePayrollModal && (
            <CreateEditPayrollCategory
              payrollList={payrollList}
              visible={openCreatePayrollModal}
              onCancel={() => setOpenCreatePayrollModal(false)}
              handleChangePayroll={handleChangePayroll}
              isCreate={true}
            />
          )}
        </Row>
        <Button
          type="primary"
          shape="round"
          size="large"
          htmlType="submit"
          className="btn-save"
          onClick={onFinish}
        >
          {intl.formatMessage(messagesGlobal.save)}
        </Button>
      </PayrollSettingWrapper>
    </Spin>
  );
}
export default injectIntl(PayrollSetting);
