import React, { useEffect, useState, useRef } from 'react';
import styled from 'styled-components';
import { useSelector, useDispatch } from 'react-redux';
import { injectIntl } from 'react-intl';
import messagesGlobal from 'translations/messages';
import messages from '../I18t/messages';
import { PlusOutlined } from '@ant-design/icons';
import { PlusCharacter, RemoveSettingCharacter } from 'styles/icon-components';
import { FormatInput } from 'components/FormatInput';
import { useCurrencyHook } from '../../../components/CustomHook/useCurrencyHook';
import {
  makeSelectMerchantSetting,
  makeSelectShippingFees,
} from '../Store/selectors';
import { setMerchantSetting} from '../Store/actions';
import { createStructuredSelector } from 'reselect';
import FloatLabel from 'components/FormatInput/FloatLabel';
const stateSelector = createStructuredSelector({
  merchantSetting: makeSelectMerchantSetting(),
  shippingFees: makeSelectShippingFees(),
});

import { Typography, Row, Radio, Button, Space, Form, Col } from 'antd';
import { PathIcon } from 'styles/icon-components';
import _ from 'lodash';
const ConfigSettingWrapper = styled.div`
  height: 100%;
  width: 100%;
  margin-left: 1px;
  font-family: Muli;
  input.ant-input,
  .ant-select-selector,
  div.ant-picker {
    min-height: 44px !important;
    align-items: center;
  }
  textarea.ant-input {
    padding: 18px;
    border-radius: 0px;
  }
  span.ant-picker-suffix {
    align-self: initial;
  }
  .ant-input-group-addon:last-child {
    border-radius: 22px;
    border-top-left-radius: 0;
    border-bottom-left-radius: 0;
    width: 53px;
  }
  div.ant-picker {
    border-radius: 22px;
  }
  button {
    width: 110px;
    height: 44px;
    border-radius: 22px;
  }
  .path-icon {
    display: none;
  }
  .group {
    padding: 20px;
    margin-bottom: 20px;
    box-shadow: 0 1px 3px 0 rgba(0, 0, 0, 0.1);
    background-color: #ffffff;
    .label-input {
      margin-top: 1em;
    }
  }
  @media only screen and (min-width: 1200px) {
    width: 60%;
    .section {
      display: flex;
      align-items: center;
      justify-content: space-between;
      padding: 8px 0 8px 0;
      .ant-typography {
        margin-bottom: 0;
      }
      .ant-form-item {
        margin-bottom: 0 !important;
      }
      &__item {
        margin-right: 10px;
        white-space: nowrap;
      }
      &__item--small {
        margin-right: 10px;
        width: 130px;
      }
    }
    .section--no-padding {
      display: flex;
      align-items: center;
      justify-content: space-between;
      padding: 8px 0;
      .ant-typography {
        margin-bottom: 0;
      }
      .ant-form-item {
        margin-bottom: 0;
      }
      .km {
        width: 105px;
      }
      .currency {
        width: 160px;
      }
      .path-icon {
        display: block;
      }
      .fee-from {
        &__title {
          flex-basis: 40px;
        }
      }
    }
  }
  .setting-config {
    display: flex;
    > span {
      width: 87.5%;
    }
  }
`;

const { Title, Paragraph } = Typography;

interface Props {
  intl: any;
  messages: any;
}

function ShippingSetting(props: Props) {
  const { intl } = props;
  const [isDisplayFee, setIsDisplayFee] = useState<boolean>();
  const [loading, setLoading] = useState<boolean>(false);
  const dispatch = useDispatch(); // eslint-disable-line @typescript-eslint/no-unused-vars
  const addRef = useRef(null);
  const {shippingFees,merchantSetting } = useSelector(stateSelector);
  const [form] = Form.useForm();

  const currencyMemoi = useCurrencyHook();
  const hasOrderFee = shippingFees?.length? true: false;
  const onFinish = async values => {
    for (const property in values) {
      if (_.isString(values[property])) {
        values[property] = values[property]
          ? values[property].trim() === ''
            ? null
            : values[property].trim()
          : null;
      }
      if (
        typeof values[property] === 'string' &&
        /^[0-9,]+$/.test(values[property])
      ) {
        values[property] = values[property].replace(/,/g, '');
      }
    }
    dispatch(setMerchantSetting({ shippingFees: isDisplayFee? values.shippingFees :[] }));
  };

  useEffect(() => {
   setIsDisplayFee(hasOrderFee)
    if (merchantSetting.config) {
      form.setFieldsValue(merchantSetting.config);
      form.setFieldsValue({hasOrderFee: hasOrderFee});
    }
  }, [merchantSetting.config]);
  return (
    <ConfigSettingWrapper>
      <Form form={form} style={{ height: '100%' }} onFinish={onFinish}>
        <div className="group">
          <Title level={4} className="setting-config">
            <span>{intl.formatMessage(messagesGlobal.deliveryFee)}</span>
            {isDisplayFee && (
              <Button
                type="primary"
                shape="circle"
                icon={<PlusOutlined />}
                size="large"
                onClick={e => {
                  e.preventDefault();
                  (addRef as any).current.click();
                }}
              />
            )}
          </Title>
          <Space direction="vertical" size={30}>
            <Form.Item
              name="hasOrderFee"
              rules={[
                {
                  required: true,
                  message: intl.formatMessage(messagesGlobal.requiredChoose),
                },
              ]}
            >
              <Radio.Group
                name="hasOrderFee"
                onChange={e => setIsDisplayFee(e.target.value)}
              >
                <Space size={40}>
                  <Radio value={false}>
                    {intl.formatMessage(messagesGlobal.free)}
                  </Radio>
                  <Radio value={true}>
                    {intl.formatMessage(messagesGlobal.hasFeeSetting)}
                  </Radio>
                </Space>
              </Radio.Group>
            </Form.Item>
          </Space>
          {isDisplayFee && (
            <>
              <Form.List name="shippingFees">
                {(fields, { add, remove }) => (
                  <Row>
                    <div className="setting-menu">
                      <span
                        onClick={() => add()}
                        ref={addRef}
                        style={{ display: 'none' }}
                      />
                    </div>
                    {fields.map((field, index) => (
                      <Row gutter={[10, 0]} className="row-shipping">
                        <Col span={7}>
                          <Form.Item
                            name={
                              field
                                ? [field.name, 'from']
                                : ['items', '0', 'from']
                            }
                            rules={[
                              {
                                required: true,
                                message: intl.formatMessage(
                                  messagesGlobal.requiredInputNumber,
                                ),
                              },
                              {
                                validator : (rule, value, callback) =>{
                                  const currentToValue =  form.getFieldValue("shippingFees")[index].to;
                                  if(currentToValue){
                                    if(Number(value) >= Number(currentToValue)){
                                      callback( intl.formatMessage(
                                        messagesGlobal.requireInputValueLessThan,
                                      ).replace("{value}",currentToValue));
                                    }
                                  }
                                  callback();
                                },
                              }
                            ]}
                          >
                            <FloatLabel
                              style={{
                                verticalAlign: 'middle',
                                width: '100%',
                                color: '#DC3B7F',
                              }}
                              addonAfter="km"
                              label={intl.formatMessage(messages.from)}
                            />
                          </Form.Item>
                        </Col>
                        <Col span={7}>
                          <Form.Item
                            className="input-price-policy"
                            name={
                              field ? [field.name, 'to'] : ['items', '0', 'to']
                            }
                            rules={[
                              {
                                required: true,
                                message: intl.formatMessage(
                                  messagesGlobal.requiredInputNumber,
                                ),
                              },
                              {
                                validator : (rule, value, callback) =>{
                                  const currentFromValue =  form.getFieldValue("shippingFees")[index].from;
                                  if(currentFromValue){
                                    if(Number(value) <= Number(currentFromValue)){
                                      callback( intl.formatMessage(
                                        messagesGlobal.requireInputValueGreaterThan,
                                      ).replace("{value}",currentFromValue));
                                    }
                                  }
                                  callback();
                                },
                              }
                            ]}
                          >
                            <FormatInput
                              style={{
                                verticalAlign: 'middle',
                                width: '100%',
                                color: '#DC3B7F',
                              }}
                              placeholder={intl.formatMessage(messages.to)}
                              addonAfter="km"
                            />
                          </Form.Item>
                        </Col>
                        <Col span={7}>
                          <Form.Item
                            className="input-price-policy"
                            name={
                              field
                                ? [field.name, 'feeAmount']
                                : ['items', '0', 'feeAmount']
                            }
                            rules={[
                              {
                                required: true,
                                message: intl.formatMessage(
                                  messagesGlobal.requiredInputNumber,
                                ),
                              },
                            ]}
                          >
                            <FormatInput
                              style={{
                                verticalAlign: 'middle',
                                width: '100%',
                                color: '#DC3B7F',
                              }}
                              placeholder={intl.formatMessage(
                                messagesGlobal.deliveryFee,
                              )}
                              addonAfter={currencyMemoi}
                            />
                          </Form.Item>
                        </Col>
                        <Col span={3}>
                          <span onClick={() => remove(index)}>
                            <RemoveSettingCharacter />
                          </span>
                        </Col>
                      </Row>
                    ))}
                  </Row>
                )}
              </Form.List>
            </>
          )}
        </div>
        <Button
          type="primary"
          shape="round"
          size="large"
          loading={loading}
          htmlType="submit"
          className="btn-save"
        >
          {intl.formatMessage(messagesGlobal.save)}
        </Button>
      </Form>
    </ConfigSettingWrapper>
  );
}
export default injectIntl(ShippingSetting);
