import React, { useEffect, useState } from 'react';
import { useSelector, useDispatch } from 'react-redux';
import { createStructuredSelector } from 'reselect';
import moment from 'moment';

import { useInjectReducer, useInjectSaga } from 'utils/redux-injectors';

import reducer from './reducer';
import saga from './saga';
import messages from './messages';
import {
  Button,
  DatePicker,
  Form,
  Input,
  Select,
  Upload,
  notification,
  Typography,
  Row,
  Col,
} from 'antd';

import { AvatarIcon } from '../../styles/icon-components';
import { useHistory } from 'react-router-dom';
import { StaffAddWrapper } from './styles';
import { createStaff, getCommissions } from './actions';
import {
  makeSelectRestaurants,
  makeSelectRoles,
  makeSelectPhoneCountries,
} from '../App/selectors';
import makeSelectStaffAddNew from './selectors';
import { FormatInput } from 'components/FormatInput';
import { digits } from 'utils/number';
import { injectIntl } from 'react-intl';
import messagesGlobal from 'translations/messages';
import cacheUtils from '../../utils/cacheUtils';
import { getRestaurants } from '../App/actions';
import { validateRequiredSpace, fileValidation } from 'utils/validation';
import { usePhoneAddon } from '../../components/CustomHook/usePhoneAddon';
import { disabledDate, formatCurrency } from 'utils/common';
import { useCurrencyHook } from '../../components/CustomHook/useCurrencyHook';
import FloatLabel from 'components/FormatInput/FloatLabel';
const { Option } = Select;
const { Title, Paragraph } = Typography;
const globalSelector = createStructuredSelector({
  restaurants: makeSelectRestaurants(),
  roles: makeSelectRoles(),
  staffAdd: makeSelectStaffAddNew(),
  phoneCountries: makeSelectPhoneCountries(),
});

interface Props {
  intl: any;
}

function StaffAddNew(props: Props) {
  useInjectReducer({ key: 'staffAddNew', reducer: reducer });
  useInjectSaga({ key: 'staffAddNew', saga: saga });

  const { intl } = props;
  const { restaurants, roles, staffAdd, phoneCountries } = useSelector(
    globalSelector,
  );
  const { commissions } = staffAdd;
  const dispatch = useDispatch();

  const dateFormat = 'DD/MM/YYYY';
  const [imageUrl, setImageUrl] = useState<any>();
  const [form] = Form.useForm();
  const currencyMemory = useCurrencyHook();
  const prefixSelector = usePhoneAddon(phoneCountries);

  const onFinish = values => {
    values = formatCurrency(values);
    if (values.paySetting) {
      values.paySetting.commissionId =
        values.paySetting.commissionId !== 0
          ? values.paySetting.commissionId
          : null;
      values.paySetting = JSON.stringify(values.paySetting);
    }
    if (values.username) values.username = values.username.toLowerCase();
    if (values.phoneNumber) {
      values.phoneNumber = values.phoneCountryCode + values.phoneNumber;
    }
    const { files, dateOfBirth } = values;
    values.files = files?.file;
    values.dateOfBirth = dateOfBirth
      ? dateOfBirth.format(dateFormat)
      : dateOfBirth;
    dispatch(createStaff(values));
  };

  const beforeUpload = file => {
    if (!fileValidation(file.name)) {
      notification.error({
        message: intl.formatMessage(messagesGlobal.invalidFile),
      });
      return false;
    }
    const reader = new FileReader();
    reader.onloadend = e => {
      setImageUrl(reader.result!);
    };
    reader.readAsDataURL(file);
    return false;
  };

  useEffect(() => {
    const merchantId = cacheUtils.getMerchantId();
    dispatch(getRestaurants(merchantId));
    dispatch(getCommissions());
  }, []);

  return (
    <StaffAddWrapper>
      <Form form={form} onFinish={onFinish}>
        <div className="form">
          <Row>
            <Col className="staff-card" xl={8} lg={8} md={8} sm={24} xs={24}>
              <div>
                <h3 className="title">
                  {intl.formatMessage(messages.staffInfo)}
                </h3>
                <Form.Item className="no-mr">
                  <Form.Item
                    name="firstName"
                    rules={[
                      validateRequiredSpace(
                        { getFieldValue: form.getFieldValue },
                        intl.formatMessage(messagesGlobal.requireStaffFirstName),
                      ),
                    ]}
                    style={{
                      display: 'inline-block',
                      width: 'calc(50% - 5px)',
                    }}
                  >
                    <FloatLabel
                      label={`${intl.formatMessage(messages.firstName)}*`}
                      maxLength={100}
                    />
                  </Form.Item>
                  <Form.Item
                    name="lastName"
                    rules={[
                      validateRequiredSpace(
                        { getFieldValue: form.getFieldValue },
                        intl.formatMessage(messagesGlobal.requireStaffName),
                      ),
                    ]}
                    style={{
                      display: 'inline-block',
                      width: 'calc(50% - 5px)',
                      margin: '0 0 0 10px',
                    }}
                  >
                    <FloatLabel
                      label={`${intl.formatMessage(messages.lastName)}*`}
                      maxLength={100}
                    />
                  </Form.Item>
                </Form.Item>
                <Row className="row__gender">
                  <Col span={12}>
                    <Form.Item name="gender">
                      <FloatLabel
                          className="padding-item-select"
                          label={intl.formatMessage(messages.gender)}
                          items={[{id: "male", name: intl.formatMessage(messages.male)},
                          {id: "female", name: intl.formatMessage(messages.female)},]}
                          typeInput="selectMutil"
                        />
                    </Form.Item>
                  </Col>
                  <Col span={12}>
                    <Form.Item
                      className="row__role mutil-select"
                      name="roleIds"
                      rules={[
                        {
                          required: true,
                          message: intl.formatMessage(messages.requireRole),
                        },
                      ]}
                    >
                      <FloatLabel
                        className="padding-mutil-select"
                        size="middle"
                        mode="multiple"
                        label={intl.formatMessage(messages.roleAsterisk)}
                        showArrow
                        typeInput="selectMutil"
                        items={roles}
                      />
                    </Form.Item>
                  </Col>
                </Row>
                <Row className="row__phone">
                  <Col span={12}>
                    <Form.Item
                      name="phoneNumber"
                      rules={[
                        {
                          message: intl.formatMessage(
                            messages.invalidPhoneNumber,
                          ),
                          pattern: /^[\d]+$/,
                        },
                        () => ({
                          validator(rule, value) {
                            if (value.length > 10) {
                              return Promise.reject(
                                intl.formatMessage(messagesGlobal.invalidPhoneNumber),
                              );
                            } else {
                              return Promise.resolve();
                            }
                          },
                        }),
                      ]}
                    >
                      <FloatLabel
                        addonBefore={prefixSelector}
                        label={intl.formatMessage(messages.phoneNumber)}
                        maxLength={100}
                      />
                    </Form.Item>
                  </Col>
                  <Col span={12}>
                    <Form.Item className="row__dob" name="dateOfBirth">
                      <FloatLabel
                        className="row__dob__picker"
                        label={intl.formatMessage(messages.dob)}
                        format={dateFormat}
                        disabledDate={disabledDate}
                        typeInput="datepicker"
                      />
                    </Form.Item>
                  </Col>
                </Row>
                <Form.Item name="address">
                  <FloatLabel
                    label={intl.formatMessage(messages.address)}
                    maxLength={150}
                  />
                </Form.Item>
                <Form.Item
                  name="restaurantId"
                  rules={[
                    {
                      required: true,
                      message: intl.formatMessage(messages.requireStore),
                    },
                  ]}
                >
                  <FloatLabel 
                  label={`${intl.formatMessage(messages.store)}*`}
                  items={restaurants}
                  typeInput="selectMutil"
                  />
                </Form.Item>
                <Form.Item className="no-mr">
                  <Form.Item
                    name="email"
                    rules={[
                      {
                        message: intl.formatMessage(messages.invalidEmail),
                        type: 'email',
                      },
                    ]}
                    style={{
                      display: 'inline-block',
                      width: 'calc(50% - 5px)',
                    }}
                  >
                    <FloatLabel label="Email" maxLength={100} />
                  </Form.Item>
                  <Form.Item
                    name="mst"
                    style={{
                      display: 'inline-block',
                      width: 'calc(50% - 5px)',
                      margin: '0 0 0 10px',
                    }}
                  >
                    <FloatLabel
                      label={intl.formatMessage(messagesGlobal.mst)}
                      maxLength={100}
                    />
                  </Form.Item>
                </Form.Item>
              </div>
            </Col>
            <Col
              className="staff-card payroll"
              xl={10}
              lg={10}
              md={10}
              sm={24}
              xs={24}
            >
              <div>
                <Title level={3} className="title">
                  {intl.formatMessage(messagesGlobal.payroll)}
                </Title>
                <Row className="payroll-content">
                  <Col span={12}>
                    <div>
                      <Form.Item name={['paySetting', 'type']}>
                        <FloatLabel
                          size="large"
                          label={intl.formatMessage(
                            messagesGlobal.payrollType,
                          )}
                          items={[
                            {id: 1, name: intl.formatMessage(messagesGlobal.hour)},
                            {id: 2, name: intl.formatMessage(messagesGlobal.week)},
                            {id: 3, name: intl.formatMessage(messagesGlobal.month)},
                            {id: 4, name: intl.formatMessage(messagesGlobal.year)}
                          ]}
                          typeInput="selectMutil"
                        />
                      </Form.Item>
                      <Form.Item
                        name={['paySetting', 'payAmount']}
                        rules={[
                          {
                            pattern: new RegExp('^[0-9,]*$'),
                            message: intl.formatMessage(
                              messages.requireNumberInput,
                            ),
                          },
                        ]}
                      >
                        <FormatInput
                          formatter={digits}
                          addonAfter={currencyMemory}
                          placeholder={`${intl.formatMessage(
                            messagesGlobal.basicSalary,
                          )}`}
                        />
                      </Form.Item>
                      <Form.Item
                        name={['paySetting', 'extraAmount']}
                        rules={[
                          {
                            pattern: new RegExp('^[0-9,.]*$'),
                            message: intl.formatMessage(
                              messages.requireNumberInput,
                            ),
                          },
                        ]}
                      >
                        <FormatInput
                          formatter={digits}
                          addonAfter={currencyMemory}
                          placeholder={`${intl.formatMessage(
                            messagesGlobal.allowance,
                          )}`}
                        />
                      </Form.Item>
                    </div>
                  </Col>
                  <Col span={12}>
                    <div>
                      <Form.Item
                        name={['paySetting', 'commissionId']}
                        rules={[
                          {
                            pattern: new RegExp('^[0-9]*$'),
                            message: intl.formatMessage(
                              messagesGlobal.requiredInputNumber,
                            ),
                          },
                        ]}
                        className="padding-item-select"
                      >
                        <FloatLabel
                          size="large"
                          label={intl.formatMessage(
                            messagesGlobal.commission,
                          )}
                          items={[...commissions]}
                          typeInput="selectMutil"
                        />
                      </Form.Item>
                      <Form.Item 
                        name={['paySetting', 'note']}
                      >
                        <FloatLabel
                          label={intl.formatMessage(messagesGlobal.note)}
                          className="note-input"
                          typeInput="TextArea"
                        />
                      </Form.Item>
                    </div>
                  </Col>
                </Row>
              </div>
            </Col>
            <Col
              className="staff-card account"
              xl={6}
              lg={6}
              md={6}
              sm={24}
              xs={24}
            >
              <div>
                <h3 className="title">
                  {intl.formatMessage(messages.account)}
                </h3>
                <Form.Item name="files">
                  <Upload
                    name="avatar"
                    listType="picture-card"
                    className="avatar__upload"
                    showUploadList={false}
                    beforeUpload={beforeUpload}
                    accept="image/*"
                  >
                    {imageUrl ? (
                      <img
                        className="avatar__upload__thumb"
                        src={imageUrl}
                        alt="avatar"
                      />
                    ) : (
                      <div className="avatar__upload__thumb">
                        <AvatarIcon />
                        <div className="ant-upload-text avatar__upload__thumb__text">
                          {intl.formatMessage(messages.avatar)}
                        </div>
                        <span className="img-hint">
                          {intl.formatMessage(
                            messagesGlobal.encourageCondition,
                            {
                              height: 150,
                              width: 150,
                            },
                          )}
                        </span>
                      </div>
                    )}
                  </Upload>
                </Form.Item>
                <Form.Item
                  name="username"
                  rules={[
                    {
                      required: true,
                      message: intl.formatMessage(messages.requireUsername),
                    },
                  ]}
                >
                  <FloatLabel
                    label={`${intl.formatMessage(messages.username)}*`}
                    maxLength={100}
                  />
                </Form.Item>
                <Form.Item
                  name="password"
                  rules={[
                    {
                      required: true,
                      message: intl.formatMessage(messages.requirePassword),
                    },
                  ]}
                >
                  <FloatLabel
                    label={`${intl.formatMessage(messages.password)}*`}
                    maxLength={100}
                    autoComplete="new-password"
                    typeInput="password"
                  />
                </Form.Item>
                <Form.Item
                  name="verifyPassword"
                  dependencies={['password']}
                  rules={[
                    {
                      required: true,
                      message: intl.formatMessage(
                        messages.requireConfirmPassword,
                      ),
                    },
                    ({ getFieldValue }) => ({
                      validator(rule, value) {
                        if (!value || getFieldValue('password') === value) {
                          return Promise.resolve();
                        }
                        return Promise.reject(
                          intl.formatMessage(messages.incorrectConfirmPassword),
                        );
                      },
                    }),
                  ]}
                >
                  <FloatLabel
                    label={`${intl.formatMessage(messages.retypingPassword)}*`}
                    maxLength={100}
                    typeInput="password"
                  />
                </Form.Item>
              </div>
            </Col>
          </Row>
        </div>
        <Button
          className="submit"
          type="primary"
          htmlType="submit"
          loading={staffAdd.isFetching}
        >
          {intl.formatMessage(messages.save)}
        </Button>
      </Form>
    </StaffAddWrapper>
  );
}

export default injectIntl(StaffAddNew);
