import React, { useState } from 'react';
import styled from 'styled-components';
import _ from 'lodash';
import { useHistory } from 'react-router-dom';
import { ExclamationCircleOutlined } from '@ant-design/icons';
import { Button, Space, Modal } from 'antd';
import Avatar from 'react-avatar';
import { PUBLIC_API } from 'containers/App/urls';
import { RoutePaths } from 'containers/App/routes';
import { StaffModel } from '../types';
import { Card as StyledCard } from 'styles/custom-styled-components';
import { EditIcon, RemoveIcon } from 'styles/icon-components';
import { initials } from 'utils/string';
import NumberFormat from 'react-number-format';
import { FormattedMessage, injectIntl } from 'react-intl';
import DEFAULT_AVATAR from '../../../images/defaultAvatar.png';
import messages from 'translations/messages';
import { extractPhoneNumber } from 'utils/common';

const HovereddCard = styled.div`
  position: relative;
  top: calc(-100px / 2 - 10px);
  text-align: center;
`;

interface Props {
  staff: StaffModel;
  onClickRemove: any;
  roles: any;
  intl?: any;
}

const HoveredCard = injectIntl((props: Props) => {
  const { staff, onClickRemove, intl } = props;
  const history = useHistory();
  const handleOnClickEdit = () =>
    history.push(`${RoutePaths.staffDetail}/${staff.id}`);
  const handleClickRemove = () => {
    if (onClickRemove) {
      onClickRemove(staff.id);
    }
  };
  const confirm = () => {
    Modal.confirm({
      title: intl.formatMessage(messages.deleteConfirmMess),
      icon: <ExclamationCircleOutlined />,
      content: intl.formatMessage(messages.deleteConfirmContent, {
        name: staff.firstName,
      }),
      okText: intl.formatMessage(messages.deleteBtn),
      cancelText: intl.formatMessage(messages.cancelBtn),
      maskClosable: true,
      onOk: handleClickRemove,
    });
  };
  return (
    <HovereddCard>
      <Space>
        <Button
          shape="circle"
          style={{ width: 50, height: 50 }}
          onClick={handleOnClickEdit}
        >
          <EditIcon />
        </Button>
        <Button
          shape="circle"
          style={{ width: 50, height: 50 }}
          onClick={confirm}
        >
          <RemoveIcon />
        </Button>
      </Space>
    </HovereddCard>
  );
});

function CardDetail(props) {
  const { staff, roles } = props;
  const [code, number] = extractPhoneNumber(
    staff.phone,
    staff.phoneCountryCode,
  );
  const {
    avatar,
    phone,
    phoneCountryCode,
    firstName,
    lastName,
    roleIds = [],
  } = staff;
  const rolesNames = [...roleIds]
    .map(id => {
      const role = _.find(roles, role => role.id === id);
      return role ? role.name : null;
    })
    .filter(role => role !== null);
  return (
    <>
      <div className="restaurant-card__header">
        {avatar ? (
          <div style={{ position: 'relative' }}>
            <img className="image" src={`${PUBLIC_API}/${avatar}`} />
            <span
              className="restaurant-card__dot2"
              style={{
                backgroundColor:
                  staff.status.statusId === 1 ? '#9cd339' : '#9C9696',
              }}
            ></span>
          </div>
        ) : firstName || lastName ? (
          <div style={{ position: 'relative' }}>
            <Avatar
              className="image"
              name={`${firstName ? firstName : ''}${
                lastName ? ' ' + lastName : ''
              }`}
              size="70"
              initials={initials}
            />
            <span
              className="restaurant-card__dot2"
              style={{
                backgroundColor:
                  staff.status.statusId === 1 ? '#9cd339' : '#9C9696',
              }}
            ></span>
          </div>
        ) : (
          <img className="image" src={DEFAULT_AVATAR} />
        )}
        <div
          style={{
            display: 'flex',
            flexDirection: 'column',
            justifyContent: 'space-between',
          }}
        >
          <span className="name" style={{ alignSelf: 'flex-start' }}>
            {firstName ? firstName : ''}
            {lastName ? ' ' + lastName : ''}
          </span>
          <span className="role">{rolesNames.join(', ')}</span>
          <span className="phone">
            {phone ? (
              <NumberFormat
                value={number}
                displayType={'text'}
                format={
                  phoneCountryCode === '+84' ? '## ### ####' : '### ### ####'
                }
                renderText={value => `${phoneCountryCode} ${value}`}
              />
            ) : (
              ''
            )}
          </span>
        </div>
      </div>
    </>
  );
}

export default function Card(props: Props) {
  const { staff, roles, onClickRemove } = props;
  const [isHovered, setHovered] = useState(false);

  return (
    <StyledCard
      isHovered={isHovered}
      onMouseOver={() => setHovered(true)}
      onMouseLeave={() => setHovered(false)}
      height="100px"
      className="touch-card"
    >
      <CardDetail staff={staff} roles={roles} />
      {isHovered && <HoveredCard staff={staff} onClickRemove={onClickRemove} />}
    </StyledCard>
  );
}
