import React from 'react';
import { Table, Typography, Button, Pagination } from 'antd';
import styled from 'styled-components';

import moment from 'moment';
import NumberFormat from 'react-number-format';
import { injectIntl } from 'react-intl';
import { useDispatch, useSelector } from 'react-redux';
import { createStructuredSelector } from 'reselect';
import { makeStorages } from '../selectors';
import {
  getInvent,
  getHistories,
  showAddModal,
  getInvents,
  setSearch,
  showListModal,
} from '../actions';

import messages from 'translations/messages';
import { PlusCharacter } from 'styles/icon-components';
import IMG_INVENTORY_EDIT from 'images/inventory-edit.svg';

const ListWrapper = styled.div`
  box-shadow: 0 1px 3px 0 rgba(0, 0, 0, 0.1);
  background-color: #ffffff;
  font-family: Muli;
  margin: 1px 0;
  padding: 20px 20px 0;
  .ant-typography-ellipsis-single-line {
    font-size: 15px;
    font-weight: 600;
    overflow: unset;
    white-space: unset;
    @media screen and (max-width: 576px) {
      font-size: 12px;
    }
    @media screen and (max-width: 420px) {
      font-size: 10px;
    }
  }
  .ant-table-cell {
    @media screen and (max-width: 576px) {
      font-size: 10px;
    }
    @media screen and (max-width: 420px) {
      font-size: 8px;
    }
  }
  @media screen and (min-width: 1200px) {
    padding: 0 0;
  }
  .working-hours {
    &_header {
      h3 {
        font-size: 18px;
        font-weight: bold;
        color: #333333;
      }
      @media screen and (min-width: 1200px) {
        margin: 20px 20px 30px;
      }
    }
    &_main {
      .txt_green {
        color: #35a211;
        text-align: center;
      }
      .txt_red {
        color: #dc5454;
        text-align: center;
      }
      .wrapper-group-btn {
        display: flex;
        justify-content: space-around;
        flex-wrap: nowrap;
      }
      .wrapper-btn {
        width: 44px;
        height: 44px;
        .ant-btn-circle.ant-btn-lg {
          min-width: 100%;
        }
        .detail-btn{
          width: 100%;
          height: 100%;
          border-radius: 50%;
        }
      }
      .action-row,
      .remain-row {
        text-align: center;
      }
      .ant-table-thead > tr > th,
      .ant-table-tbody > tr > td,
      .ant-table tfoot > tr > th,
      .ant-table tfoot > tr > td {
        padding: 15px 5px;
      }
      @media screen and (max-width: 1200px) {
        .wrapper-btn {
          width: 35px;
          height: 35px;
          min-width: 0px;
          svg {
            font-size: 8px;
            height: 25px;
          }
        }
      }

      @media screen and (max-width: 765px) {
        .wrapper-btn {
          width: 30px;
          height: 30px;
          min-width: 0px;
          svg {
            font-size: 5px;
            height: 20px;
          }
        }
      }
    }
  }
`;

const stateSelector = createStructuredSelector({
  inventories: makeStorages(),
});

function showTotal(total, range) {
  return `${range.join('-')} of ${total}`;
}

export const InventoryList = injectIntl(props => {
  const { intl, search } = props;
  const { inventories } = useSelector(stateSelector);

  const dispatch = useDispatch();
  const handleOnClickAdd = key => {
    dispatch(getInvent(key.productId, key.productSizeId));
    dispatch(showAddModal());
  };

  const handleClick = key => {
    dispatch(getInvent(key.productId, key.productSizeId));
    dispatch(
      getHistories({
        productId: key.productId,
        sizeId: key.productSizeId,
        limit: 50,
        page: 1,
        restaurantId: search.restaurantId
      }),
    );
    dispatch(showListModal());
  };

  const handleClickPageNumber = (page, pageSize) => {
    const newSearch = { ...search, page: page };
    dispatch(getInvents(newSearch));
    dispatch(setSearch(newSearch));
  };

  const handleClickChangeSize = (current, size) => {
    const newSearch = { ...search, limit: size, page: 1 };
    dispatch(getInvents(newSearch));
    dispatch(setSearch(newSearch));
  };

  const config = {
    pagination: {
      pageSizeOptions: ['10', '30', '40'],
      showSizeChanger: true,
      total: inventories.count,
      size: 'small' as any,
      showTotal: (total, range) => showTotal(total, range),
      onChange: (page, pageSize) => handleClickPageNumber(page, pageSize),
      onShowSizeChange: (current, size) => handleClickChangeSize(current, size),
    },
  };
  const columns = [
    {
      title: (
        <Typography.Text ellipsis>
          {intl.formatMessage(messages.skuCd)}
        </Typography.Text>
      ),
      dataIndex: 'productSku',
      key: 'productSku',
      width: '3.8%',
      render: (text, row, index) => {
        return {
          children: <span>{text}</span>,
          props: {
            colSpan: 1,
          },
        };
      },
    },
    {
      title: (
        <Typography.Text ellipsis>
          {intl.formatMessage(messages.productName)}
        </Typography.Text>
      ),
      dataIndex: 'productName',
      key: 'productName',
      width: '26.4%',
      render: (text, row, index) => {
        return {
          children: <span>{text}</span>,
          props: {
            colSpan: 1,
          },
        };
      },
    },
    {
      title: (
        <Typography.Text ellipsis>
          {intl.formatMessage(messages.category)}
        </Typography.Text>
      ),
      dataIndex: 'categoryName',
      key: 'categoryName',
      responsive: ['xxl', 'xl', 'lg', 'md', 'sm'] as any,
      width: '20%',
      render: (text, row, index) => {
        return {
          children: <span>{text}</span>,
          props: {
            colSpan: 1,
          },
        };
      },
    },
    {
      title: (
        <Typography.Text ellipsis>
          {intl.formatMessage(messages.size)}
        </Typography.Text>
      ),
      dataIndex: 'size',
      key: 'size',
      width: '8.5%',
      render: (text, row, index) => {
        return {
          children: <span>{text}</span>,
          props: {
            colSpan: 1,
          },
        };
      },
    },
    {
      title: (
        <Typography.Text ellipsis>
          {intl.formatMessage(messages.priceImport)}
        </Typography.Text>
      ),
      dataIndex: 'importPrice',
      key: 'importPrice',
      width: '8.5%',
      render: (text, row, index) => {
        return {
          children: (
            <span>
              <NumberFormat
                value={text}
                displayType={'text'}
                thousandSeparator={true}
                renderText={value => value}
              />
            </span>
          ),
        };
      },
    },
    {
      title: (
        <Typography.Text ellipsis>
          {intl.formatMessage(messages.quantityRemain)}
        </Typography.Text>
      ),
      dataIndex: 'quantity',
      key: 'quantity',
      width: '10.8%',
      className: 'remain-row',
      render: (text, row, index) => {
        return {
          children: <span>{text}</span>,
          props: {
            colSpan: 1,
          },
        };
      },
    },
    {
      title: (
        <Typography.Text ellipsis>
          {intl.formatMessage(messages.updateDate)}
        </Typography.Text>
      ),
      dataIndex: 'updatedTime',
      responsive: ['xxl', 'xl', 'lg', 'md'] as any,
      key: 'updatedTime',
      width: '11%',
      render: text => (
        <span>{text ? moment(text).format('DD/MM/YYYY') : ''}</span>
      ),
    },
    {
      title: <></>,
      dataIndex: 'action',
      key: 'action',
      width: '11%',
      className: 'action-row',
      render: (text, row, index) => (
        <span className="wrapper-group-btn">
          <div className="wrapper-btn">
            <img
              src={IMG_INVENTORY_EDIT}
              alt=""
              onClick={e => handleClick(row)}
              className="detail-btn list-pro"
            />
          </div>
          <div  className="wrapper-btn">
            <Button
              className="detail-btn"
              onClick={e => handleOnClickAdd(row)}
              size="large"
              type="primary"
              shape="circle"
            >
              <PlusCharacter />
            </Button>
          </div>
        </span>
      ),
    },
  ];
  return (
    <ListWrapper>
      <div className="working-hours_main">
        <Table
          dataSource={inventories.list}
          columns={columns}
          rowKey={record => record.key}
          pagination={config.pagination}
        />
      </div>
    </ListWrapper>
  );
});
