import React, { useState, useEffect } from 'react';
import styled from 'styled-components';
import _ from 'lodash';
import { Modal, Form, Divider, Typography, Table, Spin } from 'antd';
import { StorageModel } from '../types';
import messages from 'translations/messages';
import { useDispatch, useSelector } from 'react-redux';
import { createStructuredSelector } from 'reselect';
import { injectIntl } from 'react-intl';
import { PUBLIC_API } from 'containers/App/urls';
import moment from 'moment';
import { closeListModal, getHistories } from "../actions";
import { makeStorage, makeIsFetching, makeHistories, makeShowModalInventory } from '../selectors';
import { MenuItemIcon } from 'styles/icon-components';
interface Props {
  inventory: StorageModel;
  productInvens: any;
  onClickRemove: any;
  roles: any;
  intl?: any;
  visibleList: boolean;
  setVisibleList: any;
  showModal: any;
  search: any;
}

const stateSelector = createStructuredSelector({
  inventory: makeStorage(),
  isFetching: makeIsFetching(),
  histories: makeHistories(),
  showModalInventory: makeShowModalInventory(),
});

const ModalWrapper = styled.div`
  .container {
    display: flex;
    flex-direction: column;
    flex-wrap: nowrap;
    justify-content: space-between;
    align-items: baseline;
    align-content: stretch;
    margin: auto;
    .header {
      width: 100%;
      &__title {
        font-size: 15px;
        font-weight: 500;
        margin-bottom: 20px;
        text-align: center;
      }
      &__desc {
        display: flex;
        flex-direction: row;
        justify-content: space-between;
        align-items: start;
        .quantity {
          width: 53px;
          height: 44px;
          display: flex;
          align-items: center;
          border-radius: 22px;
          background-color: #DC3B7F;
          span {
            margin: auto;
            color: white;
          }
        }
      }
      &__infor {
        padding-top: 15px;
        &__name {
          font-size: 13px;
          font-weight: 600;
          margin-bottom: 20px;
        }
        &__sku {
          color: #9c9696;
          line-height: 13px;
        }
      }
      &__img {
        height: 70px;
        width: 120px;
        text-align: center;
        margin-right: 20px;
        background-color: #ececec;
        img {
          height: 70px;
          width: 120px;
          object-fit: cover;
        }
      }
    }
    .title {
      color: #333333;
      font-size: 16px;
      font-weight: 700;
      line-height: 19px;
      text-align: center;
      padding-bottom: 20px;
    }
    .btn-wrapper {
      width: 100%;
      text-align: center;
      .add-button {
        width: 70%;
        height: 44px;
        margin-top: 20px;
        font-weight: 600;
      }
    }
  }

  .element-input-wrapper {
    width: 100%;
    input.element-input,
    div.element-input {
      width: 90%;
      height: 44px;
      border-radius: 22px;
      border: solid 1px #e6e6e6;
    }
  }

  .form-field {
    width: 100%;
    &__row {
      display: flex;
      flex-direction: row;
    }
    &__col {
      display: flex;
      flex-direction: row;
      flex: 0 1 90%;
      .ant-row {
        &:nth-child(1) {
          flex: 0 1 25%;
          margin-right: 2%;
        }
        &:nth-child(2) {
          flex: 0 1 70%;
        }
      }
    }
    &__plus {
      flex: 0 1 10%;
    }
  }
  .table {
    margin: auto;
    width: 100%;
    .ant-typography-ellipsis-single-line {
      font-size: 15px;
      font-weight: 600;
      overflow: unset;
      white-space: normal;
    }
    @media screen and (max-width: 576px) {
      .ant-table-thead > tr > th, .ant-table-tbody > tr > td, .ant-table tfoot > tr > th, .ant-table tfoot > tr > td {
        padding: 10px 10px;
      }
    }
    margin-top: 25px;
    .txt_green {
      color: #35a211;
      text-align: center;
    }
    .txt_red {
      color: #dc5454;
      text-align: center;
    }
  }
`;

function showTotal(total, range) {
  return `${range.join('-')} of ${total}`;
}

export default injectIntl((props: Props) => {
  const { inventory, isFetching, histories, showModalInventory } = useSelector(stateSelector);
  const { intl, search } = props;
  const dispatch = useDispatch();

  const handleCancel = () => {
    dispatch(closeListModal());
  };

  const handleClickPageNumber = (page, pageSize) => {
    dispatch(getHistories({productId: inventory.productId, sizeId: inventory.productSizeId, limit: pageSize, page: page, restaurantId: search.restaurantId}));
  }

  const config = {
    pagination: {
      pageSize: 10,
      total: histories.count,
      size: 'small' as any,
      showTotal: (total, range) => showTotal(total, range),
      onChange: (page, pageSize) => handleClickPageNumber(page, pageSize),
    },
  };
  const columns = [
    {
      title: (
        <Typography.Text ellipsis>
          {intl.formatMessage(messages.updateDate)}
        </Typography.Text>
      ),
      dataIndex: 'createdTime',
      responsive: ['xxl', 'xl', 'lg', 'md', 'sm'] as any,
      key: 'createdTime',
      render: text => (
        <span>{text ? moment(text).format('DD/MM/YYYY') : 'N/A'}</span>
      ),
    },
    {
      title: (
        <Typography.Text ellipsis>
          {intl.formatMessage(messages.titleItemTableQuantity)}
        </Typography.Text>
      ),
      dataIndex: 'quantity',
      key: 'quantity',
      render: text => (
      <span style={{color: '#DC3B7F'}}>{text > 0 ? `+${text}` : `${text}`}</span>
      ),
    },
    {
      title: (
        <Typography.Text ellipsis>
          {intl.formatMessage(messages.storageName)}
        </Typography.Text>
      ),
      dataIndex: 'restaurantName',
      key: 'restaurantName',
    },
    {
      title: (
        <Typography.Text ellipsis>
          {intl.formatMessage(messages.storageCd)}
        </Typography.Text>
      ),
      dataIndex: 'code',
      key: 'code',
    },
    {
      title: (
        <Typography.Text ellipsis>
          {intl.formatMessage(messages.quantityRemain)}
        </Typography.Text>
      ),
      dataIndex: 'quantityInStock',
      key: 'quantityInStock',
    },
  ];

  return (
    <Spin spinning={isFetching}>
     (
      <Modal
        visible={showModalInventory}
        onCancel={handleCancel}
        okText="Thêm"
        centered
        footer={null}
        width={750}
      >
        <ModalWrapper>
        <div className="container">
            <div className="header">
              <div className="header__title title-global">{intl.formatMessage(messages.detail)}</div>
              <div className="header__desc">
                <div className="header__desc">
                  <div className="header__img" style={{paddingTop: inventory.productPhoto ? '0px' : '10px'}}>
                  {inventory.productPhoto ? <img src={`${PUBLIC_API}/${inventory.productPhoto}`} /> : <MenuItemIcon />}
                  </div>
                  <div className="header__infor">
                    <div className="header__infor__name">
                      {inventory ? `${inventory.productName ? inventory.productName : 'N/A'} - ${inventory.size ? inventory.size : 'N/A'}` : `N/A - N/A`}
                    </div>
                    <div className="header__infor__sku">{inventory ? inventory.productSku : 'N/A'}</div>
                  </div>
                </div>
                <div className="quantity"><span>{inventory.quantity ? inventory.quantity : '0'}</span></div>
              </div>
            </div>
            <Divider />
            <div className="table">
              <Table
                dataSource={histories.list}
                columns={columns}
                rowKey={record => record.key}
                pagination={config.pagination}
              />
            </div>
          </div>
        </ModalWrapper>
      </Modal>
    )
    </Spin>
  );
});
